/**
 * 
 * scaffolding Domains
 * 
 * Dieses Skript durchsucht eine Datenbank und legt fuer jede
 * zu findende Tabelle eine Domaindatei an, wenn dies nicht schon geschen
 * ist.
 * 
 * Die Konfiguration erfolgt durch Konstanten.
 * 
 * 
 * Autor:   W.Nagy
 * Datum:   26.02.2008
 */


import groovy.sql.Sql


// CONFIGURATION-SECION
//

final static String   DRIVERCLASSNAME  = 'com.mysql.jdbc.Driver'
final static String   USERNAME         = 'root'
final static String   PASSWORD         = 'janitor'
final static String   URL              = 'jdbc:mysql://localhost:3306/sonja'
  
final static String   DEST_DIR         = 'c:/temp/export'

def connection = null

/**
 * 
 * herstellen der Datenbankverbingung
 * 
 * @param   'url'          Datenbankurl
 * @param   'username'     Benutzername
 * @param   'password'     Password
 * @param   'driverclass'  Datenbanksteuerklasse
 * 
 * 
 */
Object connect (Map param) {
   //
   // Stellt die Datenbankverbindung her
   // Das Datenbankhandle wird als Klassenvariable abgelegt.
   //


   try {
      connection = Sql.newInstance(param.url,param.username,param.password,param.driverClassName)
      }
   catch (Exception e){
      throw new Exception("Grauenvoller Datenbank Verbindungsfehler: "+e.message)
      return null
      }
   return connection
   }

/**
 * Umwandeln SQL Datentypen in Groovy Datentypen
 */
String getGType(String datatype) {

   switch(datatype) {
      case 'VARCHAR' :
         return 'String'
         break
      case 'TEXT' :
         return 'String'
         break
      case 'CHAR' : 
         return 'String'
         break
      case 'INT' :
         return 'Integer'
         break
      case 'LONG' :
         return 'Long'
         break
      case 'TIMESTAMP' :
         return 'Date'
         break
      case 'DATE' :
         return 'Date'
         break
      case 'DOUBLE' :
         return 'Double'
         break
      default :
         return "--> ${datatype}"
      }
   }


/**
 * Erzeuge eine Liste von Tabellenname
 * 
 * Unterschiedliche Datenbaksysteme stellen
 * verschieden Methoden zu Verfgung um 
 * eine Liste von Tabellen zu erstelle. Hier kann
 * auch eine manuell erstellt Liste erzeugt werden.
 * 
 */
List getTablenNames() {
   List tables = []
   dbh.eachRow('show tables'){it->
      tables.add(it[0].toString())
      }
   return tables
   }

/**
 * Tabellendaten
 */
void oneTable(String tablename) {
   //            columnList += "${meta.getColumnLabel(i+1)} = ${meta.getColumnTypeName(i+1)} ${getGType(meta.getColumnTypeName(i+1))}"

   String filename = "${tablename}Domain.groovy"
   File oFile = new File("c:/temp/export/domains/${filename}")
   List b = []
   String sql = "select * from ${tablename}"

   b.add('/**')
   b.add(" * Domain fuer Tabelle ${tablename}")
   b.add(' *')
   b.add(' *')
   b.add(' *')
   b.add(' */')
   b.add('package MVC.domain')
   b.add('')
   b.add("class ${tablename}Domain { ")

   List columnList = []
   dbh.query(sql){ rs->
      def meta = rs.metaData
      if (meta.columnCount <= 0) return

      /** Erzeugen Properties */ 
      for (i in 0..<meta.columnCount) {
         b.add("   ${getGType(meta.getColumnTypeName(i+1)).padRight(16)}\t\t${meta.getColumnLabel(i+1)}")
         }

      b.add('')
      b.add('')
      b.add('')

      /** Erzeugen Meta */ 
      for (i in 0..<meta.columnCount) {
         b.add("   ${meta.getColumnLabel(i+1).padRight(16)} : [dbfield:'${meta.getColumnLabel(i+1)}', type:'${getGType(meta.getColumnTypeName(i+1))}']")
         }
      }
   oFile.write(b.join("\n"))
   }


// DBH erzeugen
//
dbh = connect(
   driverClassName:DRIVERCLASSNAME,
   username       :USERNAME,
   password       :PASSWORD,
   url            :URL
   )

List tables = getTablenNames()

/** fuer alle Tabellen */
tables.each{table->
   oneTable(table)
   }

