// ////////// An und Abmeldung ////////////////////////////
//
// Dieses Modul zeigt eine Moeglichkeit die Authentifizierung
// der Benutzer durchzuführen.
// 
// Die Ueberpruefung wird durch <root>/mframe/work.groovy gesteuert.
// 
// Die Methode check prueft, ob die Sessionveriable "user" vorhanden ist.
// Ist dies der Fall, so wird angenommen, dass der Benutzer angemeldet ist.
// Wenn keine Variable "user" vorhanden ist, so wird der Viewer 'login.gsp' 
// aufgerufen.
// 
// 
// Url Parameter:
// @parameter: username
// @parameter: password
//
//
// -- wann --+-wer-+-- was ----------------------------------------------
// 12.10.2007|  ny | estellt
// 28.07.2008|  ny | Verallgemeinert, Dokumentation
//           |     |
//

package MVC.authen

import groovy.sql.Sql
import java.util.logging.Logger
import MVC.domain.RechtDomain

class CheckUser {

   String tablename
   def db

   Logger log = Logger.getLogger("")

  /**
   * <pre>
   * Ueberprüfen Benutzerkennung
   * 
   * Ablauf:
   * -------
   * Ist Benutzer OK dann Session Variable user
   * mit Benutzername gesetzt.
   * Hier werden auch die fuer die Appliaktion notwendigen
   * Benutzerinformationen  z.B.: Rolle etc. in der Session
   * abgelegt.
   * 
   *               * aufruf check() durch work.groovy
   *               |
   *               |   wenn "user" vorhanden
   *              < >--------------------------> (*)
   *               |
   *               | "user" ist nicht vorhanden
   *               V
   *         ( Redierkt auf login.gsp )<--.
   *               |                      |
   *               |    loginfehler       |
   *              < > --------------------' 
   *               |
   *               |
   *         ( Setzen "user" sessionvariable )
   *               |
   *              (*)
   * 
   *
   * Benutzerdaten:
   * --------------
   * Die Benutzerdaten könne auf Verschiedene Art und weise gehalten werden.
   * 
   * Standarsmaessig wird angenommen, dass die Benutzerdaten in einer Datenbank-
   * Tabelle namens "USER" in der Applikationsdatenbank liegen.
   *
   * Der Routine wird ein Datenbankhandle von der Applikation übergeben.
   * Die kann Benutz werden, wenn die Benutzertabelle in der Appliaktionsdatenbank
   * ist.
   * 
   * Hinweis:
   * Sollte die Benutzerkennung in eine anderen Datenbank liegen Ensprechenden
   * Codeteil auskommentieren.
   * 
   * </pre>
   */
   def check = {param ->

     /**
      * Wurde Datenbankverbindung
      * übergeben, wird diese verwendet
      */
      if (param.db != null) this.db = param.db
      
      
      /**
       *  Wenn Benutzerkennung in einer
       *  Eigenen Datenbank liegt kann diese
       *  hier ueberschrieben werden.
       *
       *      try {
       *         db.dbh = Sql.newInstance(
       *            'jdbc:hsqldb:file:/home/user/media/MMC-SD/partition1/jetty/webapps/mframe/WEB-INF/database/employee',
       *            'sa',
       *            '',
       *            'org.hsqldb.jdbcDriver'
       *            )
       *         }
       *      catch (Exception e){
       *         throw new Exception("Grauenvoller Datenbank Verbindungsfehler: "+e.message)
       *         return null
       *         }
       */

      def request = param.get('request',null)
      def session = request.session

      /** Benutzerkennung aus Session holen */
      String user = session.getAttribute('user')

     /**
      * Wurde benutzer noch nicht definiert,  
      * wird die Autentifizierung durchgefueht.
      */
      if (user == null) { // noch nicht angemeldet

        /**
         * Benutzerkennung und Passwort aus CGI holen
         */
         user = request.getParameter('username')
         String password = request.getParameter('password')

        /**
         * Abfrage zusammenbauen
         */
         String sql = "select * from USER where username = '${user}' and password='${password}'"

        /**
         * Benutzerkennung
         */
         def usrRec = null

         /** aus DB holen */
         try {
            usrRec = db.dbh.firstRow(sql)
            }
         catch (Exception e){
            throw new Exception("Grauenhafter Fehler beim einloggen "+e.message)
            return false
            }

        /** 
         * Wenn Datensatz gefunden, 
         * so werden Benutzerkennungen in 
         * Sessionvariable geschrieben
         */
         if (usrRec != null) {
            request.session.setAttribute('user',usrRec.USERNAME+" (${usrRec.USERID})")
            request.session.setAttribute('personalnummer',usrRec.USERID)

           /**
            * Ist Rechteveraltung gewuenscht, 
            * wird diese hier abgehandelt
            */
            RechtDomain recht = new RechtDomain(db:db.dbh)
            session.setAttribute('rechte',recht.RechteListe(usrRec.USERID.toInteger()))
            log.info("Benuterrechte fuer ${usrRec.USERID} ${session.getAttribute('rechte')}")
            
            }
         }
      return true
      }

  /**
   * Aufruf beim ausloggen
   * 
   * An sich genuegt es, wenn die Sessionvariable 'user'
   * entfernt wird. Es wird aber empfohlen
   * auch alle eventuell angelegten Benutzerinformationen 
   * wie z.B.: Rechte zu entfernen
   *
   */ 
   def logout = {sess ->
     	sess.removeAttribute('user')
     	sess.removeAttribute('rechte')
      }
   }