package MVC.domain

/**
 * List of Value
 * 
 * Bearbeiten der Datenbanktabelle LOV
 * 
 */
class LovDomain extends framework.database.DAO.DataAccessObject {
   //
   // .--update--.
   // -- wann----+-wer+-- was -------------------------------------
   // 02.11.2007 | ny | erstellt
   //            |    |
   //            |    |
   //            |    |

  
   Integer lovID
   String  lovClass
   String  lovKey
   String  lovValue
   String  lovFlag1
   String  lovFlag2
   String  lovFlag3
   String  lovFlag4
   String  lovRemark


   def meta = [
      /* Deklarition des Keyfelds */
      idfield:[db:'LOVID',domain:'lovID'],
      /* Tabellenname */
      tablename: 'LOV',
      /* liefert den Naechsten Key */
      nextval:{
         return db.firstRow('select MAX(LOVID)+1 as LOVID from LOV')[0]
         },
      /* 
      Vor dem Schreiben in die Db. 
      In diesem Fall wird, bei Neuanlage der naechsthoehere Schluessel 
      ermittelt.
      */
      before_write:{
         if (this.lovID == null) this.lovID = this.nextval()
         },

      /*
      Verbindung von Klassenvariablen zu den 
      Datenbankfelder
      */
      fields: [
         lovID:      [dbfield:'LOVID',       type:'Integer'],
         lovClass:   [dbfield:'LOVCLASS',    type:'String'],
         lovKey:     [dbfield:'LOVKEY',      type:'String'],
         lovValue:   [dbfield:'LOVVALUE',    type:'String'],
         lovFlag1:   [dbfield:'LOVFLAG1',    type:'String'],
         lovFlag2:   [dbfield:'LOVFLAG2',    type:'String'],
         lovFlag3:   [dbfield:'LOVFLAG3',    type:'String'],
         lovFlag4:   [dbfield:'LOVFLAG4',    type:'String'],
         lovRemark:  [dbfield:'LOVREMARK',   type:'String']
         ]
       ]


  /**
   * Spezialisierte get Methode:
   * Setzt das Objekt auf angegebenen Klasse und key
   * 
   * usage
   *  lov(db:db.dbh).get('CLASS','CLASS')
   * 
   *  Setzt das Objet auf lovClass='CLASS' und lovKey='CLASS'
   * 
   * @param classname      Klassenkennung
   * @param key            Schluessel
   * 
   * @return   t/f         True wenn gefunden
   * 
   */
   Boolean get(String classname,String key) {
      return this.get(where:"LOVCLASS='${classname}' and LOVKEY='${key}'")
      }

  /**
   * liefert einen Value fuer einen Key in Klasse
   *
   * usage: println lov.getValue('RECHTE','DEVELOPER')
   *
   * @param: class     Klasse
   * @param: key       Schluesselwert
   *
   * @return: LOVVALUE oder null wenn nicht gefunden
   */
   String getValue(classname,keyname) {
      def rec = this.db.firstRow('select LOVVALUE from LOV where LOVCLASS = ?  and LOVKEY = ?',[classname,keyname])
      return rec?.LOVVALUE
      }


  /**
   * Wird zur Erzeugung von Selectbox im Taglib
   * benoetigt.
   * 
   * @param      selectedvalue  Eintrag, welcher zur Bildung der Selected Option benutzt wird
   * @param      classname      Inhalt des Feldes LOVCLASS
   * @param      where          optional: zusaetzliches Wherestatmiten (z.B: fuer flags)
   * @peram      orderby        optional: Sortierklausel Default = LOVVALUE
   * @param      nokey          verhindert das anzeigen des Schluessels bei Auswahl
   * @param      truncate       Schneidet den Anzeigetext auf die angegenen Anzahl von Zeichen ab.
   * @param      addempty       
   * @param      optgroup       Wird dieser Parameter bergeben, so werden Optionsgrouppen
   *                            Daten in die Liste eingefgt. Als Label in der Optgroup Option
   *                            wird der Inhalt des Parameter agenommen.
   * usage: lovList('name','NAMENSKLASSE')
   * 
   */ 
   List lovList(Map param) {
      if (param == null) param = [:]

      String selectedvalue = param.get('selected','')
      String classname = param.classname
      
      Boolean nokey = param.get('nokey',false)

      assert selectedvalue != null : "LovDaomin::lovList Option selected fehlt"
      assert classname != '' : "LovDomain::lovList Option classname fehlt"

      String where = ''
      if (param.get('where','') != '') {
         where = ' and '+param.where
         }

      String orderby = 'LOVVALUE'

      if (param.get('orderby','') != '') {
         orderby = param.orderby
         }

      String sql ="""select LOVVALUE as "VALUE", 
                             LOVKEY as "KEY" 
                        from LOV 
                       where LOVCLASS = '${classname}' ${where}
                       order by ${orderby}
                       """
      List retval = []

      // Optgroup Verwaltung einbauen
      //
      if (param.get('optgroup','') != '') {      
         retval.add(['<optgroup>',param.optgroup,null])
         }

      if (param.addempty != null) {
         String empty = param.addempty
         List opt = empty.split(':')
         retval.add([opt[1],opt[0]])
         }
      
      this.db.eachRow(sql) {row->
         Boolean selected = false
         try {
            if (row.KEY.toString() == selectedvalue) selected = true
            }
         catch (Exception e){
            throw new Exception('selectbox: Umwandlungsfehler bei selected: ('+e.message+') value: '+selectedvalue)
            }

         String text = ''
         if(nokey) {
            text = row.VALUE
            }
         else {
            text = "${row.VALUE} (${row.KEY})"
            }

         String popuphelp
         // Abschneiden
         if (param.get('truncate',0) > 0) {
            text = (text.padRight(param.truncate)).getAt((0..param.truncate-1))
            }

         retval.add([row.KEY,text,selected,popuphelp])
         }

      // Wenn optgroup dann wieder ausschalten
      if (param.get('optgroup','') != '')  retval.add(['</optgroup>',null,null])
      return retval
      }

   } // end of domain