/**
 *  Bearbeiten von Angestgelltendaten
 *  
 *  
 * HINTERGRUND
 *    Dieser Controller demonstriert die
 *    Verwendung der Klasse Viewhandler als Maskenlayout
 * 
 * META
 *    Die Domain EmployeesDomain wird bearbeitet
 * 
 * VORAUSSETZUNEN
 *    LovDomain
 *    View  list.tpl       Anzeigen und Filtern von Employees
 *    View  edit.tpl       Bearbeiten eines Datensatzes
 *
 */
package MVC.root.wartung.lov


/**
 * Impertieren der notwendigen Klassen
 */
import MVC.domain.EmployeesDomain
import framework.helper.Viewhandler


/**
 * Basierend auf der CGI Variable filter
 * wird eine Where Klausel Konstruiert
 */
def getFilter = {
   String where = '(1=1)'
   if (cgiparam(name:'filter',nvl:'') != '') {
      where = "lower(LAST_NAME) like lower('${cgiparam(name:'filter',nvl:'')}%')"
      }
   return where
   }


/**
 * Erstellen einer SQL Anweisung, 
 * welche die Datensaetze fuer 
 * die Anzuzeigenden Seiten ermittelt.
 * Die Daten stammen aus dem Listwidget, welches im
 * Viewer angelegt wird.
 */
String sql = db.getSelectLimit(
   table:'EMPLOYEES',
   where:getFilter(),
   orderby:'LAST_NAME,FIRST_NAME',
   pageno:cgiparam(name:'pageno',nvl:'1'),
	server:server,
   domain:new EmployeesDomain(db:db.dbh)
	)


/** 
 * Aus dem List Viewer (list.tpl) wird ein Clousure
 * uebergeben. Die Routiene Filtert die Datansaetze aus der Datenbank
 * und ruft das Closure auf. Dem Closure wir der Datensatz uebergeben.
 * 
 * @param   closure        Programmteil der im Viewer erstellt wird.
 * 
 */
def list = {Closure closure->
   EmployeesDomain employee = new EmployeesDomain(db:db.dbh)

   // Schleife durch die Datensaetze, 
   // sortiert nach Lov Klasse und Lov Schluessel
   employee.eachDomainWithSql(sql) {empRecord->
      closure.call(empRecord)
      }
   }

/**
 * Erstellen der Klasse
 */
EmployeesDomain employee = new EmployeesDomain(db:db.dbh,log:logger)

/**
 * Initialisiere das CRUD Objekts
 *
 */
Viewhandler crud = new Viewhandler(
   /**
    * Bei der Definition des Viewer
    * wird ein Logger uebergeben.
    * Damit werden die SQL Eintraege geloggt (info).
    */
    domain: employee,

   /**
    * Dem Viewer wird der Controller Uebergeben
    */
    controller:this,

   /**
    * Als Layout wird Mask angegeben (Vorgabewert ist GRIDLAYOUT)
    */
    layout:Viewhandler.MASKLAYOUT,              

   /** 
    * Da der Viewhandler im Mask Layout verwendet wird muss
    * auf Listing/Edit Maske umgestellt werden
    */
    action:cgiparam(name:'action',nvl:'list'),

   /**
    * Mit dem Bindingsmechanismus werden 
    * Daten an den Viewer uebergeben.
    *
    */
    viewBindings:[                              
       list:list,
       domain:employee,
       filter:getFilter
       ]
    )


/**
 * Aufrufen von CRUD
 *
 */
crud.run()
