<% 
/**
 * HINTERGRUND
 *    Zeigt die Verwendung des Viewhandlers im Edit-Mode
 *    Dies ist eine Editiermaske. Es wird ein Prompt-Value Layout
 *    angezeigt.
 * 
 * 
 * BEISPIELE (Taglib)
 *     Hidden Fields
 *     Eingabe Felder
 *     Verschieden Methoden von Eingabefelder
 *     Besonder styles in Abhngikeiten von Dateninhalten.
 *     
 *     
 * HINT
 *    Das geheimins der Datenstruktur 'fields'
 *    Die Daten werdem vom Framework dem Viewer
 *    als String uebergeben. "fields" ist eine Map
 *    der Key ist der Domain Feldname.
 *    
 *    Je nach Zustand kommen die Daten aus der Datenbank oder
 *    dem CGI. Dies geschieht fuer den Entwickler vllig transparent.
 *    
 */
import MVC.domain.EmployeesDomain
import MVC.domain.JobsDomain
import helper.DateConverter

DateConverter dc = new DateConverter()

// Taglibobjekt erzeugen
//
def taglib = new MVC.taglib.Taglib(request:request) 

// Zur bequemeren Bearbeitung Daten aus CGI umspeichern
//
String   path              = taglib.cgiparam(name:'path',nvl:'')
String   submitbutton      = taglib.cgiparam(name:'submitbutton',nvl:'')

%>
<h1 class="blockname">Mitarbeiter Stammdaten</h1>

<p>
Stammdaten eines Mitarbeiters.<br />
Dem Mitarbeiter sind ein Job (JOBS) und ein Vorgesetzter zugeordnet.
</p>

<!-- Begin der FORM -->
${taglib.form(name:'edit')}

   <!-- 
     Hiddenfelder:
     Dies demonstriert die Verwendung der Methode hidden
     der Taglib.
   -->
   ${taglib.hidden name:"path",     value:path}
   ${taglib.hidden name:"filter",   value:taglib.cgiparam(name:'filter',nvl:'')}
   ${taglib.hidden name:'action',   value:'edit'}
   <table border="0" cellpadding="1px" cellspacing="1">
    
   <!-- 
   Beispiel einer Feldes das Angezeigt und als hidden deklariert wird.

   ACHTUNG Wird ein Feld als disabled gekennzeichnet (type:'disabled') 
   so wird dieses nicht an das CGI uebergeben.
   -->
   ${
   /* BEISPIEL: Feld als EL-Anweisung */
  /** 
   * HINT:
   *     hier keine Formatierung der AUsgabe durchfuehren
   *     da ein Leerfeld (ist NULL) ein Insert ausloest.
   */
   taglib.promptinput(
       name:'employeeID',
       prompt:'ID',
       size:5,
       fromcgi:false,
       type:'displayandhidden',
       value:(fields?.employeeID?:'')
       )
       }

   <!--
   /**
    * Demonstriert ein Zwei-Felder Layout
    * mit genau einem Prompt
    */ 
    -->

   <tr><th class="msk">Vor- und Familienname</th><td>

   ${taglib.promptinput(
      name:'firstName',  
      prompt:'',
      nohtml:true,
      size:32,
      maxlength:32,
      fromcgi:false,
      nvl:'',
      value:fields?.firstName?:'',
      disabled:false
      )
   }

   ${taglib.promptinput(
      name:'lastName',  
      prompt:'',
      nohtml:true,
      size:32,
      maxlength:32,
      fromcgi:false,
      nvl:'',
      value:fields?.lastName?:'',
      disabled:false
      )
   }

   </td></tr>


   ${
   taglib.promptinput(
      prompt:'e-mail',
      name:'email',
      size:12,
      maxlength:12,
      fromcgi:false,
      nvl:'',
      value:fields?.email?:'')
      }
   ${
   /** 
    * Demonstriert die Verwendung von 
    * Datumsumwandlung in deutsches Format
    */
   taglib.promptinput(
      prompt:'Anstellungsdatum',
      name:'hireDate',
      size:12,
      maxlength:12,
      fromcgi:false,
      nvl:'',
      value:dc.giveAsGermanDate(fields.hireDate))
   }

   <%
   /**
    * Demonstriert die Ausgabe 
    * von formaitierter Ausgabe ein 
    * Dezimalzahl.
    * HINT
    *    In der Datenstruktur fields
    *    werden die Daten als String uebergeben.
    *    Damit ist eine Umwandlung in Double notwendig.
    *    Die Umwandlung muss fehleresistent gestaltet werden.
    */

  /**
   * Hilfsfeld zur Ausgabe
   */
   String auxSalary = '0,00'

  /**
   * Dieses Feld markiet wenn, dass Gehaltsfeld 
   * ueber ein bestimmtes Limit
   */
   Boolean markSalary = false

  /**
   * Wir vesuchen eine Formatierung unter
   * der Annahme, dass das Feld eine korrekte
   * Dezimalzahl enthaellt.
   */
   try {
      auxSalary = sprintf('%.2f',fields.salary.toDouble())
      markSalary = (fields.salary.toDouble() > 3000.0)
      }
   catch (Exception ex) {
     /**
      * Wenn etwas schief ging, dann wird der Inhalt des
      * durch CGI uebergebenen Feldes verwendet.
      */
      auxSalary = fields.salary
      }

   out<<taglib.promptinput(
      prompt:'Gehalt',
      name:'salary',
      size:12,
      maxlength:12,
      fromcgi:false,

     /**
      * Hier wird gezeigt, wie basierend auf einem DB Wert 
      * den Style gendert werden kann
      */
      style:(markSalary ? 'text-align:right;color:red;' : 'text-align:right;color:blue;'),

     /**
      * Formatierte ausgabe
      */
      value:auxSalary
      )
   %>

   ${
   taglib.promptinput(
      prompt:'Manager',
      name:'managerID',
      type:'select',
      fromcgi:false,
      datasource:new EmployeesDomain(db:db.dbh).selectDatasource((fields.managerID?:'0').toInteger())
      )
   }

   ${
   taglib.promptinput(
      prompt:'Job',
      name:'jobID',
      type:'select',
      fromcgi:false,
      datasource:new JobsDomain(db:db.dbh).selectDatasource(fields.jobID?:'0')
      )
   }
   <!-- 
      Wenn ein ID Uebergeben wurde wird der Savebutton/Editbutton wenn nicht nur Savebutton ausgegeben  
    -->
   <tr>
     <td colspan="99">
        <%=(submitbutton == taglib.SUBMITBUTTON_NEWEDIT ? taglib.getNewButton() : taglib.getSaveAndDeleteButton())%>
     </td>
   </tr>

  </table>
</form>
