/**
 * Dynamische Finder:
 * 
 * Das spezielle dieser Mglichkeit besteht darin, 
 * dass die Feldnamen der Domain und nicht der Datenbanktabelle und operatoren bei einer Abrage in
 * den Methodenaufruf kodiert werden. Damit muss der 
 * Controller nichts mehr von den Datenbaktabelle Felnamen wissen.
 * 
 * Beispiel:
 *    jobs.eachDomainByMinSalary(10000)
 * 
 * Dies ist die einfachste Form
 * Es werden alle Jobs selektiert, welche ein
 * das Minimale Gehalt von 10000 haben.
 * Sql: select * from jobs where MIN_SALARY = 10000;
 * 
 * Beispiel:
 *    jobs.eachDomainByJobTitleLike('PU%')
 * Entspricht der SQL ANweisung: select * from  jobs where JOB_TITLE like 'PU%'
 * 
 * 
 */
import MVC.domain.JobsDomain

JobsDomain jobs = new JobsDomain(db:db.dbh)

def query1 = {Closure closure->
   jobs.eachDomainByJobTitleLikeAndMinSalaryGreaterThan(
      orderby:['minSalary','JobID'],
      conditions:['Pu%',1000]) {domJob->
      closure.call(domJob)
      }
   }

def query2 = {Closure closure->
   jobs.eachDomainByJobID(
      conditions:['AC_ACCOUNT']) {domJob->
      closure.call(domJob)
      }
   }


view filename:'results.tpl',
     bindings:[
        query1:query1,
        query2:query2
        ]