/**
 * Demonstriert die Verwendung von eachDomain mit 
 * Einem QueryBuilder
 *
 */
import MVC.domain.JobsDomain
import MVC.taglib.Taglib
import framework.database.DAO.helper.QueryBuilder

Taglib taglib = new Taglib(request:request)

/** Job Domain anlegen */
JobsDomain jobs = new JobsDomain(db:db.dbh)


/**
 * Die Option criteria direkt verwenden
 */
def list1 = {Closure closure->
   /** Daten aus Datenbank holen */
   jobs.eachDomain(criteria:{greaterequals(minSalary:8000)}){domJob->
      closure.call(domJob)
      }
   }

view filename:'query1.tpl',
     bindings: [
        taglib:taglib,
        list:list1,
        where:'',
        text:"""
            <h1>Direkte Angabe in eachDomain</h1>
         jobs.eachDomain(criteria:{greaterequals(minSalary:8000)}){domJob->
            closure.call(domJob)
            }
           """
        ]

/**
 * Einen externen Kriterienbuilder anlgen
 *
 */
QueryBuilder criteria = new QueryBuilder()
criteria.domain = jobs

render "<h1>Criteren seperat erzeugen</h1>"
criteria {
   equal(jobID:'PU_MAN')
   }

list1 = {Closure closure->
   /** Daten aus Datenbank holen */
   jobs.eachDomain(criteria:criteria){domJob->
      closure.call(domJob)
      }
   }

view filename:'query1.tpl',
     bindings: [
        taglib:taglib,
        list:list1,
        where:criteria.getWhere(),
        text:"""
        <h1>Einfaches equal</h1>
            criteria {
               equal(jobID:'PU_MAN')
               }
            list1 = {Closure closure->
               /** Daten aus Datenbank holen */
               jobs.eachDomain(criteria:criteria){domJob->
                  closure.call(domJob)
                  }
               }
           """
        ]

/**
 * AND
 */
criteria.reset()
criteria {
   equal(jobID:'PU_MAN',
         minSalary:8000)
   }

list1 = {Closure closure->
   /** Daten aus Datenbank holen */
   jobs.eachDomain(criteria:criteria){domJob->
      closure.call(domJob)
      }
   }

view filename:'query1.tpl',
     bindings: [
        taglib:taglib,
        list:list1,
        where:criteria.getWhere(),
        text:"""
        <h1>equals mit AND verkn&uuml;pfen</h1>
         criteria {
            equal(jobID:'PU_MAN',
                  minSalary:8000)
            }
           """
        ]

/**
 * OR
 */
criteria.reset()
criteria {
   equal(jobID:'PU_MAN')
   or {
      equal(jobID:'AD_VP')
      }
   or {
      equal(jobID:'FI_MGR')
      }
   }

view filename:'query1.tpl',
     bindings: [
        taglib:taglib,
        list:{Closure closure->
           jobs.eachDomain(criteria:criteria){domJob->
              closure.call(domJob)
              }
            },
        where:criteria.getWhere(),
        text:"""
        <h1>Undverkn&uuml;pfung von gleichnahmigen Feldern </h1>
         criteria {
            equal(jobID:'PU_MAN')
            or {
               equal(jobID:'AD_VP')
               }
            or {
               equal(jobID:'FI_MGR')
               }
            }
           """
        ]

/**
 * EQUAL OR
 */
criteria.reset()
criteria {
   equalOr(
      jobID:'PU_MAN',
      minSalary:20000
      )
   }

view filename:'query1.tpl',
     bindings: [
        taglib:taglib,
        list:{Closure closure->
           jobs.eachDomain(criteria:criteria){domJob->
              closure.call(domJob)
              }
           },
        where:criteria.getWhere(),
        text:"""
        <h1>Oder-Verkn&uuml;pfung</h1>
         criteria {
            equalOr(
               jobID:'PU_MAN',
               minSalary:20000
               )
            }
           """
        ]
