/**
 *  LovController
 *  
 *  
 * HINTERGRUND
 *    Dieser Controller demonstriert die
 *    Verwendung der Klasse Viewhandler als Maskenlayout
 * 
 * META
 *    Die Domain LOV wird bearbeitet
 * 
 * VORAUSSETZUNEN
 *    LovDomain
 *    View  list.tpl       Anzeigen und Filtern von LOV
 *    View  edit.tpl       Bearbeiten eines Datensatzes
 *
 */
package MVC.root.wartung.lov


import MVC.domain.LovDomain
import framework.helper.Viewhandler

/** 
 * Aus dem List Viewer (list.tpl) wird ein Clousure
 * uebergeben. Die Routiene Filtert die Datansaetze aus der Datenbank
 * und ruft das Closure auf. Dem Closure wir der Datensatz uebergeben.
 * 
 * @param   closure        Programmteil der im Viewer erstellt wird.
 * 
 */
def list = {Closure closure->
   LovDomain lov = new LovDomain(db:db.dbh)

   // Schleife durch die Datensaetze, 
   // sortiert nach Lov Klasse und Lov Schluessel
   lov.eachDomain('(1=1)','LOVCLASS,LOVKEY') {lovRecord->
      closure.call(lovRecord)
      }
   }

// Initialisiere das Objekt
//
Viewhandler crud = new Viewhandler(
    domain: new LovDomain(db:db.dbh),           // DOMAIN
    controller:this,                            // Der Controller wird uebergeben
    layout:Viewhandler.MASKLAYOUT,              // Als Layout wird Mask angegeben (Vorgabewert ist GRIDLAYOUT)
   /** 
    * Da der Viewhandler im Mask Layout verwendet wird muss
    * auf Mask Layout ungestellt werden
    */
    action:cgiparam(name:'action',nvl:'list'),  // ACTION setzen
    viewBindings:[                              
       list:list                                // Uebergebe die ListClosurue dem Viewer
       ]
    )

crud.run()