package MVC.root.extras.sonstiges.transaction
/**
 * Dieser Controller demonstriert die Verwendung der Transaktionssteuerrung 
 * der Datenbank.
 * 
 * HINT:
 *    Es obliegt der Verantwortung des Programmieres 
 *    sich zu Ueberzeugen, das die unterliegende Datenbank
 *    oder der Datenbaktreiber die Transaktionskontrolle beherrscht!
 * 
 */
import MVC.domain.JobsDomain

JobsDomain jobs = new JobsDomain(db:db.dbh)

render '<h1>Demonstration des Transactinshandlings</h1>'

render '<pre>'
render '''
Folgende Sequenz erzeugt einen neuen Datensatz in der JOBS Tabelle,
liest diesen wieder ein.
Danach wird ein Rollback durchgef&uuml;hrt. Ein nochmaliges einlesen des
Datensatzes schl&auml;gt fehl, weil Rollback durchgef&uuml;hrt wurde.<p>
<strong>HINWEIS</strong><br />
Die Operationen beginTransaction, commit, rollback und autocommit k&ouml;nnen
auf Controller oder Domainebene durchgef&uuml;hrt werden! In beiden F&auml;llen gelten sie aber f&uuml;r die Datenbankverbindung!<br />
'''

/**
 * Transaktionssteuerung einleiten
 */
beginTransaction()

render "BEGINNE TRANSAKTION\n"
render "<em>beginTransaction()</em>\n\n"

jobs.jobID     ='PUTZ'
jobs.jobTitle  = 'Putze'
jobs.minSalary = 10000.0
jobs.maxSalary = 32000.0

/**
 * Erzwinge INSERT, 
 * uebersteuerte automatische Insert/Update Erkennung
 */
render "ID: ${jobs.jobID}\n"
render "TItle: ${jobs.jobTitle}\n"
render "min: ${jobs.minSalary}\n"
render "max: ${jobs.maxSalary}\n"
Boolean result = jobs.save(mode:JobsDomain.MODE_INSERT)
render "Schreibe Record <em>jobs.save(mode:JobsDomain.MODE_INSERT)</em>, Status: ${result} (true = schreiben OK)\n"

/**
 * Einlesen des geschriebenen Datensatzes
 */
render "READ\n"
jobs.get('PUTZ')

render "Gelesener Record, Status: ${result} (true = lesen OK)\n"
render "ID: ${jobs.jobID}\n"
render "TItle: ${jobs.jobTitle}\n"
render "min: ${jobs.minSalary}\n"
render "max: ${jobs.maxSalary}\n"

render "ROLLBACK <em>rollback()</em>\n\n"
rollback()

render "READ  <em>result = jobs.get('PUTZ')</em>\n"
jobs.clear()
result = jobs.get('PUTZ')

render """Gelesener Record Status: <span style="color:red;font-weight:bold">${result}</span>, 'false' zeigt an, dass der geschriebene Datensatz nicht gefunden wurde (Rollback hat funktioniert) \n"""

/**
 * Autocommit wieder einschalten,
 * ACHTUNG: Dies ist unbedingt notwendig, 
 * wenn Datenbankverbindung gecached wird und somit 
 * die Einstellung weiterhin bestehen Koennten!
 */
autocommit()
