/**
 * Bearbeiten von Jobs
 * 
 * Dieser Controller Konstruiert das SQL Statment
 * Die Datenmenge wird Paginiert dargestellt.
 * 
 * Die CGI Variable 'filter' schraenkt die Datensatzauswahl ein.
 * 
 * Aus Sicht des Viewer wird die Liste der Datenstze aus der Routine
 * 'list' gewonnen.
 *
 */
package MVC.root.jobs

/** 
 * Importieren der bentigten Klassen
 */
import framework.helper.Viewhandler

/**
 * Die zu behandelnde Domain
 */
import MVC.domain.JobsDomain

/**
 * Erstellen einer SQL Anweisung, 
 * welche die Datensaetze fuer 
 * die Anzuzeigenden Seiten ermittelt.
 * Die Daten stammen aus dem Listwidget, welches im
 * Viewer angelegt wird.
 * 
 * Dies wird auf basies der im mframe.xml definierten
 * SQL Anweisung fuer die Paginierung erstellt.
 * 
 */
String sql = db.getSelectLimit(
      table:'JOBS',
		orderby:'JOB_ID',
      pageno:cgiparam(name:'pageno',nvl:'1'),
		server:server,
      domain:new JobsDomain(db:db.dbh)
		)

/**
 * Liste von Jobs
 * Diese Routine setzt ein SQL Statment ab.
 *
 * Basierend auf diesem Statment, wird der Datenstrom
 * durchalufen und fuer jeden Datensatz die uebergebene 
 * Closure aufgerufen
 */
def list = {Closure closure->
   JobsDomain jobs = new JobsDomain(db:db.dbh)

   jobs.eachDomainWithSql(sql) {theJobs->
      closure.call(theJobs)
      }
   }

/**
 * Konstruktion und ausfuehren des CRUD
 * Dem Handler wird die Routine 'list' zur
 * Datensatzgewinnung mitgegeben.
 * 
 * Das CRUD Objekt wird 
 * erzeugt und dabei Konfiguriert.
 * 
 */
Viewhandler crud = new Viewhandler(
   /**
    * Job Domain
    */
    domain: new JobsDomain(db:db.dbh),

   /**
    * Der Controller wird fuer verschiedene Methoden
    * benoetigt
    */
    controller:this,

   /**
    * Wir moechten ein Grid Layout
    */
    action:'grid',

   /**
    * Wird der Parameter gesetzt, so 
    * gibt das Framework vor dem Lschen
    * eine Sicherheitsabfrage aus.
    * 
    * Im Standardverhalten wird nach dem
    * betaetigen der Loesch Schaltflaeche sofort
    * geloescht.
    * 
    */
    gridConfirmOnDelete:Viewhandler.GRID_COFIRM_ON_DELETE,

   /**
    * dem Viewer koennen Daten uebergeben werden.
    *
    * In diesem Fall wird nur der Closure welche
    * die Datenliste fuer den Viewer bereitstellt
    * uebergeben.
    */
    viewBindings:[
       list:list
       ]
    )

/**
 * Der Aufruf regelt das gesamte
 * Create Read Update und Deltet Verhalten
 */
crud.run()

