<%
/**
 * Dieser Viewer demonstriert:
 * ===========================
 * 
 * + die Verwendung von Grid Layout, mit Paginierung
 * + die Verwendung des Inline Editiermodus.
 * + Formatieren von Listenfelder
 * + Formatieren von Eingabefeldern
 * 
 * Inline Edit
 * -----------
 * Das standardverhalten des Viewhandlers kopiert
 * nach dem betaetigen der Editieraktionsschaltflaeche die 
 * Daten der zu editierenden Zeile in den Eingabebereich.
 * 
 * Der Inline Editierer zeigt den Editierbereich in der ausgewaehlten
 * Zeile an. 
 * In allen anderen Faellen, wird der Eiditierbereich am Ende
 * der Liste ausgegeben.
 * 
 */


import helper.DateConverter    // Verschiedene Methoden zur Datumsdarstellung
import MVC.domain.JobsDomain   // Jobs Domain

import MVC.taglib.Taglib       // Applikationspezifische Taglib 
                               // erbt von der Standard Taglib

// Datenkovertierer herstellen
DateConverter dc = new DateConverter()
// Talib Objekt erstellen
def taglib = new Taglib(request:request)

%>

<h1 class="blockname">Jobs</h1>

<!--
 Einbau des Paginierungswidget.
 Im Controller wird aus den Inhalten des CGI
 das SQL Statment konstruiert.
-->

${
/**
 *
 * Listwidget ohne Filter
 *
 */
taglib.listwidget(
   table:'JOBS',
   title:'Jobs',
   filterwidget:false,
   db:db)
}

<table border="0" cellpadding="1px" cellspacing="1" width="100%" id="grid">
  <colgroup>
    <col width="1%">
    <col width="1%">
    <col width="33%">
    <col width="1%">
    <col width="1%">
    <col width="99%">
  </colgroup>

  <%
  // Erzeuge Ueberschriften
  out<<taglib.listheader(
   class:'listHeader',
   th:[
    '#',
    'ID',
    'Name',
    'min&nbsp;Salary',
    'max&nbsp;Salary',
    '&nbsp;'
    ],
    tr:true
    )
   %>

   <!-- ************** -->
   <!-- AUSGABE ZEILEN -->
   <!-- ************** -->
   <%
   /**
    * Diese Datenstruktur stellt die Eingabemaske dar.
    * 
    * hiddenfields:        eine Liste von Hiddenfields fuer die Aktionsschaltflaechen
    * fields:              Eine Liste von Feldern, welche im Eingabebereich 
    *                      angezeigt werden sollen.
    *                      Hier ist auch der Platz, die Aktionsschaltflaechen
    *                      eingegeben werden.
    */
   Map editmask = 
      [
      /**
       * eine Liste von Hidden fields 
       * wird erstellt.
       */
      hiddenfields : [
         taglib.hidden(name:'path', value:taglib.cgiparam(name:'path',nvl:'')),
         taglib.hidden(name:'pageno', value:taglib.cgiparam(name:'pageno',nvl:'1')),
         taglib.hidden(name:'action',value:'grid')
         ],
      /**
       * Fuer jedes Eingabefelds wird ein Inhalt angegeben
       * ACHTUNG: unbedingt das automatische erzeugen der HTML-TD Tags
       * (nohtml:true) angeben.
       *
       * Fuer jeden Eintrag wird eine <td>Eingabefeld</td> erzeugt.
       *
       */
      fields : [
         /** leerspalte anlegen */
         '&nbsp;',
         taglib.promptinput(
            name:'jobID',
            prompt:'',
            size:10,
            maxlength:10,
            type:'text',
            fromcgi:false,
            nohtml:true,
            value:fields.jobID
            ),
         taglib.promptinput(
            name:'jobTitle',
            prompt:'',
            nohtml:true,
            fromcgi:false,
            popuphelp:'Title',
            type:'text',
            size:64,
            maxlength:255,
            value:fields.jobTitle
            ),
         taglib.promptinput(
            name:'minSalary',
            prompt:'',
            nohtml:true,
            size:10,
            maxlength:10,
            type:'text',
            fromcgi:false,
            value:fields.minSalary,
            /** Deklariere Popuphelp des Feldes */
            popuphelp:'Minimales Gehalt, keine Dezimalstellen!',
            /** Extra styles mitgeben */
            style:'text-align:right;font-family:courier;'
            ),
         taglib.promptinput(
            name:'maxSalary',
            prompt:'',
            size:10,
            nohtml:true,
            maxlength:10,
            type:'text',
            fromcgi:false,
            /** Deklariere Popuphelp des Feldes */
            popuphelp:'Maximales Gehalt, Keine Dezimalstellen!',
            value:fields.maxSalary,
            /** Extra styles mitgeben */
            style:'text-align:right;font-family:courier;'
            ),
         taglib.gridSubmitbutton(action,submitbutton)
         ],
      ]
   
   
   

  /**
   * Durchlaufen der Datensatze und fuer jeden Datensatz
   * eine Zeile erzeugen.
   * 
   */
   Integer rowid = 0             // Aktuelle Zeile der Maske
   Boolean showNewInput = true  // Flag, welches anzeigt, ob ein Satz gefuden wurde

   list {rec->

     /**
      * Ermittle ob eine Inline Edit vorliegt
      *
      * Im Flag showNewInput wird gepseichert, wenn 
      * ein Inline Grid Vorliegt.
      *
      */
      if (showNewInput) showNewInput = !(rec.jobID == fields.jobID)

     /**
      * Erstellen der Zeile
      */
      out<<taglib.gridEdit (
         /** Eingabemaske */
         editmask:editmask,
         /** Wenn der zu editierende Datensatz auftritt, dann wird wahr uebergeben */
         showeditmask:(rec.jobID == fields.jobID),
         domain:rec,
         nobr:true,
         values:[
            ++rowid,
            /** Feld, Popuphelp, style */
            [rec.jobID,'Jobkennzeichen','color:gray'],
            rec.jobTitle,
            [taglib.formatDecimal(zahl:rec.minSalary,format:'#,##0'),'','text-align: right;font-family:courier'],
            [taglib.formatDecimal(zahl:rec.maxSalary,format:'#,##0'),'','text-align: right;font-family:courier'],
            ],
         addhidden:[
            /** Wenn Paginierungsfunktion muss aktuelle Seitennummer mitgegeben werden */
            "pageno=${taglib.cgiparam(name:'pageno',nvl:'1')}"
            ]
         )
      }

   /** 
    * Wurde keine Zeile zum 
    * Inline Edit gefunden werden, wird
    * Input Eingabe Bereich angeboten.
    *
    */
   if (showNewInput) {
      out<<taglib.gridEdit (
         editmask:editmask,
         domain:[:],
         showeditmask:true
         )
      }
   %>
</table>
<script type="text/javascript">document.edit.jobID.focus();</script>