// /////////////////////////////////////////////////////////////////
//
// Globale Startroutine der Applikation
//
//
//
//  -- wann --+-wer-+-- was ----------------------------------------------
//  12.10.2007|  ny | estellt
//  21.10.2007|  ny | Datenbankverbindung erstellen
//  19.11.2007|  ny | Produktname und Version fr Datenbank
//  22.12.2007|  ny | Verbesserte Suchstrategie nach Controllerfile
//  14.05.2008|  ny | Einbau von Javascript Untersttzung
//  26.05.2008|  ny | Einbau von Work und logger
//  31.08.2008|  ny | Einbau von Backlinkbar bei navigate 
//  03.10.2008|  ny | Deklaraton eines speziellen Helpfiles
//            |     | 
//
package framework

import framework.helper.Navigate
import framework.database.*
import framework.database.DAO.helper.*
import MVC.authen.CheckUser
import groovy.sql.Sql
import helper.Zeitmessung
import framework.logging.Logging


/**
 * <h2>Zentrale Steuerklasse des Frameworks!</h2><tt>
 * Die KLasse fuegt die Komponenten
 * <ul>
 *  <li>DataAccessObject (DAO)<br/>
 *      Datenzugriffsklasse
 *  <li>Navigate<br/>
 *      Erstellt die Navigationsleisten
 *  <li>Create Read Updte Deltet (CRUD)<br/>
 *      Automaitsches Bearbeitungsmasken
 * </ul>
 * Zusammen.
 * @param request         Requestobjekt
 * @param response        Responseobject
 * @param environment     Datenbankumgebung (mframe.xml database)
 * @param servername      im mframe.xml kann f&uuml;r jeden Server Konfigurationsparameter deklariert werden                        
 * 
 * 
 */
// =====================================================================
class Work {
// =====================================================================


   String body       = ''    // Buffer fr Aufrufe von render im Controller
   String flash      = ''    // Flash

   def db               // Database Object
   def request          //
   def response
   String servername    // Um die Serverkonfiguration zu Bestimmen wird
                        // vom starter (start.groovy) der Servername deklariert
                        // Dieser wird zur Namensbestimmung im mframe.xml <server name="..."
                        // herangezogen.
   String environment   // Name der Datenbankumgebung
                        // wird mit Kontruktor bergeben


   final static VERSION = framework.helper.Version.getVERSION()
   String controllersource
   String path

   String APPROOT
   def xmlConfig
   def navigate
   def Server
   def data = [:]
   def sess
   def dbProductName                // Datebankproductname
   def dbProductVersion             // Datenbankversion
   def applicationversion           // Version der Applikation wird in start.groovy gesetzt
   def stoppuhr
   def maintemplate = 'main.gsp'
   def info = ''
   def helpfilename = 'help.hlp'
   def logger = null                // Logger, Kann von Start initialiert und verwendet werden.
   def javascript='function isRunning() {return true}' // Javascript content:
                                    // Wird in den Header zwischen Skript-Tags eingebaut
   def oRenderhelpfile = null       // Objekt, welches eine Rendermaschine fuer 
                                    // das Helpfile enthaelt. Diese muss die Methode 
                                    // getRenderText(String text) enthalten.
  /**
   * Applikationsglobale Datenbankverbindung herstellen<tt>
   *
   * @param:     environment       DB-Servereinvionment fuer XML
   *
   * @return:    Datenbankobjekt
   *
   * HINT:
   *    Environment wird bein star.groovy gesezt.
   *    Dieses wird dbSetup als parameter mitgegeben
   *
   *    Das Datenbankhandle (dbh) ist Teil des Datenbankobjekts
   *
   */
   //----------------------------------------------------------------------
   def dbSetUp (param) {
   //----------------------------------------------------------------------

      if (param?.environment == null) {
         throw new Exception ('dbSetup muss Option environment haben')
         return false
         }

      def dbParam = Server.database[0].environments[0].get(environment)[0]

      if (db == null) {
         //
         // HINT:
         //       Environment weitergeben
         //
         db = new Database(environment:environment)
         String dbUrl = dbParam.'@url'

        /**
         * Platzhalter root in DB Url verwendet
         */
         if (dbUrl =~ /\$\{root\}/) {
            dbUrl = (dbUrl =~ /\$\{root\}/).replaceFirst(APPROOT)
            // Sicherheitshalber auch noch Doppelslash entfernen
            dbUrl = (dbUrl =~ /[\/][\/]/).replaceAll('/')
            logger.info("DB Database URL auf ${dbUrl} gesetzt")
            }

         Sql dbh = new DbHelper().connect([
                     url:dbUrl,
                     username:dbParam.'@username',
                     password:dbParam.'@password',
                     driverClassName:dbParam.'@driverClassName'
                     ])
         db.dbh = dbh
         if(dbh == null) throw new Exception ("keine Datenbankverbindung herstellbar<br/>"+dbParam)

         def metadata       = dbh.getConnection().getMetaData()
         db.productName    = metadata.getDatabaseProductName()
         db.productVersion    = metadata.getDatabaseProductVersion()
         }
      return db
      }

  /**
   * Initialisierung des Frameworks
   * 
   * + Session
   * + Erzeugen von Status und Hilfsvariabel APPROOT etc.
   * + Erzeugen der Navigation
   * + Verbindung zur Datenbank herstellen
   * + Konfiguration des Containers
   * 
   */
   def init() {

      //
      // Initialisierung
      //
      sess = request?.getSession()
//      request.setCharacterEncoding("ISO-8859-1")

      APPROOT = (String) request?.getRealPath('/')
 
      /** Umwandeln backslash in slash */ 
      APPROOT = (APPROOT =~ /[\\]/).replaceAll('/')

      // Das Konfigurationsfile wird eingelesen

      if (sess.getAttribute('xmlConfig') == null) {
         xmlConfig = new XmlParser().parse(new File("${APPROOT}/WEB-INF/mframe.xml"))
         }
      else {
         xmlConfig = sess.getAttribute('xmlConfig')
         }
      // Frameworkklasse
      //
      navigate = new Navigate(xml:xmlConfig,Server:servername,request:request)
      navigate.approot = APPROOT

      Server = navigate.getServerobject()

      // Caching der Datenbankverbindung
      //
//      if(sess.getAttribute('db') == null) {sess.setAttribute('db',dbSetUp(environment:environment))}
//      db = sess.getAttribute('db')
      db = dbSetUp(environment:environment);
      if (db == null) {
         throw new Exception("Start::init   Datenkbanverbindung nicht herstellbar")
         }
      path =  request.getParameter('path')
      navigate.path = path

      if (request.getParameter('flash') != null) {
         setFlash request.getParameter('flash')
         }
      }


  /**
   * 
   * Fgt in die Javascriptvariable
   * einen text an.
   * 
   * @param script      Skripttext
   * 
   */
   def addJavascript= {String script ->
      javascript += script
      }

  /**
   * Setzen des Haupttemplatefilename
   * 
   * @param   filename Name des Templatefiles
   * 
   */ 
   def setContainer = {param->

      switch (param) {
         case String :
            maintemplate = param
            break
         case Map :
            maintemplate = param.get('filename','')
            break 
         default :
            throw new Exception("Es wurde bei setContainer ein ungueltiger Datentyp uebergeben'")
            break
         }

      // Pruefe ob Datei vorhanden ist
      File cFile = new File("${APPROOT}/WEB-INF/${maintemplate}")
      if (!cFile.exists()) throw new Exception("Container ${maintemplate} nicht vorhanden.")
      }

   /**
    * umsetzen es Helpfilenamens
    * 
    * @param   myHelpfile        Name des aktuellen Helpfiles
    * 
    */
   def setMyHelpfilename={String myHelpfile->
      helpfilename=myHelpfile
      }

   /**
    * Kann von GET und POST verwendet werden.
    * Es wird versucht den Querystring als GET
    * zu erhalten. Ist dies erfolgreich, so wird dieser uebergeben.
    * 
    * Kann kein Querystring gefunden werden so wird angenommen,
    * das ein POST vorliegt. Diese Parameter werden zu einem
    * QueryString zusammengebaut und zurueckgegeben.
    * 
    * HINT:
    *    request.getRequestMethod() funktioniert im Jetty nicht richtig.
    *    
    *    
    */
   def getQueryString = {
      List retval = []
      // Wenn als GET Request dann wird 
      // Deser genommen
      if (request.getQueryString() != null) return request.getQueryString()

      // Sonst auslesen der Parameter
      request.getParameterNames().each{p->
         retval.add("${p}=${request.getParameter(p)}")
         }
      return retval.join('&')
      }

   /**
    * In der Session variable reichte ist eine mit Kommas getrennte
    * Liste von Rechten enthalten. Dieser wird ausgelesen und 
    * als Liste zurckgeliefert.
    */ 
   List getRightsForUser() {
      if (sess.getAttribute('rechte') == null) return []
      return sess.getAttribute('rechte')
      }
 

   // --------------------------------------------------------------------
   Boolean checkUserRights(List entryRights,List userRights) {
   // --------------------------------------------------------------------
      // 
      // Durchlaufen der Benutzerrechte
      // Wenn ein oder mehrer rechte dann darf der Benutzer
      // 
      // @param   entryRights    Geforderte Liste im Menueeintrag
      // @param   userrights     Rechte des Benutzers
      // 
      // @return  false          wenn kein Match
      // 

      // wir nehmen einmal an, dass der Nutzer 
      // die Funktion nicht asusfuehren darf
      boolean found = false

      // Durchlaufe Eintragsrechte
      // und pruefe gegen die Rechte des Benutzers
      //
      userRights.each {recht->
         if (entryRights.grep(recht)) found = true
         }

      // Ergebnis zurueckliefern
      return found
      }
   
   //--- Ausgabe auf out --------------------------------------------------
   def render = {text ->
   //----------------------------------------------------------------------
      //
      // Diese Routine wird dem Controller mitgegeben 
      // 
      body += text
      }

   //---read controllerfile -----------------------------------------------
   def readTemplatefile = {param->
   //----------------------------------------------------------------------
      //
      // @param: filename        Name des Templatfiles
      // @param: rec             Map auf Vorgabewerte
      //
      // @return: gerendertes Templatefile
      //

      if (param == null) param = [:]
      String filename = param.get('filename','')

      if (filename == null) { // kein Filename uebergeben
         throw new Exception("Beim aufruf des Viewer wurde keine Filename uebergeben")
         return
         }


      def template
      String templateFilepath
      String retval = ''

      if (filename =~ /^[\/]/) {
         templateFilepath = filename
         }
      else {
         templateFilepath = "${path}/${filename}"
         }

      templateFilepath = (templateFilepath =~ /^\//).replaceFirst('')
      String filePath = navigate.buildFilePath(templateFilepath)

      // erzeugen Template
      def engine = new groovy.text.SimpleTemplateEngine()

      File tplFile 
      try {
         tplFile = new File(filePath)
         }
      catch (Exception e) {
         throw new Exception ("Kann auf Templatefile ${filePath} nicht zugreffen ${e.message}")
         }

      if (!tplFile.exists()) {
         throw new Exception ("Templatefile ${filePath} ist nicht vorhanden")
         }

      try {
         template = engine.createTemplate(tplFile)
         }
      catch (Exception e){
         throw new Exception ("Fehler bei lesen von '${filePath}' Fehler: ${e.message}")
         return
         }

      if (db == null) {
         throw new Exception ("Start::readTemplateFile   Datenbankobjekt (db) wurde nicht erzeugt oder ist null")
         }

      // Bindingvariable
      Map bindings = [
         db:db,
         request:request,
         response:response,
         session:sess,
         rec:param.get('rec',null),
         where:param.get('where',''),
         server:navigate.getServerobject(),
         addFunction:addFunction,
         reload:reload,
         view:view,
         sview:sview,
         flash:setFlash,
         cgiparam:cgiparam
         ]

      // Wurden zusaetzliche Bindingvariable uebergeben
      //
      if (param.bindings != null) {
         param.bindings.each{b->
            bindings[b.key] = b.value
            }
         }

      return template.make(bindings)
      }


   //--- Aufrufen eines viewer ---------------------------------------------
   def view = {param ->
   //----------------------------------------------------------------------
      //
      // Shortcut fuer readTemplatefile
      //
      render readTemplatefile(param)
      }

   //--- Gibt ausgabe direkt zurck-----------------------------------------
   def sview = {param ->
   //----------------------------------------------------------------------
      //
      // gibt ausgabe nicht in Render aus sonder als String zurck
      //
      return readTemplatefile(param)
      }

   //--- Aufrufen eines viewer ---------------------------------------------
   def reload = {query_string->
   //----------------------------------------------------------------------
      //
      // Shortcut fuer readTemplatefile
      //
      if (query_string == null) query_string = getQueryString()
      String url = Server.entrypoint[0].'@value'+'?'+query_string
      response.sendRedirect(url)
      }
   

   //----------------------------------------------------------------------
   def cgiparam = {param->
   //----------------------------------------------------------------------
      //
      //
      //
      if (param == null) param = [:]

      String nvl
      String retval = ''

      if (param.nvl) nvl = param.nvl
      if (param.isnull) nvl = param.isnull

      retval = request.getParameter(param.name)
      // Blank wenn kein Parameter gefunden wurde
      if (retval == null) {
         // Wenn ein Vorgabewert definiert wurde
         retval = ''
         if (nvl) retval = nvl
         }
      return retval
      }


   // --------------------------------------------------------------------
   def addFunction = {param ->
   // --------------------------------------------------------------------
      //
      // fuegt eine Eintrag in der Linkleiste hinzu
      //
      // usage addFunktion(path:'/root',text:'...)
      //
      // @param: path: Pfad z.B. '/root'
      // @param: text: Anzeigetext
      // @param: title: Text des Popup Help
      // @param: controller: Name des Controllers (doit.groovy)
      // @param: cgiparameter: Liste von parameter welche an den Url angehaengt werden z.B.: ['eins=1','zwei=2']
      //
      if (param.path == null) param.path = path
      navigate.addFunction(param)
      }

   // --------------------------------------------------------------------
   def setFlash = {message->
   // --------------------------------------------------------------------
      //
      // Flash setzen
      //
      flash = message
      }

   // --------------------------------------------------------------------
   def addData = {key,value->
   // --------------------------------------------------------------------
      //
      // Dem Viewer werden Daten zu Verfgung gestellt.
      // diese werden in der Variable 'data' gesammelt.
      // Diese Methode erlaubt das setzen von Varible
      //
      //  @param:    key      Schluesselfeld der Map
      //  @param:    value    Inhalt
      //
      // HINT:
      //    Standartvarible werden in der Methode run
      //    gesetzt.
      //
      // usage: addData('test','1234')
      //
      data[key] = value
      }

   // --- darf controller verwendet werden--------------------------------
   Boolean userHasRight (entry) {
   // --------------------------------------------------------------------
      // 
      // Ist im Menueeintrag eine Rechteeinschraenkung notiert (rights)
      // Wird gegen die Benutzerrechte des angemeldeten Benutzer validiert
      // 
      // HINT: Die Methode hat mehrer Ausgaenge
      // 
      // 
      // @param:  entry    Verweis auf den Menueeintrag aus mframe.xml
      // @return: true wenn Benutzer berechtigt ist sonst false
      // 

      // kein Rechte in Menueintrag
      // der Benutzer darf somit den Controller aufrufen
      //
      if (entry.rights[0] == []) {
         return true
         }

      List entryRights = (entry.rights?.'@value'[0][0]).split(',')
      List userRights = getRightsForUser()
      boolean found = checkUserRights(entryRights,userRights)

      // wenn kein match, dann darf der Benutzer nicht
      // Ausgeben loggeintrag und tschuess...
      if (!found) { 
         logger.info("Die angeforderte Funktion '${path}' benoetigt das Recht '${entryRights}' dieses wurde der Benutzerkennung ${sess.getAttribute('user')} nicht zugeteilt.")
         return false
         }

      // wenn wir bishierhin gekommen sind
      // darf der Benutzer...
      return true
      }

  /**
   * Transaktion der Datenbank behandeln
   * 
   * Folgnde Funktionen stehen zu Verfuegung:
   * 
   * + beginTransactin        starten der Transaktion
   * + commit                 Finalisieren der Datenbankeintraege
   * + rollback               Ruecksetzen der Datenabnk
   * + autocommit             Datenbank wieder in den Autocommitzustand versetzen
   * 
   * Aus Bequemlichkeitsgruenden koennen diese Funktion auf Controller und Domainebene
   * Durchgefuehrt werden.
   * 
   * 
   */
   def beginTransaction = {db.dbh.useConnection.autoCommit=false}
   def commit           = {db.dbh.commit()}
   def rollback         = {db.dbh.rollback()}
   def autocommit       = {db.dbh.useConnection.autoCommit=true}

  /**
   * Behandelt das Erzeugen eines Controllers
   * 
   * + Bindingsvariable bereitstelle
   * + Ueberpruefen, ob User diesen Controller aufrufen darf
   * + Lokalisieren des Controllers im Dateisystem
   * + Starten des Controlles (mit GroovyShell)
   * 
   * Resultat: 
   *  Der Controller kann null, true oder false zurueckiefern
   *  bei null oder true wird die Anzeige vom Framework uebernommen.
   *  Liefert der Controller false, so wird das Anzeigen dem Controller  
   *  uberlassen. Es wird keine Containerverarbeitung durchgefuehrt.
   * 
   */
   Boolean handleController(controllerFilePath)  {
      Boolean result = true
      
      def binding = new Binding(render:render,
                                sess:sess,
                                request:request,
                                response:response,
                                db:db,
                                addFunction:addFunction,
                                readTemplatefile:readTemplatefile,
                                view:view,
                                flash:setFlash,
                                cgiparam:cgiparam,
                                body:body,
                                mframe:navigate,
                                APPROOT:APPROOT,
                                path:path,
                                reload:reload,
                                xmlconfig:xmlConfig,
                                javascript:addJavascript,
                                helpfilename:setMyHelpfilename,
                                logger:this.logger,
                                work:this,
                                server:navigate.getServerobject(),
                                setContainer:setContainer,
                                beginTransaction:beginTransaction,
                                commit:commit,
                                rollback:rollback,
                                autocommit:autocommit,
                                getQueryString:getQueryString
                               )
      def shell = new GroovyShell(binding)    // Erzeugen Shell

      if (shell == null) { // Fehler abfangen
         throw new Exception ('Start::run   Kann shell nicht erzeugen')
         }
    
      def entry = navigate.myEntry(request.getParameter('path'))
      if (entry == null) {
         throw new Exception("Der Pfad ${request.getParameter('path')} existiert nicht")
         return false
         }

      String controllerFileName = entry?.controller[0].'@path'[0]
      // ueberprfen, ob Controller von eingeloggten Benutzer
      // verwendet werden darf.
      //
      if (!userHasRight(entry)) {
         setFlash("Sie haben nicht das Recht diese Funktion (${path}) auszuf&uuml;hren")
         Object session = request?.getSession()
         session.removeAttribute('user')
         controllerFileName = ''
         }

      // Absoluter oder Relativer Pfad
      //
      if (controllerFileName =~ /^\//) { // Beginnt Filepfad mit "/"
         // Es wird angenommen, dass der Benutzer irgendwo innerhalb
         // $APPROOT/classes/$ENTRYPOINT den controller abgelegt hat.
         controllerFilePath = controllerFileName
         }
      else {
         controllerFilePath = request.getParameter('path')+'/'+controllerFileName
         controllerFilePath = (controllerFilePath =~ /^\//).replaceFirst('')
         }
      controllersource = navigate.readControllerfile(controllerFilePath)

      // Wenn eine Controllerscript gefunden wurde
      // dieses ausfhren
      if (controllersource != '') {
         // Das Controllerscript gibt einen
         // wharheitswert zurck
         // Bei FALSE wird die Bearbeitung abgebrochen
         
         try {
            // Beim abarbeiten des Controllers knnen
            // unterschiedliche Fehler auftreten
            // 1) Fehler bei der Eingabe (es wird false zurckgliefert)
            // 2) Ausnahmebehandlung (alles andere)
            //
            // wurde ein Fehler entdeckt so
            // wird die Bearbeitung abgebrochen
            result = shell.evaluate(controllersource)
            }
         catch (Exception e) {
            throw new Exception ("\nAusnahmebeandlung im Controller ${controllerFilePath} \n ${e.message}\n")
            return false
            }
         }
      return result
      }

  /**
   *
   * Prft ob im Pfad ein Helpfile vorhanden ist (help.hlp)
   * und erzeugt einen Hilfetext
   * 
   */
   String getHelp() {
      if ((helpfilename?:'') == '')  helpfilename = 'help.hlp'
      String helpFilepath = "${path}/${helpfilename}"
      helpFilepath = (helpFilepath =~ /^\//).replaceFirst('')
      String filePath = navigate.buildFilePath(helpFilepath)
      
      if (new File(filePath).exists()) {
         String helpText = new File(filePath).text

         // ist das Objekt definiert
         if (oRenderhelpfile != null) {
            helpText = oRenderhelpfile.getRenderText(helpText)
            }

         return """ <a href="javascript:showhelp()"><img src="/${Server.approot[0].'@value'}/images/skin/help.png" border="0" title="Schaltes Hilfe ein und aus">Hinweise</a><div class="helpbox" id="help" style="display:none">${helpText}</div> """
         }
      else {
         return ''
         }
      }

  /**
   * Start des Frameworks
   * Nach dem Init wird das Framework gestartet.
   * <ul>
   *  <li> aufrufen des Controllers
   *  <li> einfuegen des Resultats des Controllers in den Containers
   * </ul>
   *
   */
   String run () {
      /**
      *
      *  MAIN DIVISON
      *
      **/
      def template
      boolean isOK = true               // Rckgabewert des Controllers
      String controllerFilePath         // Dateiname und Pfad auf Controller

      // wird kein Pfad angegeben wird Startsequenz ausgefhrt
      //
      if (request.getParameter('path') == null) { 
         controllerFilePath = path == null ? xmlConfig.menu[0].config[0].start[0].'@path' : path
         }
      else  {
         Boolean result

         // AUFRUF des Controllers
         // HINT:
         //   liefert der Controller false zurck
         //   so wird eine Leerstring an start.groovy
         //   zurckgegeben. Die Bewirkt, dass 
         //   kein Output geniert wird.
         //
         result = handleController(controllerFilePath)
         if (result == false) return ('')
         }
      // Templatefactory erzeugen
      def engine = new groovy.text.SimpleTemplateEngine()

      // //// Speichert wohin gesprungen wird
      //
      String redirectto  = ''

      // ///// Spezialbehandlung von Login/Logout
      //
      switch (path) {

         // Loginpath
         case xmlConfig.menu[0].config[0].login[0].'@path' :    // Login (check)
            new MVC.authen.CheckUser().check(request:request,db:db);
            if (!sess.getAttribute('user')) {
               flash = 'Fehlerhafte Anmeldung'}
            else {
               String redirect = xmlConfig.menu[0].config[0].start[0].'@path'
               if (cgiparam(name:'redirectto',nvl:'') != '') redirect = cgiparam(name:'redirectto',nvl:'')

               response.sendRedirect(Server.entrypoint[0].'@value'+'?path='+redirect)
               }
            break;

         // Logout
         case xmlConfig.menu[0].config[0].logout[0].'@path' :        // Ausloggen
            new CheckUser().logout(sess)
            response.sendRedirect(Server.entrypoint[0].'@value'+'?path='+xmlConfig.menu[0].config[0].start[0].'@path')
            break

         }

      // Prfe ob login notwendig
      // Wenn keine Sessionvariable user deklariert
      // ist wird angenommen, dass noch kein login
      // vorgenommen wurde.
      //
      if (sess.getAttribute('user')) {
         redirectto = maintemplate
         }
      else {
         redirectto = 'login.gsp'
         }


      // Erzeuge Template
      // im Fehlerfall wird das Programm abgebrochen
      //
      try {
         template = engine.createTemplate(new File(APPROOT + 'WEB-INF/' + redirectto))
         }
      catch (Exception e){
         throw new Exception('Fehler bei redirect: '+redirectto+'<br/>'+e.message)
         return
         }

      // ///// Bindngs
      //
      data['realpath']           = APPROOT
      data['path']               = path
      data['session']            = sess
      data['user']               = sess.getAttribute('user')
      data['menu']               = navigate.menu(path,sess.getAttribute('rechte'))
      data['mframeversion']      = VERSION
      data['startpath']          = xmlConfig.menu[0].config[0].start[0].'@path'
      data['loginpath']          = xmlConfig.menu[0].config[0].login[0].'@path'
      data['logoutpath']         = xmlConfig.menu[0].config[0].logout[0].'@path'
      data['redirectto']         = cgiparam(name:'redirectto',nvl:'')
      data['body']               = body
      data['flash']              = """<div id="flash" class="msk-flash" style="${(flash =~ /^\s*$/) ? 'display:none' : 'display:block'}">${flash}</div>"""
      data['server']             = navigate.getServerobject()
      data['request']            = request
      data['response']           = response
      data['stoppuhr']           = stoppuhr
      data['applicationversion'] = applicationversion
      data['info']               = info
      data['help']               = getHelp()
      data['javascript']         = """ <script type="text/javascript">\n// Javascript des Controller aus ${path} \n${javascript}</script> """

      response.setHeader('path',path)
      return template.make(data)
      }
   

   } // END OF CLASS