// ////////////// Datenbankhilfroutinen ////////////////////////////
//
// Enthllt Hilfsroutinen fuer die Datenbankbenutzung
// 
// @autor: W.Nagy
// 
// -- wann --+-wer-+-- was ----------------------------------------------
// 18.12.2007|  ny | Einbau von OraDateStringToDate Routine
//           |     | 
//           |     | 
//           |     | 
//           |     | 
//
//
//

package framework.database.DAO.helper

import groovy.sql.Sql
import java.text.SimpleDateFormat


class DbHelper {

   def connection = null

   // --- Verbindung -------------------------------------------------------
   def connect = {param->
   // ----------------------------------------------------------------------
      //
      // Stellt die Datenbankverbindung her
      // Das Datenbankhandle wird als Klassenvariable abgelegt.
      //


      try {
         connection = Sql.newInstance(param.url,param.username,param.password,param.driverClassName)
         }
      catch (Exception e){
         throw new Exception("Grauenvoller Datenbank Verbindungsfehler: "+e.message)
         return null
         }
      return connection
      }
   
   // --- Konvertieren eines ANSI Date-Time String -------------------------
   Date OraDateStringToDateExtend (String datum) {
   // ----------------------------------------------------------------------
      // 
      // Um das Datumsproblem bei Spalten mit DATE Type zu lsen
      // Wird bei der Methode get des DAO Objektes das Datum zu einem
      // ANSI-Datumsstring konvertiert (CCYY-MM-DD HH:MI).
      // 
      // Diese Methode konvertiert das diesen String in eine groovy Date Objekt
      // 
      // @param:  datum    Datumstring im ANSI Format CCYY-MM-DD HH:MI
      // 
      // @return: Datumsobjekt
      // 

      java.text.ParsePosition pos = new java.text.ParsePosition((int) 0)
      
      String format = 'yyyy-MM-dd k:m'
      switch(datum) {
         case ~/^d{4}\-\d{2}\-\d{2}\s\d{2}:\d{2}$/ : 
            format = 'yyyy-MM-dd k:m'
            break
         case ~/^d{4}\-\d{2}\-\d{2}$/ :
            format = 'yyyy-MM-dd'
            break
         default :
            //throw new Exception("DbHelper::OraDateStringToDate '${datum}' unbekanntes Datumsformat")
            break
         }
      SimpleDateFormat df = new SimpleDateFormat(format, Locale.GERMAN);

      Date theDate = df.parse(datum,pos)
      // Trat ein Datumumwandlungsfehler auf
      if (pos.getErrorIndex() != pos.getIndex()) {
         throw new Exception("Datumsumwandlungsfehler bei ${datum}")
         }
      return theDate
      }
   
   // --- Konvertieren eines ANSI Date-Time String -------------------------
   Date OraDateStringToDate (String datum) {
   // ----------------------------------------------------------------------
      // 
      // Um das Datumsproblem bei Spalten mit DATE Type zu lsen
      // Wird bei der Methode get des DAO Objektes das Datum zu einem
      // ANSI-Datumsstring konvertiert (CCYY-MM-DD HH:MI).
      // 
      // Diese Methode konvertiert das diesen String in eine groovy Date Objekt
      // 
      // @param:  datum    Datumstring im ANSI Format CCYY-MM-DD HH:MI
      // 
      // @return: Datumsobjekt
      // 
      SimpleDateFormat df = new SimpleDateFormat('yyyy-MM-dd k:m', Locale.GERMAN);
      Date theDate = df.parse(datum)
      return theDate
      }   } // END OF CLASS