package framework.database.DAO.helper
/**
 * Klasse fuer Groovy Builder
 * 
 * Diese Klasser ermglicht die Erstellung
 * der Kriterien fuer eine Datenbankabfrage mittels der Domainfelder.
 * 
 * Conditions:
 * 
 * Abfragekrien:
 *  diese werden durch den Relationaloperator (z.B: equal) und einer Map 
 *  bestehend aus dem Feldnamen und dem Kriterium gebildet.
 *  Beispiel:   equal(jobID:'PU_ACCOUNT')
 *  
 *  Es knnen auch gleich mehrere Kriterion zusammengefasst werden
 *  Beispiel:   equal(jobID:'PU_ACCOUNT',maxSalary:10000). Diese werden
 *  mit AND verkettet: jobID = PU_ACCUNT AND maxSalary=10000
 * 
 *  Durch erweitern des Kriteriennamens um 'Or' oder 'And' kann die 
 *  Verkettung beinflusst werden,
 *  
 * Relationsoperatoren 
 *    + equal
 *    + notequal
 *    + like
 *    + greater
 *    + greaterequals
 *    + less
 *    + lessequals
 *    + like
 *    + notlike
 * 
 * Logische Operatoren
 *    + or
 *    + and
 *    + not
 * Die im Clousure zusammengefasseten Realationsoperatoren weden 
 * geklammert und mit dem Logschien Opeator geklammert.
 * 
 * def query = new QueryBuilder()
 * 
 * query {
 *    equal(x:12,y:'abc')
 *    or {
 *       equalAnd(z:23,y:'xxxx')
 *       and {
 *          equalOr(a:2,b:3)
 *          }
 *       }
 *    }
 * 
 * 
 * println query.getWhere()
 *
 */
class QueryBuilder extends BuilderSupport {

   def level = 0
   List whereList = []
   Integer klammern = 0
   Boolean andNeed = true
   def domain

  /**
   * Zusammenbau einer Condition
   *
   * Eine Condition kann aus mehren Kriterein besthen.
   * Die Kreiterin werden mit logischen Operatoren (AND|OR) getrennt
   *
   */
   protected void newCondition (attr,operator,lop) {

      // Wenn where schon inhalt hat 'AND' anfuegen
      if (where.size() > 0 && andNeed) whereList.add(' AND ')

      // Hilfsvariable
      List auxWhere = []

      // Durchlaufen  der Felder und Values
      attr.each{fldname,fldvalue->
         String dbFieldName = domain.getDbFieldName(fldname)
         String value

         // String quoten
         switch(domain.getType(fldname)) {
            case 'String' :
               value = "'${fldvalue}'"
               break
            default :
               value = fldvalue
               break
            }
         switch(operator) {
            case 'in' :
               auxWhere.add("${dbFieldName} ${operator} (${fldvalue}) ")
               break 

            default :
               auxWhere.add("${dbFieldName} ${operator} ${value} ")
               break
            }
         }
      whereList.add(auxWhere.join(" ${lop} "))
      andNeed = true
      }

   protected createNode(name) {
      if (name == 'call') return
      createNode(name,null,null)
      }


   protected createNode(name,value) {
      createNode(name,null,value)
      }

   
   protected createNode(name, Map attr) {
      createNode(name,attr,null)
      }

   protected createNode(name, Map attr, value) {

      String lop = 'AND' // logischer Operator

      switch (name) {
         case ~/^.*Or$/:
            lop = 'OR';
            name = (name =~ /Or$/).replaceFirst('')
         break
         case ~/^.*And$/:
            lop = 'AND';
            name = (name =~ /And$/).replaceFirst('')
         break
         }

      // Tiefenstufe errechnen
      def level = current ? current : 0
      
      switch(name) {
         case 'and' :
            whereList.add(' AND (')
            klammern++
            andNeed = false
            break

         case 'or' :
            whereList.add('OR (')
            klammern++ 
            andNeed = false
            break
         
         case 'not' :
            if (where.size() > 0 && andNeed) whereList.add(' AND ')
            whereList.add(' NOT (')
            klammern++
            andNeed = false
            break

         case 'equal' :
            newCondition(attr,'=',lop)
            break
         
         case 'notequal' :
            newCondition(attr,'<>',lop)
            break

         case 'greater' : 
            newCondition(attr,'>',lop)
            break
         
         case 'greaterequals' : 
            newCondition(attr,'>=',lop)
            break

         case 'less' : 
            newCondition(attr,'<',lop)
            break

         case 'lessequals' : 
            newCondition(attr,'<=',lop)
            break

         case 'like' : 
            newCondition(attr,'LIKE',lop)
            andNeed = true
            break

         case 'notlike' : 
            newCondition(attr,'NOT LIKE',lop)
            andNeed = true
            break

         case 'in' : 
            newCondition(attr,'IN',lop)
            andNeed = true
            break
         
         default : 
            throw new Exception("Ein unbekannte Operator '${name}' stellt sich vor")
         }
      return level+1
      }

   void setParent(Object parent, Object child) {
      }

   void nodeCompleted(parent, node) {

      // Oberste ebne erreicht,
      // Alle offenen Klammern schlieen
      if (node == null) { 
         whereList.add(')' * klammern)
         }
      }

   /**
    * Bei der Mehrfachverwendung von Criteria
    * muss der Builder zurueckgesetzt werden
    * 
    * Domain wird nicht zurueckgesetzt
    * 
    */
   void reset() {
      level = 0
      whereList = []
      klammern = 0
      andNeed = true
      }

  /**
   * Liefert das Kriteren
   * 
   * @return where Clause
   * 
   */
   String getWhere() {
      return whereList.join(null)
      }
   }

