// ////////// Datenbankverbindung ////////////////////////////
//
//
// .--update--.
// -- wann----+-wer+-- was -------------------------------------
// 13.10.2007 | ny | erstellt
// 02.11.2007 | ny | auf DAO angepasst
// 09.11.2007 | ny | Paginatesql aus XML Configuration verwenden
//            |    |

/**
 *
 * Hintergrund<br/>
 *
 * Die Klasse Kapselt DAO Objekte und stellt basisoperationen
 * fuer Buisiness Objekte bereit.
 *
 */
package framework.database

import groovy.sql.Sql

class Database extends framework.database.DAO.DataAccessObject {

   /**
   *
   *     Klassenvariable
   *
   */

   static final defaultdbname    = "mydatabase"
   String lastSQLError           = ''
   String productName            = ''
   String productVersion         = ''
   def dbh
   String environment            = ''        // Datenumgebung aus mframe.xml

   // --- Sqlanweisung ausfuehren ------------------------------------------
   def exec = {param ->
   // ----------------------------------------------------------------------
      //
      // Zentrale Stelle an der SQL Anweisungen durchgefhrt werden
      //
      // @param: sql: Sqlstatment
      // @values: (optional) Liste von Werten fuer prepared SQL Statment
      // @return: true/false
      //

      if (executeit) {
         if (param.values == null) {
            try {
               connections[this.currentConnectionName].execute(param.sql)
               }
            catch (Exception e){
               lastSQLError = e.message+"\n"+param.sql
               return false
               }
         }
         else {
            try {
               connections[this.currentConnectionName].execute(param.sql,param.values)
               }
            catch (Exception e){
               lastSQLError = "SQLERROR: '${e.message}'"+"\n"+param.sql
               return false
               }
            }
         }
      return true
      }

   // --- Limit-Select -----------------------------------------------------
   def getSelectLimit = {param->
   // ----------------------------------------------------------------------
      //
      // Liefert eine SQL-Select das paginiert (von-bis)
      //
      // HINT: fuer ORACLE defniert
      //
      //
      // @param: table        Tabellenamen
      // @param: from         von Datensatz (in Oposition mit pageno)
      // @param: pageno       Seitennummer (from wird errechnet; in Opostion mit from)
      // @param: size         Anzahl der Datenstze
      // @param: where        Zusaetzliche WHERE CLAUSE
      // @param: pagesize     Seitengroesse (Anzahl der Datensaetze, optional Vorabe=16)
      // @param: statment     gesamtens Statment. Vom System wird nur from und to eingesetzt
      //
      // Optional: 
      // @param: orderby      Sortierkriterium
      // @param: listclause   Liste von Listfelder z.B: ['LOVID','LOVCLASS']
      //                      Nicht kompatieble mit domain
      // @param: domain       Domainobjekt
      //                      Nicht kompatiebel mit listclause
      //                      Es werden alle Felder aus der Domain genommen
      // @param: fromextend   Erweiterung der FROM clause z.B: fromextend:'inner join PERSON on (perID=xID)
      // @param: server       Datenbank Server aus xmlConfig
      //
      //
      // @result: SQL Anweisung
      //
      if (param == null) param = [:]

      Integer pagesize = param.get('pagesize',16)
      String orderby = param.orderby != null  ? " order by ${param.orderby} " : ''
      Integer pageno = param.pageno.toInteger()

      Integer from = param.get('from',null)
      // kein from bergeben, es muss pageno uebergeben worden sein
      if (from==null) {
         assert param.pageno != null : "getSelectLimit: die Option pageno fehlt"
         from = pageno == 1 ? 1 : (pageno-1)*pagesize+1
         }

      Integer to = from + pagesize-1
      String where = param.get('where',' (1=1)')

      String sql = param.server.database[0].environments[0].get(this.environment).paginatesql[0].text()


      String fromextend = param.get('fromextend','')

      def engine = new groovy.text.SimpleTemplateEngine()

      def template = new groovy.text.SimpleTemplateEngine().createTemplate(sql)
      
      // Feldliste fuer SQL Anweisung als List
      List listClause = []
      // die Parameter listclause und domain sind zueinander inkompatiebel
      // 
      if (param.listclause == null) {
         if (param.domain == null) {
            throw new Exception ('Database::getSelectLimit: weder listclause noch domain Option angegeben')
            return
            }
         else {
            // Felder der Domain durchlaufen
            param.domain.meta.fields.each {field->
               String fieldname = field.value?.dbfield == null ? field.key : field.value?.dbfield
               listClause.add(fieldname)
               }
            }
         }
      else  {
         listClause = param.get('listclause',[])
         }


      if (pageno == 0) { // alle
         sql = 'select '+listClause.join(',')+" from ${param.table} ${orderby}"
         }
      else {
         sql = template.make([
                             fields:listClause.join(','),
                             table:param.table,
                             where:where,
                             from:from,
                             to:to,
                             orderby:orderby,
                             fromextend:param.fromextend,
                             pagesize:pagesize
                             ])
         }
      return sql
      }

   // ----------------------------------------------------------------------
   def paginateList = {param->
   // ----------------------------------------------------------------------
      //
      // gibt eine Liste von Domains fr Paginate zurck
      //
      //

      def list = []

      def domain = param.get('domain',null)
      String where = param.get('where',null)
      String orderby = param.get('orderby',null)

      domain.eachDomain(
                        where:where,
                        orderby:orderby
                        ) {d->
         list.add(d)
         }
      return list
      }

   } // end of class
