// ////////// CRUD  ////////////////////////////
//
// Kapselt Create Read Update und Deleteoperationen
//
// Die Klasse erhaellt eine Domainobjekt
// Basierend auf Inhalten des CGI knnen Datebankopeationen Durchgefhrt werden.
//
// @Author: Wilhelm Nagy
//
// -- wann --+-wer-+-- was ----------------------------------------------
// 09.11.2007| ny  | erstellt
// 31.01.2008| ny  | einbau INCLUDE
//           |     |
//           |     |
//           |     |
//

package framework.helper

import MVC.taglib.*

/**
 * <h1 style="background-color:gray;color:darkblue">Create Read Update und Delete Operationen</h1>
 * 
 * <abstract>
 * <tt><pre>
 * Voraussetzungen
 * ===============
 *
 * Die Domain muss nach den Regeln der Domainspezifikation erstellt sein
 * (POGO + meta)
 *
 * Informatinen aus dem CGI
 * ------------------------
 *    + action
 *      enhaellt die gewnshte aktion
 *    + submitbutton
 *      bei schreibenden Operationen (action = 'save')
 *      werden Zusatzinformationen (delete, update' bertragen
 *    + <domainfelder>
 *      Die Routine befllt die Darten des Objektes mit gleichnamigen
 *      Inhalten aus dem CGI.
 *      HINT:
 *          Achtung auf Gross- und Kleinschreibung
 *
 *
 * Zur Darstellung muss es im Verzeichnis des Controllers zwei
 * Viewer zum Darstellen der Maske und der Liste geben.
 * Maske:      'edit.tpl'
 * Liste:      'list.tpl'
 *
 * Die Namen knnen durch entsprechende Methoden gendert werden.
 *
 * <h2>Beispiel</h2>
 *  CRUD crud = new CRUD(
 *                   request:request,
 *                   domain:new RechtDomain(
 *                      db:db.dbh,
 *                      autor:request.getSession().getAttribute('personalnummer')
 *                      ),
 *                   // Funktionen aus dem Framework
 *                   addFunction:addFunction,
 *                   flash:flash,
 *                   view:view,
 *                   readTemplatefile:readTemplatefile,
 *                   render:render
 *                   )
 *
 * <h2>Schnittstelle</h2>
 * @param domain            Domainobjekt
 *                          Der Domain wird das Datenbankhandle und von der Domain noch bentigte Parameter bergeben.
 * @param request           Requestobjekt
 * @param addFunction       Die Frameworkfunktion zum Hinzufgen von Linkeintraegen
 * @param flash             Die Frameworkfunktion zur Bearbeitung der Flashingfunktion
 * @param readTeplatefile   Die Frameworkfunktion zum Templatefile lesen
 * @param view              Die Frameworkfunktion zum anzeigen der Viewertemplates
 * @param render            Die Frameworkfunktion zum Direkten Anzeigen von Text aus dem Controller
 *
 * @result                  true/false        Status der Operation
 *
 * <h2>Methoden</h2>
 * lastError()       liefert eine Fehlermeldung wenn result != true
 * setEditFilename   Name der Datei welche den Maskenviewer enthlt
 * setListFilename   Name der Datei welche den Listviewer enthllt
 *
 *
 *
 * 
 * </pre></tt>
 * </abstract>
 */

// ----------------------------------------------------------------
class CRUD {
// ----------------------------------------------------------------
   //
   // .--update--.
   // -- wann----+-wer+-- was -------------------------------------
   // 09.11.2007 | ny | erstellt
   // 01.02.2008 | ny | "fields" eingebaut
   //            |    | Liefert der View eine Liste von Feldinhalten
   //            |    | diese werden in Abhngikeit von action gesetzt
   //            |    | 
   //            |    | 
   //            |    | 
   //
   //

   // Attribute
   // ---------
   String command                        // Commando
   def    request                        // request Objekt
   def    response                       // response Objekt
   def    domain                         // Domain
   def    addFunction
   def    flash
   def    view
   def    readTemplatefile
   Map    listBindings = [:]             // Bindings fuer Listtemplate
   Map    editBindings = [:]             // Bindings fuer Editmaske
   def    render
   def    reload                         // Seite nochmal laden (z.B. nach Fehler)
   def    controller                     // Referenz auf aufrufenden Controller

   def    taglib

   def    USE_STANDARD_FUNCTIONS = true  // Zeigt an ob bei den Masken die Standardfunktionen gesezt werden
                                         // wir diese Variable auf fals gesetzt werden keine Funktione angezeigt.
 
   //! HINT: Die Pfade der Datein werden aus dem Path gebildet und muessen somit
   //!       im selben Verzeichnis wie der Controller residieren
   //!
   String editFilename = 'edit.tpl'      // Name des Edit-Viewer Datei
   String listFilename = 'list.tpl'      // Name des List-Viewer Datei

   String afterSave = 'list'             // Weleches View soll nach geglcktern
                                         // Save aufgerufen werden
   List errorFields = []
   String error = ''
   Map    fields = [:]                   // Felder der Domain (von CRUD aufbereitet)

   /**
    * In Meta kann der ID Feldname in der Domain definiert sein
    * Dieser wird zurckgeliefert.
    * Standardwert is 'id'
    *
    * usage:   myFieldId = getIdFieldName(doamin)
    * 
    * @param   Domainklasse
    * 
    * @return  Name des Kennungsfeldes
    * 
    */
   // --- liefer ID Field Name -----------------------------------------
   String getIdFieldName(def domain) {
   // ------------------------------------------------------------------
      //!
      //! In Meta kann der ID Feldname in der Domain definiert sein
      //! Dieser wird zurckgeliefert.
      //! Standardwert is 'id'
      //!
      //! usage:   myFieldId = getIdFieldName(doamin)
      //!
      //!
         // Standard id Fieldname
      String idFieldName = 'id'

      // wurde dieser anders definiert
      if (domain.meta.idfield['domain'] != null) {
         idFieldName = domain.meta.idfield['domain']
         }
      return idFieldName
      }

   /**
    * Die Standardfunktionen fuer die Crudoperationen
    * <ul>
    *  <li>newadd
    *  <li>neu
    *  <li>save
    *  <li>edit
    * </ul>
    * 
    * werden automatisch gesetzt, Diese Funtion kann durch
    * den Schalter USE_STANDARD_FUNCTIONS ausgeschalten werden
    * 
    */

   // --- setzt staddarfunktionen --------------------------------------
   void setStandardFunction() {
   // ------------------------------------------------------------------
      //
      // in Abhaengigkeit der Cgi Variable "action"
      // werden Standardfunktionen (addFunktion )gesezt
      //
      //
      if (!USE_STANDARD_FUNCTIONS) return

      switch (taglib.cgiparam(name:'action',nvl:'')) {
         case 'newadd' :
            addFunction(text:'neu',        title:'Neuanlage',cgiparameter:['action=new'],tagid:1005)
            break
         case 'new' :
            addFunction(text:'zur&uuml;ck',title:'Zur&uuml;ck zur Liste',cgiparameter:['action=list'],tagid:1005)
            break
         case 'save' :
            addFunction(text:'neu',        title:'Neuanlage',cgiparameter:['action=new'],tagid:1005)
            break
         case 'edit' :
            addFunction(text:'zur&uuml;ck',title:'Zur&uuml;ck zur Liste',cgiparameter:['action=list'],tagid:1005)
            break
         default :
            addFunction(text:'neu',        title:'Neuanlage',cgiparameter:['action=new'],tagid:1005)
         }
      }


  /**
   * Liefert eine Liste von Feder der Domain, welche 
   * von CGI bergeben wurde. Dies wird bei Update bentigt
   * um nur die Felder zu beschreiben, welche verndert worden sind.
   * 
   * @return   Liste von Feldnamen
   * 
   */
   // ------------------------------------------------------------------
   List getUsedFields() {
   // ------------------------------------------------------------------
      // 
      // Liste der Felder der Domain, welche im CGI bergeben wurden
      // 
      List fields = []
      // Durchlaufen der Domainfelder
      domain.meta?.fields.each {field->
         if (request.getParameter(field.key) != null) {
            fields.add(field.key)
            }
         }
      return fields
      }

   /**
    * Stelle Felder fuer die view eine Collection von Feldern zusammen
    * 
    * Wenn Aktion (action) Edit werden die Felder
    * aus der Domain sonst aus dem CGI entnommen.
    * 
    */
   // ------------------------------------------------------------------
   void fillFields() {
   // ------------------------------------------------------------------
      
      List usedFields = getUsedFields()

      // wurde ein Fehler indiziert
      //
      switch (taglib.cgiparam(name:'action')) {
         default:
            usedFields.each {fieldname->
               fields[fieldname] = taglib.cgiparam(name:fieldname,nvl:'')
               }
            break
         case 'edit' :
            def id = getIdFieldValue()
            if (!domain.get(id)) {
               throw new Exception("crud::fillFields: Kann Domain (${domain.getClass().getName()}) fuer Tabelle '${domain.tablename}' mit id ${id} Inhalt: '${id}' nicht finden")
               }
            else {
               domain.meta?.fields.each {field->
                  fields[field.key] = domain.getProperty(field.key)?:''
                  }
               }
            break
         }
      }

   /**
    * Kann von GET und POST verwendet werden.
    * Es wird versucht den Querystring als GET
    * zu erhalten. Ist dies erfolgreich, so wird dieser uebergeben.
    * 
    * Kann kein Querystring gefunden werden so wird angenommen,
    * das ein POST vorliegt. Diese Parameter werden zu einem
    * QueryString zusammengebaut und zurueckgegeben.
    * 
    * HINT:
    *    request.getRequestMethod() funktioniert im Jetty nicht richtig.
    *    
    *    
    */
   String getQueryString () {
      List retval = []
      // Wenn als GET Request dann wird 
      // Deser genommen
      if (request.getQueryString() != null) return request.getQueryString()

      // Sonst auslesen der Parameter
      request.getParameterNames().each{p->
         retval.add("${p}=${request.getParameter(p)}")
         }
      return retval.join('&')
      }

   /**
    * Wird eine Fehler gemeldet wird an den URL die Kennung
    * <br>errorfields<br>
    * der action CGI Parameter wird durch den Wert des Parameters
    * "action" ersetzt.
    * 
    * @param   action      Ersetzt den Wert des CGI Parameter action im URL
    * 
    */
   // ------------------------------------------------------------------
   void errorHandling(Map param) {
   // ------------------------------------------------------------------
      // 
      // Fehlerbehandlung
      // 

      param = param==null ? [:] : param

      assert param.get('action',null) != null :  "CRUD::errorHandling parameter action nicht uebergeben"
      String query_string = getQueryString()
      query_string += "&flash=${error}"
      query_string += '&errorfields='+errorFields.join(',')
      query_string = (query_string =~ /action=\w+/).replaceFirst("action=${param.action}")
      query_string = 'start.groovy?'+query_string
      def dispatcher = request.getRequestDispatcher(query_string)
      try {
         dispatcher.forward(request, response)
         }
      catch (Exception e) {
         throw new Exception ("CRUD::errorHandling: \n${query_string}\nException Message ${e.message}\n")
         }
      }

   /** 
    * Gibt es den CGI Parameter 'id' so wird der Inhalt 
    * zurckgeliefert. Wird dieser nicht gefunden.
    * so wird aus der Domain der ID Fieldname entnommen
    * und danach gesucht. Ist dieser Vorhanden, wird
    * der Inhalt zurckgeliefert.
    * 
    * Gibt es auch diesen nicht wird eine Exception geworfen.
    * 
    * @return    Inhalt des ID Feldes der Domain
    * 
    */
   // ------------------------------------------------------------------
   String getIdFieldValue() {
   // ------------------------------------------------------------------
      // 
      // Liefert den Inhalt der ID
      // 
      
      // Existiert ID

      if (taglib.cgiparam(name:'id',nvl:'')!='') {
         return taglib.cgiparam(name:'id',nvl:'')
         }
      else {
         // Wie heisst das ID Field
         String idFieldName = getIdFieldName(domain)
         String idFieldValue = taglib.cgiparam(name:idFieldName)
         return idFieldValue
         }
      }

  /**
   * 
   * Starten der CRUD operationen.
   * 
   * @return  true/false     Wurde eine Fehler gemeldet, wird false zurueckgegben
   *
   */
   // ---- Durchfuehren ------------------------------------------------
   boolean run() {
   // ------------------------------------------------------------------
      //
      // Eien CRUD operation durchfuehren
      //
      assert domain != null : "CRUD Objekt ohne Domain amgelegt!"
      assert request != null : "** CRUD: die Option requst wurde beim Initalisieren nicht uebergben"  

      taglib = new MVC.taglib.Taglib(request:request)

      // Felder
      fillFields()

      listBindings['fields'] = fields
      editBindings['fields'] = fields

      // Kommando abarbeiten
      //
      boolean isOk = true
      setStandardFunction()
      switch (taglib.cgiparam(name:'action',nvl:'')) {

         // Einfuegen
         // ---------
         case 'newadd' :
            // Cgi inhalte prfen
            if (!domain.setFromCgi(request:request)) {
               domain.errors.each{ theError->
                  if (error != '') error += '<br/>'
                  error += theError['field']+' '+theError['message']+' '+theError['value']
                  errorFields.add(theError['field'])
                  isOk = false
                  }
               }
            else {
               // HINT ID nach holen von CGI befuellen, da sonst ueberschrieben
               domain.setProperty(getIdFieldName(domain),domain.nextval()) // naechst hhere Nummer

               if (!domain.save(mode:'insert')) {
                  error = "Fehler beim einfuegen in die Datenbank : "+domain.sqlError
                  if (!domain.lastSql =~ /^\s*$/) error +='<br/>sql: '+domain.lastSql
                  flash(error)
                  isOk = false
                  }
               }

            // Wenn alle OK dann View
            // sonst Fehlerbehandlung aufrufen
            //
            if (isOk) {
               editBindings['filename'] = editFilename
               editBindings['rec'] = domain

               switch (afterSave) {
                  case 'list' :
                     view(filename:listFilename,bindings:listBindings)
                     break
                  case 'edit' :
                     view(filename:editFilename,bindings:editBindings)
                     break
                  default :
                     throw new Exception("CRUD: afterSave hat ungueltigen Wert ${afterSave}")
                     break
                  }
               }
            else
               {
               errorHandling(action:'new')
               return false
               }
            break

         // Ausgabe einer Leermaske zur eingabe
         // -----------------------------------
         case 'new' :
            domain.clear()
            editBindings['filename'] = editFilename
            editBindings['rec'] = domain
            view(filename:editFilename,bindings:editBindings)
            break

         // Schreiben bestehender der Daten
         // -------------------------------
         case 'save' :
            // es gibt zwei Submitbuttons. Diese bestimmen das weitere Vorgehen
            //
            switch (taglib.cgiparam(name:'submitbutton',nvl:'')) {

               // Loeschen eines Datensatzes
               //
               case  'delete' :
                  String idFieldValue = getIdFieldValue()
                  // pruefen ob vorhanden
                  if (!domain.get(idFieldValue)) {
                     error = "kann Datensatz ber Key: '${this.getIdFieldName(domain)}' = '${idFieldValue}' nicht finden"
                     isOk = false
                     }
                  // lschen der Daten
                  if (!domain.delete()) {
                     error = "Fehler beim l&ouml;schen: "+domain.sqlError+'<br/>sql: '+domain.lastSql
                     isOk = false
                     }
                  break

               // Rueckschreiben
               //
               case 'save' :
                  Boolean saveOk = true
                  try {
                     saveOk = domain.setFromCgi(request:request)
                     }
                  catch (Exception ex) {
                     throw new Exception("CRUD::rund action: save exceptionmessage: ${ex.message?:'keine Vorhanden (null)'}")
                     }

                  if (!saveOk) {
                     domain.errors.each{ theError->
                        if (error != '') error += '<br/>'
                        error += theError['field']+' '+theError['message']+' '+theError['value']
                        errorFields.add(theError['field'])
                        }
                     isOk = false
                     }
                  else {
                     // Liste der Felder welche von CGI bergeben
                     // worden sind, Nur diese in's Update bernehmen
                     //
                     List usedFields = getUsedFields()
                     if (!domain.save(mode:'update',include:usedFields)) {
                        error =  "Fehler beim speichern (mode:update) in die Datebank: "+domain.sqlError
                        if ((domain.lastSql?:'') != '') error += '<br/>sql: '+domain.lastSql
                        errorFields.add(error)
                        isOk = false
                        }
                     else {
                        fields.each {key,value->
                           fields[key] = null
                           }
                        listBindings['fields'] = fields
                        editBindings['fields'] = fields
                        }
                     }
                  break
               default :
                  break
               }

            // Wenn alle OK dann View
            // sonst Fehlerbehandlung aufrufen
            //
            if (isOk) {
               editBindings['rec'] = domain
               switch (afterSave) {
                  case 'list' :
                     view(filename:listFilename,bindings:listBindings)
                     break
                  case 'edit' :
                     view(filename:editFilename,bindings:editBindings)
                     break
                  default :
                     throw new Exception("CRUD: afterSave hat ungueltigen Wert ${afterSave}")
                     break
                  }
               }
            else
               {
               errorHandling(action:'edit')
               return false
               }
            break

         case 'show' :
         case 'edit' :
            String idFieldValue = getIdFieldValue()
            if (idFieldValue != '') {
               if (domain.get(idFieldValue)) {
                  editBindings['filename'] = editFilename
                  editBindings['rec'] = domain
                  view(filename:editFilename,bindings:editBindings)
                  }
               else {
                  flash "SCHWERWIEGEND: Kann Datensatz mit Kennung '${taglib.cgiparam(name:'id',nvl:0)}' nicht finden "
                  return true
                  }
               }
            else {
               editBindings['filename'] = editFilename
               editBindings['rec'] = domain
               view(filename:editFilename,bindings:editBindings)
               }
            break

         case 'list' :
            view(filename:listFilename,bindings:listBindings)
            break

         default :
            view(filename:listFilename,bindings:listBindings)
            break
         }
      return isOk
      }

   }