package framework.helper

import java.text.SimpleDateFormat
import java.sql.Date
import java.util.Date
import java.sql.Timestamp

// ---- Classe ----------------------------------------------------
class DateConverter {
// ----------------------------------------------------------------
   //
   // Datumsunwandlung
   //
   //
   // .--update--.
   // -- wann----+-wer+-- was -------------------------------------
   // 06.11.2007 | ny | erstellt
   // 21.11.2007 | ny | Ausgabe von SQL Datumsformat
   //            |    |
   //            |    |
   //            |    |
   //            |    |
   //
   //

   String lastError = ''


   // --------------------------------------------------------------------
   Date fromString(String datum) {
   // --------------------------------------------------------------------
      //
      // Wandelt einen String in eine Datum um
      // Dabei knnen verschiedene Datumsformate verwendet werden
      //
      // Voraussetzungen:
      //    Jahreszahlen sind immer vierstellig anzugeben
      //
      // Formate:
      //    dD.mM.CCYY           1.12.2007
      //    dD.mM.CCYY HH:NN     1.12.2007 14:35
      //    CCYY-MM-DD           2007-01-12
      //    CCYY-MM-DD HH:NN     2007-01-12 13:35
      //
      // @param: datum       Datumsstring
      // @return: gibt ein Datum (java.sql.Date!) zurueck
      //          im Fehlerfall wird null Zurckgeliefert
      //
      //
      //
      //

      java.util.Date theDate
      this.lastError = ''

      if (datum == 'now()') {
         datum = giveAsASCIIDateTime(new Date())
         }

      datum = (datum =~ /^\s*/).replaceAll('')
      datum = (datum =~ /\s*$/).replaceAll('')
      datum = (datum =~ /\s+/).replaceAll(' ')
      datum = (datum =~ /,/).replaceAll('.')


      // Datumsforamtstring erzeugen
      String dFormat
      
         if (datum =~ /^(01|02|03|04|05|06|07|08|09|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31)(01|02|03|04|05|06|07|08|09|10|11|12)\d{2}$/) {
         dFormat = 'ddMMyy'
         }
 
      if (datum =~ /^(01|02|03|04|05|06|07|08|09|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31)(01|02|03|04|05|06|07|08|09|10|11|12)\d{2}\d{4}$/) {
         dFormat = 'ddMMyyHHmm'
         }

      if (datum =~ /^\d{1,2}\.\d{1,2}\.\d{4}$/) { // dD.mM.ccyy
         dFormat = 'd.M.yyyy'
         }

      if (datum =~ /^\d{8}$/) { // dD.mM.ccyy
         dFormat = 'yyyyMMdd'
         }

      if (datum =~ /^\d{12}$/) { // dD.mM.ccyy

         dFormat = 'yyyyMMddHHmm'
         }

      if (datum =~ /^\d{1,2}\.\d{1,2}\.\d{4} \d{1,2}\.\d{2}$/) { // dD.mM.ccyy hh.nn
         dFormat = 'd.M.yyyy k.m'
         }

      if (datum =~ /^\d{1,2}\.\d{1,2}\.\d{4} \d{1,2}:\d{2}$/) { // dD.mM.ccyy hh.nn
         dFormat = 'd.M.yyyy k:m'
         }

      if (datum =~ /^\d{4}-\d{2}-\d{2}$/) { // CCYY-MM-DD
         dFormat = 'yyyy-MM-dd'
         }

      if (datum =~ /^\d{4}-\d{2}-\d{2} \d{1,2}:\d{2}$/) { // CCYY-MM-DD hh.nn
         dFormat = 'yyyy-MM-dd k:m'
         }

      if (datum =~ /^\d{1,2}:\d{2}$/) { // hh.mm
         dFormat = 'k:m'
         }


      SimpleDateFormat df = new SimpleDateFormat(dFormat, Locale.GERMAN);
      try {
         // Schaltet die nachlaessige Umwandlung aus
         df.setLenient(false)
         // Umwandlung
         theDate = df.parse(datum)
         return new java.sql.Timestamp(theDate.getTime())
         }
      catch (Exception e){
         lastError = e.message
         throw new Exception("Datenumwandlungsfehler bei ${datum} ${e.message}")
         return null
         }
      }


   // --------------------------------------------------------------------
   String giveAsASCIIDate (datum) {
   // --------------------------------------------------------------------
      //
      // Gibt ein Datum als ANSI-Datum CCYY-MM-DD zurck
      //
      // @param: Datum
      // @return: String mit Ansidatum
      //
      //
      this.lastError = ''
      SimpleDateFormat df = new SimpleDateFormat('yyyy-MM-dd', Locale.GERMAN);
      try {
         return df.format(datum)
         }
      catch (Exception e){
         lastError = e.message
         return null
         }
      }


   // --------------------------------------------------------------------
   String giveAsASCIIDateTime (datum) {
   // --------------------------------------------------------------------
      //
      // Gibt ein Datum als ANSI-Datum CCYY-MM-DD HH:NN zurck
      //
      // @param: Datum
      // @return: String mit Ansidatum
      //
      //
      this.lastError = ''
      SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd kk:mm", Locale.GERMAN);
      try {
         return df.format(datum)
         }
      catch (Exception e){
         lastError = e.message
         return null
         }
      }


   // --------------------------------------------------------------------
   String giveAsGermanDate (datum) {
   // --------------------------------------------------------------------
      //
      // Gibt ein Datum als ANSI-Datum CCYY-MM-DD zurck
      //
      // @param: Datum
      // @return: String mit Ansidatum
      //
      //
      this.lastError = ''
      SimpleDateFormat df = new SimpleDateFormat('d.M.yyyy', Locale.GERMAN);
      try {
         return       df.format(datum)
         }
      catch (Exception e){
         lastError = e.message
         return null
         }
      }

   // --------------------------------------------------------------------
   String giveAsGermanDateTime (datum) {
   // --------------------------------------------------------------------
      //
      // Gibt ein Datum als ANSI-Datum CCYY-MM-DD zurck
      //
      // @param: Datum
      // @return: String mit Ansidatum
      //
      //
      this.lastError = ''
      
      SimpleDateFormat df = new SimpleDateFormat('d.M.yyyy HH:mm', Locale.GERMAN);

      try {
         return df.format(datum)
         }
      catch (Exception e){
         lastError = e.message
         return null
         }
      }
   
   // --------------------------------------------------------------------
   String SQLJahresanteil (String kennzeichen,String jahr) {
   // --------------------------------------------------------------------
      // 
      // Im der Taglib werden Jahresanteilkennzeichen (jahresteile) definiert
      // Diese Routine liefert eine SQL Whereklauses welche ein beliebiges
      // Datum dagegen geprft werden kann.
      // 
      // Es wird "between datum-von and datum-bis" erzeugt.
      // 
      // @param   Kennzeichen z.B.: M1, Q1 aber auch '1J' (ganzes Jahr)
      // @param   zu Pruefendes Jahr
      // 
      // @result  SQL-Where Klauses
      // 
      // usage:   SQLJahresteil('M1','2007)
      // 

      String result = ''
      switch (kennzeichen) {
         
         case '1J' : 
            result = "between to_date('1.1.${jahr}','dd.mm.yyyy') and to_date('31.12.${jahr}','dd.mm.yyyy')"
            break
         case '1HJ' : 
            result = "between to_date('1.1.${jahr}','dd.mm.yyyy') and to_date('30.6.${jahr}','dd.mm.yyyy')"
            break
         case '2HJ' : 
            result = "between to_date('1.7.${jahr}','dd.mm.yyyy') and to_date('31.12.${jahr}','dd.mm.yyyy')"
            break
         case 'Q1' : 
            result = "between to_date('1.1.${jahr}','dd.mm.yyyy') and to_date('31.3.${jahr}','dd.mm.yyyy')"
            break
         case 'Q2' : 
            result = "between to_date('1.4.${jahr}','dd.mm.yyyy') and to_date('30.6.${jahr}','dd.mm.yyyy')"
            break
         case 'Q3' : 
            result = "between to_date('1.7.${jahr}','dd.mm.yyyy') and to_date('30.9.${jahr}','dd.mm.yyyy')"
            break
         case 'Q4' : 
            result = "between to_date('1.10.${jahr}','dd.mm.yyyy') and to_date('31.12.${jahr}','dd.mm.yyyy')"
            break
         case 'M1'  : 
         case 'M2'  : 
         case 'M3'  : 
         case 'M4'  : 
         case 'M5'  : 
         case 'M6'  : 
         case 'M7'  : 
         case 'M8'  : 
         case 'M9'  : 
         case 'M10' : 
         case 'M11' : 
         case 'M12' : 
            String  monat = (kennzeichen =~ /^M/).replaceFirst('')
            result = "between to_date('1.${monat}.${jahr}','dd.mm.yyyy') and last_day(to_date('1.${monat}.${jahr}','dd.mm.yyyy'))"
            break
         default :
            throw new Exception ("DateConverter:: Ungueltiges Kennzeichen ${kennzeichen}")
            break
         }
      return result
      }

   } // END OF CLASS
