// ////////// Navigateie ////////////////////////////
//
//
//
//
// -- wann --+-wer-+-- was ----------------------------------------------
// 12.10.2007|  ny | estellt
// 23.10.2007|  ny | injectLink benannte Parameter
// 08.11.2007|  ny | Einbau Rechteverwaltung
// 22.12.2007|  ny | Reduntate Controllerfunktion entfern
// 30.12.2007|  ny | Refakturierung in Navigate
// 31.08.2008|  ny | Einbau von Backlinkbar  
//           |     | 
//           |     | 
//           |     | 
//

package framework.helper

import java.util.regex.Matcher
import java.util.regex.Pattern
import MVC.taglib.Taglib

/**
 * <b>Behandelt die Erstellung der Navigabtionsleisten</b>
 * 
 * <pre>
 * Die Klasse liest das Steuerfile <tt>mframe.xml</tt> und 
 * filtert die Menueeintr&auml;ge heraus.
 * Ein Menueeintrag besteht aus:
 * <ul>
 *  <li> entry Option: path<br/>
 *       der Pfadeintrag kennzeichnet den Eintrag eindeutig.</li>
 *  <li> controller Option: path<br/>
 *       Kennzeichet das Skript, welches beim anklicken aufgerufen wird.
 *       Wird kein Controller spezifiziert, so erfolgt keine Aktion.
 *       Der Controller kann relativ oder absolut notiert werden
 *       <ul>
 *        <li>relativ<br/>
 *            es steht nur ein Controllername ohne vorlaufenden Schr&auml;gstring
 *            Der Controller wird im Verzeichnis, welches durch entry-path beschrieben
 *            wird gesucht.</li>
 *       <li>absolut<br/>
 *           Der Controllernameeintrag (path) beginnt mit einem Sch&auml;gstrich
 *           Das Controllerskript wird absolut, beginnend mit ./classes + Eintrag gesucht
 *       </ul>
 * <li> text <br/>
 *      Anzeigetext im Menue
 * <li> title<br/>
 *      Popuphelp des Anzeigetextes
 * <li> display [yes|no|disabled]<br/>
 *      der Eintrag wird je nach Schalterstellung angezeigt
 * <li> rights<br/>
 *      Anzeigen des Eintrags je nach Rechte.
 *      In der Sessionvariable recht wird eine Liste von Rechten, welche
 *      der eingeloggte Benutzer besitzt abgelegt.
 *      Diese wird mit den Rechten des Eintrags verglichen. Gibt es mindestens
 *      eine &Uuml;bereinstimmung wird der Eintrag angezeigt.
 * </ul>
 * 
 * 
 */
// === Navigationsleisten =========================================
class Navigate {
// ================================================================
   //
   // .--update--.
   // -- wann----+-wer+-- was -------------------------------------
   // 13.09.2007 | ny | erstellt
   // 19.11.2008 | ny | Schaltflchen knnen disabled dargestellt werden
   //            |    |
   //            |    |


   def xml                  // Haelt die XML-Struktur der Konfiguration
   String Server            // Servername, wird ermittelt von aufgerufnen Programm
                            // und im Construtor bergeben

   List otherEntries = []   // Haelt zustzliche Linkbareintraege

   static final String VERSION = '0.30E003'
   String lastControllerFileName = ''
   String approot = ''
   String path = ''
   def request

   List   myRights = []     // Liste der Rechte, wird von aussen gesetzt und
                            // kommt normalerweise aus der Session

   // --- addLinkbar -----------------------------------------------------
   def addFunction = {param ->
   // --------------------------------------------------------------------
      //
      // Es knnen dynamisch Linkbars zinzugefgt werden.
      //
      if (param == null) param = [:]
      otherEntries.add([
         path:param.path,
         text:param.text,
         title:param.title,
         cgiparameter:param.cgiparameter,
         url:param.url,
         target:param.target,
         display:param.display?:'yes'
         ])
      }

   // --- liefert den Aktuellen Server------------------------------------
   def getServerobject = {
   // --------------------------------------------------------------------
      //
      // Serverkonfiguration ermitteln
      //
      def Server = xml.config.server.find{it.'@name' == this.Server}
      assert Server != null : "In Config-XML ist fuer Server '${this.Server}' kein Eintrag vorhanden"
      return Server
      }

   //--- Schneidet letzten Pfadeintrg ab ---------------------------------
   def getPrevPath = {path ->
   // --------------------------------------------------------------------
      //
      // Abschneiden aktuellen Pfadeintrag
	  //
	  // usage: getPrevPath('/root/eins/zwei')
	  //        -> '/root/eins'
	  //
	  // wird der oberste Pfadeintrag bergeben, so wird
	  // ein einzelner Schraegstrich zurueckgegeben
	  //    '/root' --> '/'
	  //
	  String Path = path
	  Path = (Path =~ /\/\w+$/).replaceFirst('')
	  if (Path == '') Path = '/'
	  return Path
	  }

   //--- injzieren Inforamtionen in Link ----------------------------------
   def injectLink = {param ->
   //----------------------------------------------------------------------
      //
      // die Informationen eines Menueeintrages werden
      // in das Template injiziert
      //

      String curPath = param.path
      String text    = param.text
      String title   = param.title
      String template = param.template
      String tagid    = param.tagid?:''
      String display  = param.display?:'yes'

      boolean isControllerfound = false

      String sApproot = getServerobject().approot[0].'@value'
      String sEntryPoint = getServerobject().entrypoint[0].'@value'
      String target = ''

      String sUrl = '' 

      if (param.url != null) {
         sUrl = param.get('url')
         }
      else {
         sUrl = "/${sApproot}/${sEntryPoint}?path=${curPath}"
         }

      if (param.cgiparameter != null) {
         param.cgiparameter.each {cgiparam->
            sUrl += "&${cgiparam}"
            }
         }

      if (param.target != null) target = """target="${param.target}" """

      switch (display) {
         case 'yes' :
            // Injection von Text, Url und Alternativtext
            //
            template = (template =~ /<text\/>/).replaceFirst(text)
            template = (template =~ /<url\/>/).replaceFirst(sUrl)
            template = (template =~ /<title\/>/).replaceFirst(title)
            template = (template =~ /<target\/>/).replaceFirst(target)
            template = (template =~ /<tagid\/>/).replaceAll(tagid)
            break
         case 'no' :
            // nop
            break
         case 'disabled' :
            template = """<span class="disabled-link" title="Diese Funktion ist im Augenblick nicht verf&uuml;gbar">${text}</span>"""
            break
         default :
            throw new Exception("Eine bislang unbekannte Display Option '${display}' stellt sich vor")
            break
         }
      return "${template}\n"
      }

   //--- Entry auswhlen --------------------------------------------------
   def myEntry = { path ->
   //----------------------------------------------------------------------
      //
      // Gibt einen Entry basieren auf einen Path zurueck
      //
      // uasge: entry = myEntry('/root/eins')
      //
      //

      def entry = null
      try {
         entry = xml.menu[0].entries[0].findAll{it.'@path' == path}
         }
      catch (Exception e) {
         entry = null
         }
      return (entry.size() == 0) ? null : entry
      }


   //--- Erzeugen eines Linkbars ------------------------------------------
   def getLinkbar = { path,rightslist ->
   //----------------------------------------------------------------------
      //
      // Alle 'Unterpfade' des aktuellen Pfades
      //
      // usage: getLinkbar('/root/eins')
      //
      String Path = path.equals('/') ? '' : path  // stelle '/' fuer regex als leeren Pfad dar
      String Bar = ''                             // Sammelbuffer
      
      Integer tagid = 0                           // nummerieren der Tagid

      // Konstanten aus Definitionsfile holen

      String sLinkbarDelimeter = xml.menu[0].linkbar[0].delimeter[0].'@value'
      String sLinkbarTemplate = xml.menu[0].linkbar[0].template[0].text()

      // aus dem XML alle Menueentrys untersuchen
      // Finde alle elemente, welche 'unter' dem angegebenen Pfad sind
      //
      xml.menu[0].entries[0].findAll{Entry->Entry.'@path' =~ /^$Path\/\w+$/}.each {Entry->

         // Anzeigekennezeichen
         String display = Entry.display[0]?.'@value'
         String userRights = Entry.rights?.'@value' == null ? '' : Entry.rights?.'@value'[0]

         // Wurde fuer diesen Eintrag rechte definiert
         if (Entry.rights?.'@value') {
            // zerlegen in Liste
            List myRights = userRights.split(',')

            // wir suchen auf Positov
            boolean found = false
            rightslist.each {recht->
               if (myRights.grep(recht)) {
                  found = true
                  return
                  }
               }
            // nicht gefunden display auf NEIN
            if (!found) display = 'no'
            }

            switch (display?:'yes') {
                case 'yes' :
                  Bar += injectLink(path:Entry.'@path',
                                   text:Entry.text[0] == null ? '' : Entry.text[0].'@value',
                                   title:Entry.title[0] == null ? '' : Entry.title[0].'@value',
                                   controller:Entry.controller[0] == null ? '' : Entry.controller[0].'@path',
                                   template:sLinkbarTemplate,
                                   )
                  break
               case 'no' :
                  // NOP
                  break
               case 'disabled' :
                  Bar = (Entry.text[0] == null ? '' : Entry.text[0].'@value')+Bar
                  break
               default :
                  throw new Exception("Eine bislang unbekante Displayoption '${display}' stellt sich vor.")
                  break
               }
         }

      // Zusatzfunktionen einbauen
      //
      otherEntries.each {other->
            Bar += injectLink(path:other.path,
                             text:other.text,
                             title:other.title,
                             url:other.url,
                             cgiparameter:other.cgiparameter,
                             target:other.target,
                             template:sLinkbarTemplate,
                             tagid:other.tagid?:"${tagid++}",
                             display:other.display
                             )
         }
      return Bar
      }


   // --- Erzegen des Backlinkbars --------------------------------------------
   def getBacklinkbar = { path,rightslist ->
   // -------------------------------------------------------------------------
      //
      // Gibt auf Grund eines gegebenen Pfades z.B.: '/root/eins/zwei'
      // einen Liste von Rcksprungadressen (/root, /root/eins, /root/eins/zwei') zurck
      //
      //  usage getBacklinkbar(path)
      //
      //

      String Path = path // Path umspeichern
      String Bar = ''    // Buffer

      // Wenn TOP-LEVEL (/) dann kein Backlink...
      // Sollte eigentlich nicht vorkommen
      //
      if (Path == '/') {
    	 return ''
         }

      // Konstanten aus Definitionsfile holen
      String sLinkbarDelimeter = xml.menu[0].backlink[0].delimeter[0].'@value'
      String sLinkbarTemplate = xml.menu[0].backlink[0].template[0].text()

      // Pfade Zerlegen
      //      
      while (Path.size() > 0) {
         String EntryTmpl = sLinkbarTemplate  // Template in Arbeitsspeicher
         xml.menu[0].entries[0].findAll{Entry->Entry.'@path'.equals(Path)}.each {Entry->

            // Anzeigekennezeichen
            String display = Entry.display[0]?.'@value'
            String userRights = Entry.rights?.'@value' == null ? '' : Entry.rights?.'@value'[0]

            // Wurde fuer diesen Eintrag rechte definiert
            if (Entry.rights?.'@value') {
               // zerlegen in Liste
               List myRights = userRights.split(',')

               // wir suchen auf Positov
               boolean found = false
               rightslist.each {recht->
                  if (myRights.grep(recht)) {
                     found = true
                     return ''
                     }
                  }
               // nicht gefunden display auf NEIN
               if (!found) display = 'no'
               }
            // wird display gewuenscht
            switch (display?:'yes') {
                case 'yes' :
                  // Trenner einfgen wenn nicht 1. Durchlauf
                  if (Bar.size() > 0) Bar = sLinkbarDelimeter + Bar // Trenner hinzufgen, wenn nicht 1.Eintrag
                  // Link am Anfang einfgen da von Hinten aufgeschluesselt wird
                  Bar = injectLink(path:Entry.'@path',
                                   text:Entry.text[0] == null ? '' : Entry.text[0].'@value',
                                   title:Entry.title[0] == null ? '' : Entry.title[0].'@value',
                                   controller:Entry.controller[0] == null ? '' : Entry.controller[0].'@path',
                                   template:sLinkbarTemplate)+Bar
                  break
               case 'no' :
                  // NOP
                  break
               case 'disabled' :
                  Bar += Entry.text[0] == null ? '' : Entry.text[0].'@value'
                  break
               default :
                  throw new Exception("Eine bislang unbekante Displayoption '${display}' stellt sich vor.")
                  break
               }
            }
         Path = (Path =~ /\/\w+$/).replaceFirst('')
         }
      return Bar
      }

   //--- Defaultmenutemplate ----------------------------------------------
   def defaultTemplate = {
   //----------------------------------------------------------------------
      //
      // liefert das Default Template aus XML
      //
      // usage: defaultTemplate()
      //
      //
      return xml.templates[0].default_menu[0].text()
      }


   //--- Defaultmenutemplate ----------------------------------------------
   def defaultStylesheet = {
   //----------------------------------------------------------------------
      //
      // liefert das Default Stylesheet
      //
      // usage: defaultStylesheet()
      //
      //
      return getServerobject().defaultstylesheet.'@href'
      }

   //--- Defaultmenutemplate ----------------------------------------------
   def defaultLogo = {
   //----------------------------------------------------------------------
      //
      // liefert das Default Logo
      //
      // usage: defaultLogo()
      //
      //
      return getServerobject().logo.'@small'
      }

   //--- Controllerfilepath -----------------------------------------------
   def buildFilePath = {filepath->
   //----------------------------------------------------------------------
      //
      //
      //
      String theFilepath = ''
      String mvcEntrypoint = this.getServerobject().mvcentrypoint[0]?.'@value'
      if (filepath =~ /^\//) { // Absoluter Filepath
         theFilepath = "${approot}WEB-INF/classes/${filepath}"
         }
      else {
         theFilepath = "${approot}WEB-INF/classes/${mvcEntrypoint}/${filepath}"
         }
      theFilepath = (theFilepath =~ /\/\//).replaceAll('/')
      theFilepath = (theFilepath =~ /[\\]/).replaceAll('/')
      return theFilepath
      }


   //---read controllerfile -----------------------------------------------
   def readControllerfile = {filepath->
   //----------------------------------------------------------------------
      //
      // einlesen Controllerfile
      // wird das File nicht gefunden, so wird ein Leerstring zurckgeliefert
      //
      // ussage: readControllerfile(<releativer-filepath>)
      //
      //
      
      String filePath = this.buildFilePath(filepath)
      def f    // Filehandle

      this.lastControllerFileName = filePath
      f = new File(filePath)

      // Wenn Controller nicht vorhanden
      // wird Leerstring zurckgeliefert
      //
      if (!f.exists())  return ""


      try {
         return f.text
         }
      catch (Exception e) {
         return ""
         }
      }

   //--- Controllersource -------------------------------------------------
   def menu = {path,rightslist ->
   //----------------------------------------------------------------------
      //
      // Gibt die Menueintrge zurck
      //
      // @param: rightslist      Liste der Benutzerrechte
      //

      String thePath = path == null ? xml.menu[0].config[0].start[0].'@path' : path

      /**
       * wurde ueber CGI der Parameter 'backlinkbar'
       * gesetzt wird dieser verwendet.
       * 
       */ 
      String sBacklinkbar = this.getBacklinkbar(thePath,rightslist)

      /**
       * Vorrangreglen
       *  1) CGI-Parameter backlinkbar
       *  2) Errechne Backlink aus Path
       */
       if (request.getParameter('backlinkpath') != null) {
          sBacklinkbar = this.getBacklinkbar(request.getParameter('backlinkpath'),rightslist)
         }

      String sLinkbar     = this.getLinkbar(thePath,rightslist)
      String sTemplate    = this.defaultTemplate()

      sTemplate = (sTemplate =~ /\<linkbar\/>/).replaceFirst(sLinkbar)
      sTemplate = (sTemplate =~ /\<backlinkbar\/>/).replaceFirst(sBacklinkbar)
      return sTemplate
      }

   } // END OF CLASS