/**
 * Die Applikation wird ueber die CGI Variable action gesteuert.
 * Diese wird von den Viewern gesetzt. Je nach Inhalt der Varible
 * werden andere Aktionen angestossen.<br>
 * <h1>Aktionen</h1>
 * <ul>
 *  <li>show<br>Anzeigen einer Maske ohne Editiermglichkeit<br></li>
 *  <li>edit<br>Anzeigen einer Maske mit vorbefllten Werten</li>
 *  <li>list<br>Anzeigen des Listviewers</li>
 * </ul>
 *  
 * <b>HINT</b><br>
 * wird im CGI keine action variable gesetzt, wird "list" angenommen 
 *  
 *  
 * .--update--.
 *  -- wann----+-wer+-- was -------------------------------------
 *  29.08.2008 | ny | Einabu von Zusaetzlichen Schaltflaechen 
 *             |    | Siehe taglib gridrow()
 *  11.09.2008 | ny | Einbau von WorkFlow
 *  12.09.2008 | ny | Anpassung
 *  16.09.2008 | ny | doDb: Beim Insert P-Key loeschen
 *             |    | 
 *             |    | 
 *  
 * Grundstzlich werden zwei unterschiedliche 
 * Viewertypen angeboten:
 * <ul>
 *  <li>Editviewer <br>Maske</li>
 *  <li>Listviewer <br>Paginiert Liste von Datenstzen</br></li>
 * </ul>
 * 
 * <h1>Detailsteuerung</h1>
 * Die AKtionsoptionen werden mittels benannten Submitbuttons gesteuert:
 * z.B. &lt;input type="submit" name="submitbutton" value="save"&gt;.
 * 
 * Zu jeder Maske knnen mehere Submitbuttons definiert werden und so 
 * unterschiedlich Reaktonen der Applikation hervorgerufen werden.
 * 
 * <h1>Fehlerbehandlung</h1>
 * Tritt whrend der Abarbeitung ein Fehler auf, wird eine Ausnahmebedingung
 * geworfen.
 * 
 * Beim Erzeugen der Klass wird eine Referenz auf den Controller mitgegeben
 * 
 * <h1>Parameter</h1>
 * @param   controller     Referenz auf den Controller
 * 
 * <h1>berschreiben von Vorgabewerten</h1>
 * Es werden von der Klasse bestimmte Werte angenommen.
 * Die Klasse untersucht den Controller, ob diese Werte berschrieben werden
 * sollen. Im Controller werden Vorgabewert in der Syntax:
 *  <tt>this.name = 'Wert' </tt>
 * geschrieben
 * 
 * 
 * <h1>WebFlow</h1>
 * Die Funktion ermglicht eine Weiterleitung des Flows
 * in eine anderen als die vorgegbene View.
 * Der Flow wird bei der Konstruktion des Viewhandlers deklariert:
 *
 * <pre>
 *       Viewhandler crud = new Viewhandler(
 *        controller:this,
 *        action:'grid',
 *        domain:new BuchungDomain(
 *                           db:db.dbh,
 *                           autor:request.getSession().getAttribute('personalnummer')?.toInteger()
 *                           ),
 *        viewBindings:bindings,
 *        WebFlow:[
 *             (Viewhandler.SUBMITBUTTON_SAVE):['fahrtenbucheintrag.tpl',handleFahrtenbuch]
 *             ]
 *        )
 * </pre>
 * 
 * Der Flow reagiert auf die Speicheroperation
 * <ul>
 *  <li>new</li>
 *  <li>save</li>
 *  <li>delte</li>
 *  <li>edit</li>
 *  <li>newedit</li>
 * </ul>
 * 
 * Fuer jede dieser Operationen kann ein eigener Eintrag vorgenommen werden.
 * Der Eintrag kann in zwei unterschiedlichen Arten erfolgen.
 * Wird ein der nchste Viewer als String uebergeben wird nach geglckter Datenbankoperation 
 * Der Viewer aufgerfuen.<br>
 * Alternativ kann auch eine Liste mit zwei Elementen uebergeben werden.<br />
 * Das erste Element ist der Dateiname des Viewer, das zweite element ist 
 * ein Closure, welches unter dem Namen callback im Viewer angesprochen werden kann.
 * 
 * <h2>Beispiel eines Viewer mit Closure Aufruf
 * <pre>
 * <h1 class="blockname">Eintrag in Fahrtenbuch</h1>
 * 
 * <%
 *    if (reise != null) {
 *       out<<sview(filename:'/root/reise/reisekopf_short.tpl',bindings:[reise:reise])
 *       }
 * 
 *    callback.call()
 * %>
 * 
 * 
 * </pre>
 */


package framework.helper

import framework.helper.Dialog

// =====================================================================
class Viewhandler {
// =====================================================================

   // KONSTANTEN
   // ----------

   static final String EDITACTION = 'edit'
   static final String SAVEACTION = 'save'
   static final String LISTACTION = 'list'
   static final String SHOWACTION = 'show'
   static final String GRIDACTION = 'grid'

   static final String SUBMITBUTTON_NEW      = 'new'
   static final String SUBMITBUTTON_SAVE     = 'save'
   static final String SUBMITBUTTON_DELETE   = 'delete'
   static final String SUBMITBUTTON_EDIT     = 'edit'
   static final String SUBMITBUTTON_NEWEDIT  = 'newedit'

   static final String DEFAULT_EDITVIEWER    = 'edit.tpl'
   static final String DEFAULT_LISTVIEWER    = 'list.tpl'
   static final String DEFAULT_GRIDVIEWER    = 'grid.tpl'

   static final String DBOPERATION_UPDATE    = 'update'
   static final String DBOPERATION_INSERT    = 'insert'
   static final String DBOPERATION_DELETE    = 'delete'

   static final Boolean SAVE_OK              = true
   static final Boolean SAVE_INVALID         = !SAVE_OK

   // Layoutformat
   static final String GRIDLAYOUT            = 'grid'
   static final String MASKLAYOUT            = 'mask'
   static final String USERLAYOUT            = 'user'

   // Beim uebernehmen der Daten aus dem CGI
   // wird bei Insert in die Datenbank das Keyfeld 
   // nicht in die Liste uebernommen.
   static final Boolean DEFAULT_REMOVE_KEY_ONINSERT = true
   static final Boolean NO_REMOVE_KEY_ONINSERT = false

   // KLASSENVARIABLE
   // ---------------
   def      controller           = null
   String   editview             = DEFAULT_EDITVIEWER
   String   listview             = DEFAULT_LISTVIEWER
   String   gridview             = DEFAULT_GRIDVIEWER
   Map      viewBindings         = [:]
   def      domain               = null
   String   viewAftersave        = DEFAULT_LISTVIEWER
   String   viewAfterdelete      = DEFAULT_LISTVIEWER
   Boolean  removeKeyOnInsert    = DEFAULT_REMOVE_KEY_ONINSERT
   String   layout               = GRIDLAYOUT

  /**
   * Loeschbesttigung anfordern 
   */
   static final Boolean GRID_COFIRM_ON_DELETE  = true
   Boolean  gridConfirmOnDelete  = false
   def WebFlow

   // Sonstige Variable
   // -----------------

   private List     errorfields    = []
   private String   error          = ''
   private String   nextview       = editview
   private String   path           = ''
   private String   action         = ''
   private String   submitbutton   = ''
   private Map      fields         = [:]
   private Boolean  saveOk         = SAVE_OK 
   private def      lastInsertIDValue
   

  /**
   * Ging speichern gut...
   */
   Boolean hasError() {
      return !saveOk
      }


   /**
    * In Meta kann der ID Feldname in der Domain definiert sein
    * Dieser wird zurckgeliefert.
    * Standardwert is 'id'
    *
    * usage:   myFieldId = getIdFieldName(doamin)
    * 
    * @param   Domainklasse
    * 
    * @return  Name des Kennungsfeldes
    * 
    */
   String getIdFieldName(def domain) {
      //
      // In Meta kann der ID Feldname in der Domain definiert sein
      // Dieser wird zurckgeliefert.
      // Standardwert is 'id'
      //
      // usage:   myFieldId = getIdFieldName(doamin)
      //
      //

         // Standard id Fieldname
      String idFieldName = 'id'

      // wurde dieser anders definiert
      if (domain.meta.idfield['domain'] != null) {
         idFieldName = domain.meta.idfield['domain']
         }

      return idFieldName
      }


   /** 
    * Gibt es den CGI Parameter 'id' so wird der Inhalt 
    * zurckgeliefert. Wird dieser nicht gefunden.
    * so wird aus der Domain der ID Fieldname entnommen
    * und danach gesucht. Ist dieser Vorhanden, wird
    * der Inhalt zurckgeliefert.
    * 
    * Gibt es auch diesen nicht wird eine Exception geworfen.
    * 
    * @return    Inhalt des ID Feldes der Domain
    * 
    */
   // ------------------------------------------------------------------
   String getIdFieldValue() {
   // ------------------------------------------------------------------
      // 
      // Liefert den Inhalt der ID
      // 
      
      // Existiert ID

      if (controller.cgiparam(name:'id',nvl:'') != '') {
         return controller.cgiparam(name:'id',nvl:'')
         }
      else {
         // Wie heisst das ID Field
         String idFieldName = getIdFieldName(domain)
         String idFieldValue = controller.cgiparam(name:idFieldName)
         return idFieldValue
         }
      }

  
  /**
   * Liefert eine Liste von Feder der Domain, welche 
   * von CGI bergeben wurde. Dies wird bei Update bentigt
   * um nur die Felder zu beschreiben, welche verndert worden sind.
   * 
   * HINT:
   *  Wenn in der Domain die Option cgi mit checkbox gesetzt wurde
   *  wird auf jeden fall das Feld als benutzt gesetzt, da nicht vorhanden
   *  als falsch gewertet wird.
   *  
   * @return   Liste von Feldnamen
   * 
   */
   // ------------------------------------------------------------------
   List getUsedFields() {
   // ------------------------------------------------------------------
      // 
      // Liste der Felder der Domain, welche im CGI bergeben wurden
      // 
      List fields = []
      // Durchlaufen der Domainfelder
      domain.meta?.fields.each {field->
         if (controller.request.getParameter(field.key) != null) {
            fields.add(field.key)
            }
         else {
            // mit der cgi Option knnen 
            // cgi Spezifische Verarbeitungen 
            // durchgefhrt werden
            // 
            if (field.value?.cgi != null) {
               switch (field.value.cgi['type']) {
                  case 'checkbox' : 
                     fields.add(field.key)
                     break
                  default :
                     throw new Exception ("DataAccessObject:: bei Feld '${field.key}' ungueltige cgi option '${field.value.cgi['type']}'")
                     break
                  }
               }
            }

         /**
          * in den Metadaten kann die Option includefields gesetzt werden.
          * Dies bewirkt, dass das Feld auf jeden Fall in die Fieldlist bernommen wird.
          * Dies ist notwendig, wenn in der Domainklasse Feldinhalte gesetzt werden, dies
          * aber nicht im CGI uebergeben werden
          */
         if (domain.meta?.includefields != null) {
            domain.meta?.includefields.each{fld->
               if (!fields.grep(field.key)) fields.add(field.key)
               }
            }

         }
      return fields
      }
   
   /**
    * Befuellen der Fields vom CGI
    * Die Fields werden als String uebergeben.
    * 
    * Dies ist notwendig, weil vom CGI Fehlerhafte
    * Zahlen oder Datums uebergeben werden koennten.
    * Damit ist eine Umwandlung in den Domain Datenyp
    * nicht moeglich.
    *
    */
   void getFieldsFromCGI() {
      List usedFields = getUsedFields()
      domain.clear()
      fields = [:]
      usedFields.each {fieldname->
         fields[fieldname] = controller.cgiparam(name:fieldname,nvl:'').toString()
         }
      }

   /**
    * Befuellen Fields von der Domain
    * 
    */
   // ------------------------------------------------------------------
   void getFieldsFromDomain() {
   // ------------------------------------------------------------------
      
      List usedFields = getUsedFields()
      usedFields.each {fieldname->
         fields[fieldname] = domain.getProperty(fieldname)?:''
         }
      }

   /**
    * Loeschen aller Felder in 
    * 
    */
   // ------------------------------------------------------------------
   void clearFields() {
   // ------------------------------------------------------------------
      
      domain.meta?.fields.each {fieldname->
         fields[fieldname] = ''
         }
      }

  
  /**
   * Befuellen Domain aus CGI
   * 
   * @return      false bei Fehler sonst true
   * 
   */
   // ------------------------------------------------------------------
   Boolean fillDomainFromCGI() {
   // ------------------------------------------------------------------
      // Nur Feldname aus dem CGI verwenden
      List usedFields = getUsedFields()
      try {
         if (!domain.setFromCgi(request:controller.request,include:usedFields)) {
            domain.errors.each{ theError->
               if (error != '') error += '<br>'
               error += 'Bei Feld '+theError['field']+': "'+theError['message']+'" ('+theError['value']+')'
               errorfields.add(theError['field'])
               }
            // Fehlermeldung ausgeben
            controller.flash error
            // und wieder Editiermaske aufrufen
            nextview = editview
            saveOk = SAVE_INVALID
            }
         }
      catch (Exception ex) {
         throw new Exception("Viewhandler::fillDomainFromCGI:${ex.message}")
         }
      return saveOk
      }


  /**
   * Lscht einen Datensatz aus der Datenbank
   * 
   * Ist die Variable gridConfirmDelete true
   * wird vor dem lschen eine Besttigungsmaske
   * aufgerufen.
   * 
   * HINT:
   *   setz die Variable saveOK
   * 
   */
   Boolean dbErase() {

      /**
       * Wenn gewuenscht, wird ein
       * Confirmationsdialog augegeben
       */
      if (gridConfirmOnDelete) {

         /** Prfen ob schon besttigt */      
         if (controller.cgiparam(name:'confirm',nvl:'no') != 'yes') {
            Dialog c = new Dialog(controller:controller)
            String querystring = controller.getQueryString()
            String okurl = querystring + '&confirm=yes'
            String backurl = querystring
            backurl = (backurl =~ /&submitbutton=\w+/).replaceFirst('')
            c.confirm(
               message:'Soll die Zeile wirklch gel&ouml;scht werden',
               okurl:okurl,
               backurl:backurl,
               buttons:['l&ouml;schen','zur&uuml;ck']
               )
            return false
            }
          }

      def id = getIdFieldValue()

      saveOk = SAVE_OK

      // Hole Domain
      if (!domain.get(id)) {
         controller.flash("Viewhandler::dbErase: kann domain mit id :${id} nicht finden.")
         saveOk = SAVE_INVALID
         return
         }
 
      // Lschen
      if (!domain.delete()) {
         controller.flash("Viewhandler::dbErase: kann domain mit id :${id} nicht loeschen")
         saveOk = SAVE_INVALID
         return
         }
      submitbutton = SUBMITBUTTON_NEW
      return true
      }

  /**
   * Fhrt die Datenbank Operation durch
   *  
   * HINT:
   *  Verndert die globale Variable nextview
   *  save:
   *     wenn Fehler wieder editiermaske aufrufen
   *     wenn OK Liste aufrufen
   *  
   *  delete:
   *     Listviewer aufrufen
   *  
   *  new:
   *     Editiermaske aufrufen
   *  
   */
   // ------------------------------------------------------------------
   Boolean doDb(dbOperation) {
   // ------------------------------------------------------------------

      String idField = getIdFieldName(domain)
      saveOk = SAVE_OK
      switch (dbOperation) {
         case DBOPERATION_UPDATE:
            List usedFields = getUsedFields()
            if (!domain.save(mode:dbOperation,include:usedFields)) {
               controller.flash "Viewhandler::doDb: bei ($dbOperation): ${domain.sqlError} ${domain.lastSql}"
               saveOk = SAVE_INVALID
               }
            break
         case DBOPERATION_INSERT :
            List usedFields = getUsedFields()
            /**
             * Um eine korrektes schreiben zu gewaehrleisten
             * wird das Key-Feld von der Liste entfernt, wenn das 
             * wenn das Flag gesetzt ist.
             */
            String keyFieldName = domain.meta?.idfield['domain']
            if (removeKeyOnInsert) {
               usedFields -= keyFieldName
               }
            try {
               if (!domain.save(mode:dbOperation,include:usedFields)) {
                  controller.flash "Viewhandler::doDb: bei ($dbOperation): ${domain.sqlError} ${domain.lastSql}"
                  saveOk = SAVE_INVALID
                  }
                }
            catch(Exception ex) {
                  throw new Exception("Viewhandler::doDb: DATENBANKERROR bei ($dbOperation): ${domain.sqlError} ${domain.lastSql} DB-Message: '${ex.message}'")
                  saveOk = SAVE_INVALID
               }

            break
         case DBOPERATION_DELETE :
            if (!domain.save(mode:dbOperation)) {
               controller.flash "Viewhandler::doDb: bei Default DB-Operation: ${domain.sqlError} ${domain.lastSql}"
               saveOk = SAVE_INVALID
               }
          default : 
             throw new Exception("Viewhandler::doDb: bei ${this.tablename} wurde keine gueltige Operation gefunden")
             break
         }

      // Ok-Meldung
      if (saveOk) {
         nextview = viewAftersave
         }
      lastInsertIDValue = domain[idField]
      return saveOk
      }

  /**
   * Setzt die Bindingsvariable fuer
   * den Aufruf des viewers
   */
   void setBindings() {


      // Uebergabefelder fuer
      // view zusammenstellen
      //
      viewBindings+=
         [
         path:path,
         action:action,
         submitbutton:submitbutton,
         fields:fields,
         domain:domain,
         render:controller.render,
         flash:controller.flash,
         errorfields:errorfields
         ]
      }


  /**
   * edit <br>
   * 
   * Zeigt eine Editiermaske.
   * 
   * Ist im CGI eine Variable id oder die ID Feld der Domain vorhanden
   * wird der Inhalt der DOmain in die View gemapped.
   * 
   */
   Boolean doEdit() {

      Boolean retval = true

      String idField 
      switch (submitbutton) {
         case SUBMITBUTTON_NEW :
            fillDomainFromCGI()
            // Keine Eingabefehler
            if (saveOk) {
               // Auf jeden Fall eventuell vorhandene
               // ID Feld entfernen, da sonst bei
               // Insert Probleme auftreten.
               //
               idField = getIdFieldName(domain)
               fields.remove(idField)

               // in DB Schreiben
               saveOk = doDb(DBOPERATION_INSERT)
               }
            // Wenn Fehler aufgetreten ist
            // wird Fehlermeldung und alle Eingabe
            // nochmals ausgegben
            //
            if (!saveOk) {
               getFieldsFromCGI()
               submitbutton = SUBMITBUTTON_NEW
               }
            else {
               submitbutton = ''
               domain.clear()
               clearFields()
               }
            setBindings()
            break

         case SUBMITBUTTON_NEWEDIT :
            clearFields()
            setBindings()
            break

         case SUBMITBUTTON_SAVE :
            // Befuellen Domon von CGI
            saveOk = fillDomainFromCGI()            

            // wenn Keine Fehler aufgetreten sind
            // dann in DB schreiben
            if (saveOk) {
               saveOk = doDb(DBOPERATION_UPDATE)
               }

            // wenn bei DB Operation Fehler
            // aufgetreten sind Feldinhalte 
            // aus CGI befuellen und nochmals ausgeben
            //
            if (!saveOk) {
               getFieldsFromCGI()
               submitbutton = SUBMITBUTTON_EDIT
               }
            else {
               domain.clear()
               getFieldsFromDomain()
               }
            setBindings()
            break

         case SUBMITBUTTON_DELETE :
            // Lschen
            retval = dbErase()
            nextview = viewAfterdelete
            break

         case SUBMITBUTTON_EDIT :
            // Primary Key holen
            //
            def id = getIdFieldValue()
            // Wenn ID gefunden
            // werden Daten aus DB geholt
            if (id != '') {
               // Hole Daten 
               if (domain.get(id)) {
                  // Wenn OK dann fields befuellen
                  domain.meta?.fields.each {field->
                     fields[field.key] = domain.getProperty(field.key)?:''
                     }
                  }
               else {
                  // Fehlermeldung ausgeben 
                  // (Hierher sollte man nicht kommen)
                  //
                  controller.flash "Viewhandler::fillFields: Kann domain mit id: '${id}' nicht finden"
                  throw new Exception("Viewhandler::fillFields: Kann domain mit id: '${id}' nicht finden")
                  }
               }
            else {
               controller.flash "Viewhandler::fileFields: bei Edit muss eine FeldID uebergeben werden "
               submitbutton = SUBMITBUTTON_NEW
               }
            break
         }

      return retval
      }
  /** 
   * Behandeln von WebFlow
   *
   */ 
   void handleWebFlow() {
      
      /** Wenn kein WEbFlow deklariert dann nix tun... */
      if (this.WebFlow == null) return
      if (saveOk) {
         if (this.WebFlow != null) {
            this.WebFlow.each{btn,param->
               switch(param) {
                  case String : 
                     if (btn == submitbutton) this.gridview = param
                     break
                  case List : 
                     if (btn == submitbutton) {
                        this.gridview = param[0]
                        viewBindings['callback'] = param[1]
                        }
                     break
                  default :
                     break
                  }
               //if (btn == submitbutton) this.gridview = fname
               }
            }
         }
      }

  /**
   * Aufruf <br>
   * 
   * <h2>HINT</h2>
   *  Es wird davon ausgegangen, dass bei der Initialiserung
   *  eine Referenz auf den Controller bergeben wurde.
   * 
   * Vorgabewerte knne im Controller durch den Sytax:
   * this.name = 'Inhalt'
   * gesetzt werden.
   * 
   * Wird Domain nicht uebergeben, so wird nur der Viewer aufgerufen
   * 
   * <h2>Layout</h2>
   * Es werden grundsaetzlich zwei Layouarten angeboten:
   * <ul>
   *  <li>Grid<br />Die Datensaetze werden Tabellenfoermig bearbeite</li>
   *  <li>Mask<br />
   *        Es wird eine Liste der mglichen Datensaetze angeboten, 
   *        Der Benutzer waehlt eine Datesatz, diese wird als Maske zum
   *        bearbeiten dargestellt.
   *   </li>
   * </ul>
   *
   * Der Vorgabewert ist GRID, dies kann durch das Attribut 'layout'
   * uebersteuert werden.
   * <ul>
   *  <li>Gridlayout<tt>layout:Viewhandler.GRIDLAYOUT,</tt></li>
   *  <li>Masklayout<tt>layout:Viewhandler.MASKLAYOUT,</tt></li>
   * </ul>
   * 
   * 
   */
   void run() {

     /**
      * Wenn spezieller Pfad angegeben,
      * wird dieser gesetzt.
      * 
      */ 
      path = controller.cgiparam(name:'path',nvl:'')


     /**
      * Standard Rueckgabewert
      */ 
      Boolean retval = true

     
     /**
      * Layout festlegen
      *
      *  GRIDLAYOUT:
      *     Die Daten werden Tabellen artig angezeigt
      *
      *  MASKLAYOUT:
      *     Die Daten werden in einer Folge von 
      *     Auswahlliste und Editiermaske angeboten
      *
      *  USERLAYOUT:
      *     Wird als Layout USERLAYOUT angewendet
      *     so wird angenommen, dass der Entwickler
      *     die Viewernamen selbst festlegt.
      *
      */
      switch (layout) {
         case MASKLAYOUT :
            gridview =  DEFAULT_EDITVIEWER
            editview =  DEFAULT_EDITVIEWER
            listview =  DEFAULT_LISTVIEWER
            break
         case MASKLAYOUT :
            editview = DEFAULT_EDITVIEWER
            listview = DEFAULT_LISTVIEWER
            gridview = DEFAULT_GRIDVIEWER
            break
         }
     /**
      * Aktion ggf. Vorgabewert setzen
      */
      if (this.action == '')        action         = controller.cgiparam(name:'action',nvl:'list')

     /**
      * Submitbutton aus CGI holen
      */
      if (this.submitbutton == '')  submitbutton   = controller.cgiparam(name:'submitbutton',nvl:'')
      
     /**
      * Controller Objekt muss auf jeden Fall uebergeben werden
      */
      assert controller != null : "Grauenhafter Fehler: es wurde keine Referenz auf den Controller beim Initialiseren uebergeben"

     /**
      * Daten an Viewer uebergeben
      */
      viewBindings+=
         [
         path:path,
         action:action,
         submitbutton:submitbutton,
         fields:fields,
         controller:controller,
         taglib:new MVC.taglib.Taglib(
            request:controller.request,
            db:controller.db,
            errorfields:errorfields
            ),
         domain:domain,
         render:controller.render,
         flash:controller.flash,
         errorfields:errorfields,
         disabled:false
         ]

      // Wenn Domin uebergen wurde
      if (this.domain != null) {
         /** Action auswerten und ausfhren */
         switch (action) {
            case SHOWACTION :
               viewBindings['disabled']=true
            case EDITACTION :
               doEdit()
               handleWebFlow()
               controller.view filename:nextview, bindings:viewBindings
               break
            case GRIDACTION :
               retval = doEdit()
               if (retval) {
                  handleWebFlow()
                  controller.view filename:gridview, bindings:viewBindings
                  }
               break
            case LISTACTION :
               controller.view filename:listview, bindings:viewBindings
               break
            case null:
               controller.view filename:listview, bindings:viewBindings
               break
            default :
               controller.view filename:listview, bindings:viewBindings
               break
            }
         }
      // sonst wird Standard aufgerufen
      else {
         controller.view filename:gridview, bindings:viewBindings
         }
      return
      }
   } // END OF CLASS