package framework.taglib

import groovy.sql.Sql
import java.net.*
import helper.Zeitmessung
import java.text.DecimalFormat
import java.text.SimpleDateFormat

/**
 * Zentrale Taglib
 * 
 * Es werden verschiedene Tags zur Verf&uuml;gung gestellt
 * 
 * 
 */
class Taglib {
   //
   // .--update--.
   // -- wann----+-wer+-- was -------------------------------------
   // 13.10.2007 | ny | erstellt
   // 19.01.2008 | ny | GRID Operationen 
   // 21.04.2008 | ny | Helpfile Implementation
   // 17.06.2008 | ny | GridWidgets auf IE angepasst
   // 29.08.2008 | ny | Mglichkeit von Zustzlichen 
   //            |    | Aktionsschaltflaechen eingebaut
   // 20.09.2008 | ny | Bei GridSubmitbutton Namens Option eingebaut
   // 13.10.2008 | ny | Einbau tagidmglichkeit bei promptinput
   // 08.11.2008 | ny | gridEdit eingebaut.
   //            |    | 
   //            |    | 
   //            |    | 

   def   request                  // wird vom Template im Kontruktur uebergeben
   def   db                       // Datenbankobjekt
   List  errorfields = null
   List  StandardCGIParameter = ['path']
   Long  zz=1                     // globaler Zeilenzaehler
   String idFieldName
   String hiddenFields = "\n"
   String httpMethod = 'POST'   


   public final static FORM_ACEPT_CHARSET_UTF_8 = 'UTF-8'
   public final static FORM_ACEPT_CHARSET_ISO_8859_1 = 'ISO-8859-1'

   String formAceptCharset = FORM_ACEPT_CHARSET_UTF_8

   public final static HTTP_METHOD_GET = 'GET'
   public final static HTTP_METHOD_POST = 'POST'

   public final static SUBMITBUTTON_SAVE       = 'save'
   final static SUBMITBUTTON_SAVE_IMG          = './images/skin/database_save.png'
   final static SUBMITBUTTON_SAVE_TEXT         = 'speichern'

   public final static SUBMITBUTTON_EDIT       = 'edit'
   final static SUBMITBUTTON_EDIT_IMG          = './images/skin/database_save.png'
   final static SUBMITBUTTON_EDIT_TEXT         = 'zur&uuml;ckschreiben'
   
   public final static SUBMITBUTTON_DELETE     = 'delete'
   final static SUBMITBUTTON_DELETE_IMG        = './images/skin/database_delete.png'
   final static SUBMITBUTTON_DELETE_TEXT       = 'sichern'
   
   public final static SUBMITBUTTON_NEW        = 'new'
   final static SUBMITBUTTON_NEW_IMG           = './images/skin/database_add.png'
   final static SUBMITBUTTON_NEW_TEXT          = 'neuanlegen'

   public final static SUBMITBUTTON_NEWEDIT    = 'newedit'


   public final static DISPLAY_DISABLED        = 'disabled'
   public final static DISPLAY_YES             = 'yes'
   public final static DISPLAY_NO              = 'no'

   //----------------------------------------------------------------------
   def getYON = {name->
   //----------------------------------------------------------------------
      """
      <select name="$name">
       <option value="1">Ja</option>
       <option value="0">Nein</option>
      </select>
      """
      }

   //----------------------------------------------------------------------
   def getLOVKlassen = {param->
   //----------------------------------------------------------------------
      //
      // Liefert ein Selectbox mit allen LOV-Klassen
      //
      //
      if (param == null) param=[:]  // fuer defaultwerte
      assert param.db != null : "Taglib-Tag getLOVKlasse muss Parameter db haben"

      String selectedvalue = param.get('selected',null)
      String select = """<select name="lovClass">\n"""
      String selected = ''
      param.db.dbh.eachRow('''select * from lov where lovClass = 'class' order by lovClass''') {row->
         if (selectedvalue != null) {  // ein Vorgabwert wurde definiert
            if (row.LOVCLASS == selectedvalue) {
               selected = ' selected="selected" '
               }
            else {
               selected = ''
               }
            }
         select += """<option value="${row.LOVKEY}" ${selected} >${row.LOVKEY}</option>\n"""
         }
      select += '</select>'
      return select
      }

  /**
   * Liefert eine CGI-parameter
   * wird dieser nicht gefunden wird blank zurckgeliefert
   *
   * @param: name: Name des CGI parameters
   * @param. isnull: wert wenn kein Parameter in CGI uebergeben wurde
   * @param: nvl:    Synonym fuer isnull (nvl bindet staerker als isnull)
   *
   * @return: Wert der CGI Variable
   */
   def cgiparam = {param->
      assert (param.name != null) : "name des CGI parameters fehlt"

      String nvl
      String retval = ''

      if (param.nvl) nvl = param.nvl
      if (param.isnull) nvl = param.isnull

      if (request != null) {
         retval = request.getParameter(param.name)
         }
      else {
         throw new Exception("Taglib: request Option nicht vorhanden: Parameter:${param.name}")
         }

      // Blank wenn kein Parameter gefunden wurde
      if (retval == null) {
         // Wenn ein Vorgabewert definiert wurde
         retval = ''
         if (nvl) retval = nvl
         }
      retval = java.net.URLDecoder.decode(retval,formAceptCharset)

      return retval
      }

 /**
  * Erzeugen einer Selectbox
  *
  * @param: name      Name Option
  * @param: dbh       Datenbankhandle
  * @param: prompt    Prompttext
  *                   Ist das Feld angegeben wird eine <tr><td> Rahmen erstellt
  *
  * @param: sql       SQL Statment
  *                   Die Listclause muss zwei Felder haben
  *                   Die Namen der Spalten muss VALUE und KEY sein. ACHTUNG auf GROSZSCHREIBUNG
  *                   KEY wird als Schluesselwert in die <option value=<key>>
  *                   VALUE als Ausgabewert eingetragen
  * @param: selected  Inhalt welcher vorselektiert werden soll
  * @param: addempty  Wenn vorhanden wird eine Option fuer Leer ode Alle angabe ausgegeben
  *                   Soll Option und Anzeigetext verschieden sein wird
  *                   dies in der Form 'anzeigetext:optiontext' angegeben
  * @param: fromcgi   t/f     
  *                   Der Vorgabewert wird vom CGI geholt
  *                   HINT:
  *                      fromcgi hat Vorrang gegenueber selected
  *                      (diese kommt normalerweise aus der domain)
  * @param: datasource      
  *                      Erwartet eine Liste von Listen mit Inhalt Key, Value
  *                   z.B: [['1','eins'],['2','zwei'],...]
  *                   Wobei die Inner List als Key/Value Paar aufgefasst wird
  *                   Diese Option darf nicht in Kombination mit SQL angewandt werden.
  * @param: disabled  true wenn das Feld als Disabled dargestellt werden soll
  * 
  * @param: class     CSS Klassenanme (vorgabe. Keine)
  * 
  * @param:  style    Extra Styleangabe
  * 
  * @param: exclude   eine Liste von nicht anzuzeigenden Inhalten (Keys)
  * 
  * @param: addtop   eine Map mit noch hinzuzufgenden Einraegen am Beginn der Liste
  *
  * @return:          HTML Selectstatment
  *
  */
  def selectbox = {param->
 
      if (param == null) param = [:]

      String selectedvalue = param.selected

      // Wenn uebergeben Class Option zusammenbauen
      //
      String cssclass = (param.get('class','') != '' 
                ? ' class="'+param.get('class','')+'" ' 
                : '')

      // Werte aus CGI holen
      if (param.fromcgi) {
         selectedvalue = request.getParameter(param.name)

         // es wurde kein Inhalt ueber das CGI geliefert
         if (selectedvalue == null) {
            selectedvalue = param.selected // Standardwert aus dem Aufruf
            }
         }

      String disabledText = param.disabled ? ' disabled ' : ''
      String style = param.style == null ? '' : ' style="'+param.style+'" '
      String tagid       = param.get('tagid','')
      
      if (tagid != '') tagid = """ id="${tagid}" """
      String tag = """<select ${cssclass} name="${param.name}" ${disabledText} ${tagid} ${style}>"""

      if (param.addempty != null) {
         String empty = param.addempty
         List opt = empty.split(':')
         tag+="""<option value="${opt[1]?:''}">${opt[0]}</option>\n"""
         }

     /**
      * Extra eintraege am beginn hinzufuegen
      * HINT:
      *  der key stellt den Datenbankeintrag, der Value den Anzeigetext dar.
      */
      if (param.addtop) {
         String selected
         param.addtop.each{key,value->
            if (key == selectedvalue.toString()) selected = ' selected="selected" '
            tag+="""<option value="${key?:''}"${selected}>${value?:''}</option>\n"""
            }
         }
      // Wird Datasource verwendet
      // oder direktes SQL
      //
      if (param.datasource == null) {  // Direktes SQL
         param.dbh.eachRow(param.sql) {row->
            String selected = ''
            try {
               if (row.KEY.toString() == selectedvalue.toString()) selected = ' selected="selected" '
               }
            catch (Exception e){
               throw new Exception('selectbox: Umwandlungsfehler bei selected: ('+e.message+') value'+selectedvalue)
               }

            // Filtern auszuschliessender Inhalte
            //
            if (param.exclude == null) {
               tag += """<option ${cssclass} value="${row.KEY}"${selected}>${row.VALUE}</option>\n"""
               }
            else {

               if (!param.exclude.grep(row.KEY)) {
                  tag += """<option ${cssclass} value="${row.KEY}"${selected}>${row.VALUE}</option>\n"""
                  } 
               }
            }
         } 
      // Datasource
      //
      else {
         List options = [] 
         param.datasource.each {value->

            if (value[0] == '<optgroup>') {
               options << '<optgroup label="'+value[1]+'">'
               return
               }
            if (value[0] == '</optgroup>') {
               options << '</optgroup>'
               return
               }

            String selected = ''
            String title = ''
            if (value[3] != null) {
               title = """title="${value[3]}" """
               }
            try {
               if (value[2]) selected = ' selected="selected" '
               }
            catch (Exception e){
               throw new Exception('selectbox: Umwandlungsfehler bei selected: ('+e.message+') value'+selectedvalue)
               }

            String optionsvalue = value[1]

            // Abschneiden
            if (param.get('truncate',0) > 0) {
               optionsvalue = (optionsvalue.padRight(param.truncate)).getAt((0..param.truncate-1))
               }


            // Filtern auszuschliessender Inhalte
            //
            if (param.exclude == null) {
               options << """<option ${title} ${cssclass} value="${value[0]}"${selected}>${optionsvalue}</option>"""
               }
            else {
               if (!param.exclude.grep(value[0])) {
                  options << """<option ${title} ${cssclass} value="${value[0]}"${selected}>${optionsvalue}</option>"""
                  } 
               }
            }
         tag += options.join("\n")
         }
      
      tag += "</select>\n"

      // Prompt bearbeiten
      //
      if (param.get('prompt','') != '') {
         return """<tr><th class="msk">${param.prompt}</th><td>${tag}</td></tr>"""
         }
      else {
         return tag
         }
      }




   /**
    * Erzeugen eines Eingabefeldes mit prompt
    *
    * @param: prompt          Prompttext
    * @param: name            Feldname
    * @param: fieldname       Name des Feldes (... name="..) Vorgabewert: 'ITEM'
    * @param: size            Eingabegroesse des Feldes (... size="...) Vorgabewert: 32
    * @param: maxlenth        maximal einzugebende Zeichenmenge (... maxlength="...) Vorgabewert: 255
    * @param: type            text|textarea|display|displayandhidden Vorgabewert ist text
    *                         text: input tag
    *                         textarey: textarea tag
    *                         display:  Nur Textausgabe
    *                         displayandhidden: Der Inhalt wird ausgegeben, es wird aber
    *                                           zustzlich ein <input type="hidden" feld erzeugt
    *                         
    * @param: rows            fuer Texteares
    * @param: cols            fuer Textares
    * @param: disabled        setzt das diable Kennzeichen
    * @param: value           Vorgabewert (Bei Masken welche von der DB befllt werden)
    * @param: fromcgi         true:  Feldinhalt wird vom CGI uebernommen (default = leer)
    *                         false: keine Bearbeitung
    *                         HINT: hat vorang gegenueber value
    * @param: nohtml          t/f wird f bergeben, so
    *                         wird der Tag nicht in HTML Tags (tr, td) eingerrahmt)
    * @param: notr            t/f wird true bergeben, so wird widget 
    *                         nicht in <tr> eingerahmt
    * @param: valueclass      wird als CLASS des TD-Tags der Valuespalte eingefgt
    * @param: style           Extra Styleangabe
    *
    * @param: promptclass     Wirdlals CLASS des TH-Tags eingefuegt
    * @param: checked         Wert welcher als Checked angegeben wird
    * @param: tagid           Wert fuer die id Option des tags
    * 
    * 
    * 
    * HINT:
    *    Kennzeichnung Felder mit Fehlern
    *    Eine CGI Variable ('errorfields') enthllt eine mit Kommas getrennte Liste
    *    mit Feldnamen welche einen Fehler hatten.
    *
    *
    */
   def promptinput = {param ->

      if (param == null) param=[:]  // fuer defaultwerte

      // vorgabewerte definieren
      String  size        = param.get('size','32')
      String  name        = param.get('name','ITEM')
      String  maxlength   = param.get('maxlength','255')
      String  type        = param.get('type','text')
      String  cols        = param.get('cols','64')
      String  rows        = param.get('rows','4')
      String  disabled    = param.get('disabled',false)
      String  nvl         = param.get('nvl',null)
      String  prompt      = param.get('prompt','Eingabe')
      String  value       = param.get('value',null)
      Boolean fromcgi     = param.get('fromcgi',true)
      Boolean noHTML      = param.get('nohtml',false)
      Boolean noTR        = param.get('notr',false)
      String  classvalue  = param.get('valueclass','grid')
      String  classprompt = param.get('promptclass','msk')
      String  popuphelp   = param.get('popuphelp','')
      String  promptpopup = param.get('promptpopup','')
      String  checked     = param.get('checked','')
      String  tagid       = param.get('tagid','')

      String  style = param.style == null ? '' : ' style="'+param.style+'" '
      List    errorfields = param.get('errorfields',cgiparam(name:'errorfields',nvl:'').split(','))


      String tagvalue = null           // Der Ausgabewert
      boolean foundCgi = false         // Wird wahr, wenn Feldinhalt vom CGI genommen wird
      boolean fieldHasError = false    // wird auf wahr gesetzt wenn Feldname in Fehlerfeldliste

      if (popuphelp != '') popuphelp = """ title = "${popuphelp}" """
      if (promptpopup != '') promptpopup = """ title = "${promptpopup}" """
      if (tagid != '') tagid = """ id="${tagid}" """


      // Wurde bein Konstruktor die Liste der Fehlerhaften Fehler (errorfields)
      // mitgegeben wird diese genommen.
      // Errorfields knnen auch noch als Option (errorfield) erzeugt.
      //
      if (this.errorfields != null) {
         errorfields = this.errorfields
         }

      // Prfen, ob das aktuelle Feld als Fehlerhaft markiert wurde
      //
      if (errorfields.grep(name)) {
         fieldHasError = true
         }

      // Prfen ob CGI Input gewnscht wurde
      if (fromcgi) {
         // befllen von CGI
         // ist kein CGI Parameter vorhanden
         // Defaultvalue einsezten
         tagvalue = request.getParameter(name)
         //
         if (tagvalue != null) {
            foundCgi = true
            }
         }

      // Wert ist ungesetzt (null) wenn kein
      // Inhalt vom CGI kommt
      //
      if (!foundCgi) {
         // Wir versuchen daten von der Domain zu beiommen
         //
         if (value != null) {
            tagvalue = value
            }
         }

      // Hat nichts gegriffen
      // Defaultwert einsetzen
      //
      // HINT:
      //    wurde auch kein Nullwert definiert
      //    wird Leerstring ('') eingesetzt um den wert 'null' in Feld
      //    zu vermeinden.
      //
      if (tagvalue==null) {
         tagvalue = nvl == null ? '' : nvl
         }
      // Tag wird mit Fehlermeldung bestueckt (Sollte  nie von Benutzer gesehen werden)
      String tag = "ungueltiger type: ${type}"

      String disabledText = param.disabled ? ' disabled ' : ''
      
      switch (type) {

         case 'text' :
            tag = """<input type="text" name="${name}" ${tagid} ${popuphelp} size="${size}" maxlength="${maxlength}" value="${tagvalue}" ${disabledText} ${style}>"""
            break

         case 'textarea' :
            tag = """<textarea cols="${cols}" ${popuphelp} ${tagid} rows ="${rows}" name="${name}" maxlength="${maxlength}" ${disabledText}${style}>${tagvalue}</textarea>"""
            break

         case 'display' :
            tag = tagvalue
            break

         case 'hidden' :
            return """<input type="hidden" name="${name}" ${tagid} value="${tagvalue}" >"""
            break

         case 'displayandhidden' :
            tag = tagvalue+""" <input type="hidden" name="${name}" ${tagid} ${popuphelp} value="${tagvalue}"> """
            break
         
         case 'checkbox' :
            String checkedvalue = ''
            if (tagvalue == checked) {
               checkedvalue = ' checked '
               }
            tag = """<input ${disabledText} type="checkbox" ${tagid} ${checkedvalue} name="${name}"> <span ${promptpopup} class="${classvalue}">${prompt}</span>"""
            prompt = ''
            break

         case 'select' :
            param.prompt = null   // Verhindert das nochmalige erzeugen von Promteintrag
            if (fromcgi) {
               param['fromcgi']=true
               }
            tag = selectbox(param)
            break
         }

      if (fieldHasError) {
         classvalue = 'msk-error'
         }
      else {
         classvalue = classprompt
         }

      // in HTML einrahmen
      if (noHTML) {
         return """<span ${promptpopup} class="${classvalue}">${prompt}</span>$tag"""
         }

      // nicht mit <tr> einrahmen
      if (noTR) {
         return """<th ${promptpopup} class="${classvalue}">${prompt}</th><td>${tag}</td>"""
         }


      return """<tr><th ${promptpopup} class="${classvalue}">${prompt}</th><td>${tag}</td></tr>"""
      }

   //--- Hidden fields-----------------------------------------------------
   def getHiddenfieldsFromCGI = {param->
   //----------------------------------------------------------------------
      //
      // Liefet HTML hidden fields
      //
      // @param: fields          eine liste mit CGI Parameter
      //

      String retval = ''

      if (param instanceof Map) {
         param.fields.each {fieldname->
            retval += '<input type="hidden" value="'+cgiparam(name:fieldname,nvl:'')+'" name="'+fieldname+'" >'
            }
         }
      else {
         retval += '<input type="hidden" value="'+cgiparam(name:param,nvl:'')+'" name="'+param+'" >'
         }
      return retval
      }


   //--- Liefert ein Icon -------------------------------------------------
   def getIcon = {param->
   //----------------------------------------------------------------------
      //
      // HINT:
      // Verweis auf das /skin verzeichnis
      //
      //
      //
      if (param == null) param = [:]
      String iconPath = './images/skin/'
      String filePath = iconPath+param.get('name','**unkown**')
      String hrefname = param.get('hrefname',null)
      String id = param.get('id',null)
      String title = param.get('title','')

      if (title != null) title = """ title="${title}" """

      if (hrefname != null) {
         hrefname = """ name="${hrefname}" """
         }
 
      if (id != null) id = """ id="${id}" """
      // wenn kein href definiert wurde nur Pfad zurckggeben
      if (param.href == null) {
         return filePath
         }
      String icon = '<a href="'+param.get('href','')+'"'+(hrefname?:'')+(id?:'')+'><img src="'+filePath+'"'+title+'" border="0"></a>'
      return icon
      }

   //--- ausgabe einer Liste-----------------------------------------------
   def listwidget = {param->
   //----------------------------------------------------------------------
      //
      // Ausgabe Schalterleiste fuer Blaetterfunktion in Listen
      //
      // @param:  pageno         Seitennummer
      // @param:  table          Tabellenname
      // @param:  filter         Filter lt. CGI
      // @param:  where          Zusaetzliche whereclause
      // @param:  pagesize       (bei mysql) pagesize
      // @param:  filterfeld     Feld auf welches sich der Filter bezieht
      // @param:  filterwidget   true/false Filterwidget ausgeben
      // @param:  filtertext     Nach Filterwidget wird angegebener Text ausgegeben
      //
      //

      assert param.table != null : "listwidget: Option table muss angegeben sein"

      if (param == null) param = [:]

      String widget= '<table style="border:0;border-spacing:0;margin:0;" cellspacing="0" cellpadding="0"><tr>'
      List widgetEntries = []

      //  where clause zusammenbauen
      String where = param.get('where',' (1=1) ')

      // Pagesize herstellen
      Integer pagesize = param.get('pagesize',16)

      // Wurde ein Databasehandle uebergebn dieses speichern
      if (param.get('db',null) != null) this.db = param.get('db',null)

      // Anzahl aller Datensaetze
      String sql = "select count(*) as RECORDS from ${param.table} where ${where}"
      def cntRec = db.dbh.firstRow(sql)

      // Seitenanzahl berechnen
      Integer pages = cntRec.RECORDS / pagesize
      if (pages < 1 ) {
         pages = 1
         }
      else {
         pages++
         }

      widgetEntries.add ('<td>'
                 +"""<form name="first" action="start.groovy" action="${httpMethod}" accept-charset="${formAceptCharset}">"""
                   +getHiddenfieldsFromCGI(fields:StandardCGIParameter)
                   +'<input type="hidden" name="pageno" value="1">'
                   +'<input type="submit" value="|&lt;" title="Spring zur ersten Seite">'
                  +'</form>'
                  +'</td>'+"\n"
                )
      Integer prevpageno = cgiparam(name:'pageno',nvl:'0').toInteger()
      prevpageno--

      if (prevpageno < 1) prevpageno = 1
      widgetEntries.add ('<td><nobr>'
                 +"""<form name="prev" action="start.groovy" action="${httpMethod}" accept-charset="${formAceptCharset}">"""
                  +getHiddenfieldsFromCGI(fields:StandardCGIParameter)
                  +'<input type="hidden" name="pageno" value="'+prevpageno.toString()+'">'
                  +'<input type="submit" value="&lt;" title="Seite zur&uuml;ckble&auml;ttern">'
                 +'</form>'
                 +'</nobr></td>'+"\n"
                )

      // Select-Tag zusammenbauen
      // Vorselektieren der pagenummer
      // Statusfeder anlegen
      widgetEntries.add """<td><nobr><form name="paginate" action="start.groovy" action="${httpMethod}" accept-charset="${formAceptCharset}">"""
      widgetEntries.add getHiddenfieldsFromCGI(fields:StandardCGIParameter)
      String select = """<select name="pageno" title="Seitenauswahl (* zeigt alle ${cntRec[0]} Datens&auml;tze an)">"""
      Integer cntSelectOptions = 0

      List selectList = []
      pages.times{pageno->
         String selected = ''
         cntSelectOptions++
         if (cgiparam(name:'pageno',nvl:0) == (pageno+1).toString()) {
            selected = ' selected '
            }
         selectList.add "<option value=\"${pageno+1}\"${selected}>${pageno+1}</option>"
         }
      switch(cntSelectOptions) {
         case 0 :
            selectList.add "<option value=\"1\">1</option>"
            break
         case 1 :
            break
         default :
            selectList.add "<option value=\"0\">*</option>"            
            break
         }
      selectList.add '</select>'
      select += selectList.join(null)

      widgetEntries.add select
      widgetEntries.add '<input type="submit" value="go" title="Springt zur angegebenen Seite">'
      widgetEntries.add '</form></nobr></td>'+"\n"

      Integer nextpageno = cgiparam(name:'pageno',nvl:'1').toInteger()
      nextpageno++

      if (nextpageno > pages) nextpageno = pages
      widgetEntries.add ('<td>'
                 +"""<form name="next" methode="${httpMethod}" action="start.groovy" accept-charset="${formAceptCharset}">"""
                  +getHiddenfieldsFromCGI(fields:StandardCGIParameter)
                  +'<input type="hidden" name="pageno" value="'+nextpageno.toString()+'">'
                  +'<input type="submit" value="&gt;" title="Seite vorbl&auml;ttern">'
                 +'</form></td>'+"\n"
                )

      widgetEntries.add ('<td>'
                 +"""<form name="last" methode="${httpMethod}" action="start.groovy" accept-charset="${formAceptCharset}">"""
                  +getHiddenfieldsFromCGI(fields:StandardCGIParameter)
                  +'<input type="hidden" name="pageno" value="'+pages+'">'
                  +'<input type="submit" value="&gt;|" title="Letzte Seite">'
                 +'</form></td>'+"\n"
                )

      // soll filterwidget ausgegeben werden
      //
      if (param.get('filterwidget',true)) {
         String filterText = param.get('filtertext','')
         String filterValue = param.get('filter','')
         widgetEntries.add ('<td>'
                    +"""<form name="filter" methode="${httpMethod}" action="start.groovy" accept-charset="${formAceptCharset}">"""
                     +getHiddenfieldsFromCGI(fields:['path','controller'])
                     +'<input type="hidden" name="pageno" value="1">'
                     +'<input type="text" name="filter" value="'+filterValue+'" title="Filterbegriff eingeben">'
                     +'<input type="submit" value="filtern" title="Filtervorgang starten">'
                     +filterText
                    +'</form></td>'+"\n"
                   )
         }

      widgetEntries.add '</tr></form></table>'+"\n"
      widget += widgetEntries.join(null)
      return widget
      }

   //--- Listenkoepfe -----------------------------------------------
   def listheader = {param->
   //----------------------------------------------------------------------
      //
      // gibt eine HTML Listenkopfzeile (<tr><th ...) aus
      //
      // @param: header    eine Liste von Spaltenueberschriften
      //
      // usage: <%=listheader(th:['ID','Schl&uuml;ssel','Wert']) %>
      //

      if (param == null) param = [:]
      String retVal = '<tr>'
      String cssclass = 'listheader'
      if (param.get('class','') != '') cssclass=param.class

      param.th.each {entry->
         String text
         String popuphelptext=''

         if (entry instanceof List) {
            text = entry[0]
            popuphelptext = entry[1]?:''
            }
         else {
            text = entry
            }
         String popuphelp = ''
         if (popuphelptext != '') popuphelp = """title="${popuphelptext}" """
         retVal += """<th ${popuphelp} class="${cssclass}">${text}</th>"""
         }

      retVal += '</tr>'
      return retVal
      }

   //--- Listenkoepfe -----------------------------------------------
   def listrow = {param->
   //----------------------------------------------------------------------
      //
      // gibt eine HTML Listenzeile aus
      //
      // @param td            Liste von Feldinhalten
      // @param notr          unterdruck <tr>, </tr>
      // @param expandlastcol Wenn false, wird die Letzte Spalte nicht auf 99% gesetzt
      //
      // usage: <%=taglib.listrow(td:[row.LOVID,row.LOVCLASS]) %>
      //

      if (param == null) param = [:]

      String cssclass = param.get('class','')

      String tdStyle = (((this.zz++) % 2) == 0) ? 'background-color:#EFEFEF' : 'background-color:#FFFFFF'
      String textStyle = ''
      

      String retVal = ''
      if (param.get('tr',true)) {
         retVal = "\n<tr>\n"
         }
      Integer cntCol = 0

      List entries = []

      param.td.each {entry->
         cntCol++   // Spaltennummer
         String text = entry
         String popuphelptext=''

         if (entry instanceof List) {
            text = entry[0]
            popuphelptext = entry[1]?:''
            if (entry[2] != null) cssclass = entry[2]
            if (entry[3] != null) {
               textStyle = """ style="${entry[3]}" """
               }
            }
         else {
            text = entry
            }
         
         if (popuphelptext != '') popuphelptext = 'title="'+popuphelptext+'"'
         if (text == null) text = ''

         String tdWidth = ''

         if ((cntCol == param.td.size()) && (param.get('expandlastcol',true))) { // Letzte Spalte
            tdWidth = 'width="99%"'
            }
         entries.add("""   <td class="${cssclass}" style="${tdStyle}" ${tdWidth}><nobr><span ${popuphelptext} ${textStyle}>${text}</nobr;></span></td>\n""")
         }

      retVal += entries.join(null)

      if (param.tr) {
         retVal += "</tr>\n"
         }

      return retVal
      }

   //--- gibt einen Submitbutton zurck------------------------------
   def actionbutton = {param->
   //----------------------------------------------------------------
      //
      // Erzeugen eines AKtionsbuttons
      // 
      // @param   name     Nameoption
      // @param   title    Titeloption
      // @param   onclick  Javascript OnClick Option
      // @param   value    Ist der Inhaltstext ACHTUNG schliesst sich mit onclick aus (siehe code)
      // @param   icon     img src Option
      // @param   text     Text (default leer)
      // @param   type     Buttontype (Default 'submit')
      // @param   class    css Klassename (Default: msk-actionbutton=
      // @param   icon     Name der Icondate (Optional)
      // 
      // 

      String button = ''
      String css_class = param.class?:'msk-actionbutton'
      button = """
           <input type="submit" name="submitbutton" title="${param.title}" class="${css_class}" onClick="this.value='${param.value?:''}'" value="${param.text?:''}"/>
         """
      return button
      }


  /**
   * Gridzeilen Aktionsknpfe
   * 
   * @param   name     Nameoption
   * @param   title    Titeloption
   * @param   onclick  Javascript OnClick Option
   * @param   icon     img src Option
   * @param   text     Text (default leer)
   * @param   type     Buttontype (Default 'submit')
   * @param   class    css Klassename (Default: msk-actionbutton=
   * @param   icon     Name der Icondate (Optional)
   * 
   */
   String gridRowActinbutton(Map param) {

      String button = ''

      button = """<input type="hidden" value="${param.get('value','')}" name="submitbutton"><input title="${param.title?:''}"  style="vertical-align:middle;border:1px solid lightgrey;margin:0;padding:0;" type="image" src="${getIcon(name:param.icon)}">"""
      return button
      }

  /**
   * Gibt eine Submitbutton fuer Grid zurueck.
   * 
   * <pre>
   * -- submitbutton --+-- Neuer Wert ---------------
   *  edit             | save
   *  save             | new
   *  delete           | new
   *                   | 
   * </pre>
   * 
   * @return   Aktonbutton
   * 
   */
   String gridSubmitbutton(String action,String submitbutton) {
      String submitbuttonvalue   =''
      String text                = ''
      String img                 = ''
      String title               = ''
      String button              = ''

      if (action != 'grid') return "<!-- FEHLERHAFTE action ${action} -->"

      switch (submitbutton) {

         case SUBMITBUTTON_EDIT :
            text += SUBMITBUTTON_SAVE_TEXT
            submitbuttonvalue += SUBMITBUTTON_SAVE
            img = SUBMITBUTTON_SAVE_IMG
            title   = 'zur&uuml;ckschreiben'
            break

         case SUBMITBUTTON_DELETE :
            text += SUBMITBUTTON_NEW_TEXT
            submitbuttonvalue += SUBMITBUTTON_NEW
            img = SUBMITBUTTON_NEW_IMG
            title   = 'l&ouml;schen'
            break

         case SUBMITBUTTON_SAVE :
            text += SUBMITBUTTON_NEW_TEXT
            submitbuttonvalue += SUBMITBUTTON_NEW
            img = SUBMITBUTTON_NEW_IMG
            title   = 'neuanlegen'
            break

         default :
            text += SUBMITBUTTON_NEW_TEXT
            submitbuttonvalue += SUBMITBUTTON_NEW
            img = SUBMITBUTTON_NEW_IMG
            title   = 'zur&uuml;ckschreiben'
            break

         }
      button = """<input type="hidden" name="submitbutton" value="${submitbuttonvalue}">
   <input type="image" src="${img}" title="${title}" name="submit-${submitbuttonvalue}"> """
      return button
      }


   def addnew_save_action = {
      //
      // Liefer in Abhaengigkeit der Aktuellen action CGI Variable
      //

      String retVal = ''
      switch (cgiparam(name:'action',nvl:'')) {
         case 'edit' :
            retVal = '<input type="hidden" name="action"     value="save">'
            break
         case  'new' :
            retVal = '<input type="hidden" name="action"     value="newadd">'
            break
         }
      return retVal
      }

   //--- Standardschaltflaechen---------------------------------------------
   def getSaveAndDeleteButton = {
   //----------------------------------------------------------------------
      //
      //
      //
      //
      String retVal

      retVal = actionbutton(name:'submitbutton',value:'save',icon:'database_save.png',text:'speichern')

      if (cgiparam(name:'action',nvl:'') != 'new') {
         retVal += actionbutton(name:'submitbutton',
                                value:'delete',
                                icon:'database_delete.png',
                                text:'l&ouml;schen',
                                onclick:'''if (!confirm('wirklich l&ouml;schen?')) return false}''',title:'l&uuml;schen des Eintrags'
			                      )
        }
      return retVal
      }

   //--- Standardschaltflaechen---------------------------------------------
   def getSaveButton = {
   //----------------------------------------------------------------------
      //
      //
      //
      //
      return actionbutton(name:'submitbutton',value:'save',icon:'database_save.png',text:'speichern')
      }

   //--- Standardschaltflaechen---------------------------------------------
   def getNewButton = {
   //----------------------------------------------------------------------
      //
      //
      //
      //
      return actionbutton(name:'submitbutton',value:'new',icon:'database_save.png',text:'neu')
      }

  /**
   *
   * Setzt den Aktinsbutton einer Maske
   * 
   * @param action      ::= [SUBMITBUTTON_NEW,SUBMITBUTTON_SAVE,SUBMITBUTTON_DELETE]
   * 
   */
   String getSubmitButton(String value) {
      String icon
      String text
      switch (value) {
         case  SUBMITBUTTON_NEW :
            text = 'neu'
            icon = 'database_add.png'
            break
         case  SUBMITBUTTON_SAVE :
            text = 'speichern'
            icon = 'database_save.png'
            break
         case  SUBMITBUTTON_DELETE :
            text = 'l&ouml;schen'
            icon = 'database_delete.png'
            break
         default :
            throw new Exception ("Taglib::getSubmitbButton: Untueltiger Parameter value '${value}'")
            break
         }
      return actionbutton(name:'submitbutton',value:value,icon:icon,text:text)
      }


   /**
    * 
    * Hilfsroutine fuer selected Value Parameter bei Option datesource in Selectboxen
    * 
    * in Abhngikeit der CGI Variable 'errorfields' (indiziert Fehler)
    * wird entweder der Inhalt von Value oder der Inhalt aus dem CGI Variable 
    * zurckgeliefert.
    * 
    * @param   "value"        Wert wenn Feld kein Fehler
    * @param   "name"         Name des Feldes
    * 
    * 
    */
   Object getSelectedValue (Map param) {

      if (param==null) param = [:]

      assert param.name  != null : "taglib::getSelectedValue:    Parameter name nicht uebergeben"

      def retval = null
      if (cgiparam(name:'errorfields',nvl:'') != '') { // Fehlerfall
         retval = cgiparam(name:param.name)
         }
      else {
         retval = param.value
         }
      return retval
      }

   /**
    * Liefert die Aktionsschaltflchen 
    * 
    * @param Es werden alle Parameter von gridRow uebergeben
    * 
    * HINT:
    *    Im Parameter "gridbuttons" wird eine Liste der zu Verwendeten
    *    Standardschaltflaechen angegeben. Dies knnen SUBMITBUTTON_DELETE und SUBMITBUTTON_EDIT
    *    sein.
    * 
    */
   String gridRowButtons(Map param) {
      
      String   deleteButtonText = param.get('deletebuttontext','&nbsp;-&nbsp;')
      String   editButtonText = param.get('editbuttontext','&nbsp;*&nbsp;')
      String   deleteIcon = param.get('deleteicon','database_delete.png')
      String   editIcon = param.get('editicon','database_edit.png')
      String   extraButtons = ''
      List     gridbuttons = param.get('gridbuttons',[SUBMITBUTTON_DELETE,SUBMITBUTTON_EDIT])

      // wurden Zustzliche Schaltflaechen angefordert
      // 
      if (param.get('addbuttons',[]) != null) {
         List tags = []

         param.addbuttons.each{button->
            tags.add("""<form  style="display:inline;margin:0;padding:0" name="add-${idFieldName}-${param.domain.getProperty(idFieldName)}" id="${param.domain.getProperty(idFieldName)}" method="${httpMethod}"  action="start.groovy" accept-charset="${formAceptCharset}"> """)
            tags.add("""<input type="hidden" name="${idFieldName}" value="${param.domain.getProperty(idFieldName)}">""")
            tags.add("""<input type="hidden" name="path" value="${this.cgiparam(name:'path',nvl:'')}">""")
            tags.add("""<input type="hidden" name="action" value="grid">""")
            tags.add("""${hiddenFields}""")
            tags.add("""${this.gridRowActinbutton(
                                name:'submitbutton',
                                value:button.action,
                                title:button.title,
                                text:button.name,
                                icon:button.icon,
                                class:'msk-actionbutton-short')}""")
            tags.add("""</form>""")
            extraButtons += tags.join()
            }
         }

      // Sollen die Standardaktionsschaltflaechen 
      // angezeigt werden.
      //
      if (param.get('actionbuttons',true)) {
         List tags = []

         tags.add("""<td>""")
         if (gridbuttons.grep (SUBMITBUTTON_DELETE)) {
            tags.add("""<form  style="display:inline;margin:0;padding:0" name="era-${idFieldName}-${param.domain.getProperty(idFieldName)}" id="${param.domain.getProperty(idFieldName)}" method="${httpMethod}"  action="start.groovy" accept-charset="${formAceptCharset}">""")
            tags.add("""<input type="hidden" name="${idFieldName}" value="${param.domain.getProperty(idFieldName)}">""")
            tags.add("""<input type="hidden" name="path" value="${this.cgiparam(name:'path',nvl:'')}">""")
            tags.add("""<input type="hidden" name="action" value="grid">""")
            tags.add("""${hiddenFields}""")
            tags.add("""${this.gridRowActinbutton(
                                name:'submitbutton',
                                value:'delete',
                                title:'Eintrag l&ouml;schen',
                                text:deleteButtonText,
                                icon:deleteIcon,
                                class:'msk-actionbutton-short')}""")
            tags.add("""</form>""")
            }

         if (gridbuttons.grep(SUBMITBUTTON_EDIT)) {
            tags.add("""<form style="display:inline" name="edit-${idFieldName}-${param.domain.getProperty(idFieldName)}" id="${param.domain.getProperty(idFieldName)}"  method="${httpMethod}" action="start.groovy" accept-charset="${formAceptCharset}"> """)
            tags.add("""<input type="hidden" name="${idFieldName}" value="${param.domain.getProperty(idFieldName)}">""")
            tags.add("""<input type="hidden" name="path" value="${this.cgiparam(name:'path',nvl:'')}">""")
            tags.add("""<input type="hidden" name="action" value="grid">""")
            tags.add("""${hiddenFields}""")
            tags.add("""${this.gridRowActinbutton(
                           name:'submitbutton',
                           value:'edit',
                           title:'Eintrag bearbeiten',
                           icon:editIcon,
                           text:editButtonText,
                           class:'msk-actionbutton-short'
                       )}""")
            tags.add("""</form>""")
            }
         tags.add("""${extraButtons}""")
         tags.add("""<td>""")
         String retval = tags.join()
         return (retval =~ /\>\s*\</).replaceAll('><')
         }
      }

   //----------------------------------------------------------------------
   String gridRow(Map param) {
   //----------------------------------------------------------------------
      // 
      // Datenzeile mit Aktionsschaltflchen
      // 
      // @param   domain            Datendomain
      // @param   editbuttontext    Text des Editierbuttons
      // @param   deletebuttontext  Text des Lschbuttons
      // @param   values            Liste der Anzeigewerte
      // @param   fieldnames        Liste mit Feldnamen aus der Domain, welche angezeigt werden sollen
      // @param   hidden            Liste von Feldnamen aus der Domain, welche als Hidden Fields generiert werden sollen
      // @param   actionbuttons     t/f (Vorgabewert t) Anzeigen der Aktionbuttons (edit/delete)
      // @param   hidden            Liste von zustzlichen Hiddenfields
      // @param   nobr              Spalten nicht umbrechen
      // @param   gridbuttons       Vorgabewert ist SUBMITBUTTON_DELETE, SUBMITBUTTON_EDIT
      // @param   hiddenfields      Liste von Hiddenfields, die zusaetzlich angegeben werden
      // 
      // @return  Eine Zeile fuer ein Grid
      // 
      // HINT:
      //    Steuerung des Aussehens:
      //    Die Values knnen als Wert oder in Form einer Liste
      //    uebergeben werden. Wird die Listform gewaehlt: [value,titletext,tdstyle,valuestyle]
      //    knne spezialisierte Formatierungen vorgenommen werden
      //    
      //    
      //    
      // 

      if (param == null) param = [:]

      assert param.domain != null : "taglib:gridRow Parameter domain fehlt"

      // Kennzeichnung der Aktionsbuttons
      String   tdclass = (((this.zz++) % 2) == 0) ? 'listcellcolor1' : 'listcellcolor2'

      // Hole ID Feldname aus Domain
      // 
      idFieldName = param.domain.meta.idfield['domain']
      if (idFieldName == null) {
         throw new Exception ("taglib:gridRow  In der Domain ist im meta idfield die Option domain nicht vorhanden")
         }

      // Zusammenbau der Zeile
      //
      String retval = ''
      retval += '<tr>'

      if ((param.buttonorientation?:'right') == 'left') retval += gridRowButtons(param)
      
      // Wenn Values bergeben aus diesen die Felder auslesen
      //
      if (param.values != null) {
         param.values.each {val->
            String value = val
            String tdStyle = ''
            String title = ''

            if (val instanceof List) {
               value = val[0]?:''
               title = ((val[1]?:'') != '') ? """ title="${val[1]}" """ : ''
               tdStyle = ((val[2]?:'') != '') ? """ style="${val[2]}" """ : ''
               value = ((val[3]?:'') != '') ? """<span style="${val[3]}">${value}</span>""" : value
               }
            retval += """<td class="${tdclass}" ${tdStyle} ${title}>${value}</td>\n"""
            }
         }


      // Wenn Fielname definiert
      //
      hiddenFields = ''
      if (param.fieldnames != null) {
         param.fieldnames.each {name->
            try {
               hiddenFields += """<td class="${tdclass}">${param.domain.getProperty(name)?:''}</td>\n"""
               }
            catch(Exception e) {
               throw new Exception ("taglib::gridRow: bei fieldlist wurde eine ungueltiger Feldname uebergeben '${name}'")
               }
            }
         retval += hiddenFields
         }
  
      // Wenn Hiddenfields aus Domain definiert
      //
      if (param.hidden != null) {
         param.hidden.each {name->
            hiddenFields += """               <input type="hidden" name="${name}" value="${param.domain.getProperty(name)}">\n"""
            }
         }

 
      // Wenn Zusaetzlich Hiddenfields 
      // definiert sind
      if (param.addhidden != null) {
         param.addhidden.each {ah_fld->
            List ahValue = ah_fld.split('=')
            // nur wenn Inhalt
            if(ahValue[1] != null)
               hiddenFields +=  """              <input type="hidden" name="${ahValue[0]}" value="${(ahValue[1..ahValue.size()-1]).join('=')}">\n"""
            }
         }
      if ((param.buttonorientation?:'right') == 'right') retval += gridRowButtons(param)
      retval += '</tr>'

      return retval
      }

  /**
   * Liefert fuer Eine Double eine
   * Dezimaldarstellung.
   * 
   * @param 'zahl'      Double
   * @param 'format'    Formatstring (Vorgabewert '#,##0.00')
   * 
   * HINT:
   *  Wird der Parameter zahl als String bergeben, so wird 
   *  dieser unverndert zurckgeliefert.
   * 
   */
   String formatDecimal(Map param) {

      if (param == null) param = [:]
      if (param.zahl instanceof java.lang.String) {
         return param.zahl
         }

      String format = param.format?:'#,##0.00'
      String retval = new DecimalFormat(format).format((param.zahl)?:0.0)
      
      if (param.width?:0 != 0) {
          retval = (retval.padLeft(param.width)).getAt(0..(param.width-1))
          if (param.space?:'' != '') {
             retval = (retval =~ /\s/).replaceAll(param.space)
             }
          else {
             retval = (retval =~ /\s/).replaceAll('&nbsp;')
             }
         }
      return retval
      }

  /**
   * Gibt eine HTML colgroup anweisung aus.
   * Dabei werden alle Widht groessen mit 1% und die lezte mit 99 angegeben
   * 
   * @param cols       Anzahl der Spalten
   * 
   * 
   */
   String colgroup(Integer cols) {
      String result = '<colgroup>'
      result += '<col width="1%">' * (cols-1)
      result += '<col width="99%"></colgroup>'
      return result
      }


  /**
   * Gibt ein Datum als ANSI-Datum CCYY-MM-DD HH:NN zurck
   *
   * @param: Datum
   * @return: String mit Ansidatum
   *
   */
   String giveAsASCIIDateTime (datum) {
      SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd kk:mm", Locale.GERMAN);
      try {
         return df.format(datum)
         }
      catch (Exception e){
         return null
         }
      }

  /**
   * Gibt ein Datum als ANSI-Datum CCYY-MM-DDzurck
   *
   * @param: Datum
   * @return: String mit Ansidatum
   *
   */
   String giveAsASCIIDate (datum) {
      SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd", Locale.GERMAN);
      try {
         return df.format(datum)
         }
      catch (Exception e){
         return null
         }
      }
   
  /**
   * gibt einen Form-Tag aus
   *
   * @param "name" Wenn nicht bergeben, dann wird 'edit' angenommen
   *
   */
   String form(Map param) {
      if (param == null) param = [:]
      String formname = param.get('name')?:'edit'
      return """<form name="${formname}" method="${httpMethod}" action="start.groovy" accept-charset="${formAceptCharset}"> """
      }

  /**
   * Gibt enform aus
   *
   */
   String endform() {
      return '</form>'
      }


  /**
   * Ausgabe eines Hiddenfields
   * 
   * @param "name"     Name des Feldes
   * @param "value"    Inhalt des Feldes
   *
   */
   String hidden(Map param) {
      if (param == null) param = [:]
      assert param['name'] != null  : 'Taglib::hidden: Die Option "name" fehlt'
      assert param['value'] != null : 'Taglib::hidden: Die Option "value" fehlt'
      return """<input type="hidden" name="${param.name}" value="${param.value}"> """
      }
  /**
   * Gibt einen HTML Table tag zurueck
   * 
   * @param "cellpadding"
   * @param "cellspacing"
   * @param "width"           Vorgabewert 100%
   * @param "name"            Vorgabewert "grid"
   */
   String table(Map param) {
      if (param == null) param = [:]
      return """<table name="${param.get('name','grid')} cellpadding="${param.get('cellpadding','1px')}" cellspacing="${param.get('cellspacing','1px')}" width="${param.get('width','100%')}">"""
      }

  /**
   * Gibt einen HTML End Table Tag zurueck
   * 
   */
   String endtable() {
      return '</table>'
      }

  /**
   * Truncate display
   * 
   * Erzeugt ein abgeschnittenes Anzeigefeld
   * Es wird die maximale Lnge bergeben. Ist das Feld laenger,
   * wird auf die Angegebene Laenge gekuertz und drei Punkte angefuuml;gt.
   * 
   * @param value    Anzeigetext
   * @param length   Lngenangabe
   */
   String truncate(String value, Integer length) {

      if (value == null) return null
      String trunc = value
      if (trunc.size() > length) {
         trunc = ((trunc?:'-').padRight(length)).getAt(0..(length-2))+'	&hellip;'
         }
      return trunc
      }

  /**
   *
   * Grid Editierfunktion
   * Diese Funktion stellt eine Kombination von List und Editierfeldern dar.
   * 
   * Diese Routine wird fuer jede Zeile aufgerufnen.
   * Die bergabeparameter fuer Anzeigezeilen entsprechen den der Routine "gridRow"
   * 
   * Wenn die Editierschaltflaeche betaetigt wurde, wird in der Zeile, welche zum veraendern
   * angeklickt wurde der Bearbeitungsbereich angeboten. Eine Solche Zeile wird
   * durch den Parameter showeditmask == true erkannt.
   * 
   * Die Editiermaske wird durch eine Datenstruktur beschrieben. Die Datenstruktur hat folgendes Format
   * 
   * editmask ::= 'hiddenfields' ':' [ html-hidden-field, ...] ',' 'fields' ':' [html-edit-fields, ...]
   * 
   */
   String gridEdit(Map param) {
      if (param == null) param = [:]

      String retval = ''
      if (param.get('showeditmask')) {
         retval = this.form(name:'edit')+
            param.get('editmask').hiddenfields.join("\n")+
            '<tr><td>'+
            param.get('editmask').fields.join('</td><td>')+
            '</td></tr></form>'
         return retval
         }

      return this.gridRow(param)
      }

   
   // ---------- END OF CLASS ------------------------------------------
   }

