// ////////////// Sendet Binaeres File an reponse.output /////////////////////////
//
//
// -- wann --+-wer-+-- was ----------------------------------------------
// 03.01.2008| ny  | erstellt
//           |     | 
//           |     | 
//           |     | 
//           |     | 
//
// Autor: W. Nagy
//

package helper


/**
 * Sendet ein binaeres File an den Responseoutputstream
 * 
 * @author: W.Nagy
 * 
 * Beim initialisieren muss das Response Objekt mitgegeben werden.
 * Der Filename kann entweder beim Konstruktor oder bei der Methode 
 * angegeben werden.
 * 
 * Die Pfadtrenner "\" werden von der Methode normalisiert nach "/"
 * 
 * Beispiel:
 *   SendFile sender = new SendFile(response:response)
 *   sender.bout (
 *      sendfilepath:xlsFilename,
 *      filename:'kfzauslastung.xls',
 *      contenttype:'application/vnd.ms-excel'
 *      )
 * 
 */
class SendFile {
   
   String sendfilepath     // Dateiname mit Pfad
   def response = null     // Response Objekt

   /**
    * Binaerer Output auf Response
    * 
    * @param   "sendfilepath" Benannter Parameter, Dateiname, wenn nicht beim Konstruktor angegeben
    * @param   "outputfilename" Name, welcher vom Downloadfenster vorgeschlagen wird.
    * @param   "contenttype"  Content-Type, default "application/octet-stream"
    * 
    */
   // --- Sende Datein ------------------------------------------------
   void bout(Map param) {
   // -----------------------------------------------------------------
      // 
      // 
      // 
      if (param == null) param = [:]

      String outputfilename = param.get('filename','datei.bin')
      String contenttype    = param.get('contenttype','application/octet-stream')

      this.sendfilepath = param.get('sendfilepath',this.sendfilepath)

      assert sendfilepath != null : "SendFile out: Nicht mit Dateiname initialisiert"
      assert response != null : "SendFile out: Nicht mit Responseobjekt initialisiert"

      // Dateipfade normalisieren
      //
      sendfilepath = (sendfilepath =~ /\/\//).replaceAll('/')
      sendfilepath = (sendfilepath =~ /[\\]/).replaceAll('/')


      // Datei 
      def file = new File(sendfilepath)

      // Initialisieren des Outputstreams
      //
      def out = response.getOutputStream();

      // Response zuruecksetzen
      response.reset();

      // Header setzen
//      response.setHeader("Content-Type", "application/vnd.ms-excel")
      response.setHeader("Content-Type", contenttype)
      response.setHeader("Content-Disposition", """attachment;filename="${outputfilename}" """)
      response.setHeader("Content-Length", file.size().toString()) 

      // Handler holen
      out = response.getOutputStream()

      // Sende File 
      out.write(file.readBytes())

      // aufraeumen
      file.close()
      out.flush()
      out.close()

      }

   } // End of class