rem = '''
@echo on
cmd /k groovy.bat build.bat
goto exit
rem ********** START GROOVY ******************    '''  

/**
 * 
 * Erzeugen einer Distribution.
 * 
 * Die Groovydatein werden compeliert und zu einem WAR-File gepackt.
 * 
 * Diese Datei ist sowohl als Windwos BAT als auch als Groovy-Datei Aufrufbar.
 * Dies ist durch die Verschachtelung von REM-Batzeilen und Groovy-Kommentaren mglich.
 * Das Batfile ruft sich selbst als Groovy Skript auf.
 * 
 * HINT:
 *    In der Datei start.groovy wird die Buildnummer veraendert.
 * 
 * Autor: W. Nagy
 * 
 */

// DATA DIVSION
//
def ant = new AntBuilder()

// CONFIGURATIONS-SECTION
//
def WAR_Filename    = 'mframe.war'

def webinfDir       = "WEB-INF"
def srcDir          = "${webinfDir}/classes/"
def libDir          = "${webinfDir}/lib"

def targetRoot      = '../../distribution/mframe'
def targetDir       = "${targetRoot}/WEB-INF"
def targetLibDir    = "${targetDir}/lib"
def targetClassDir  = "${targetDir}/classes"

def gapiDir         = "gapidoc"
def gapeFramework   = "gapiFrameworkDok"

static addDoc = false

// PROCEDRUE DIVISON
//

// ---------------------------------------------------------------------
ant.sequential {
// ---------------------------------------------------------------------

    echo "delete Distribute workdirectory"
    delete(dir: targetRoot)

    echo "Creating the output directories"
    delete(dir: targetDir)
    mkdir(dir: new File(targetClassDir))
    mkdir(dir: new File(libDir))

    echo "Defining the classpath"
    path(id: "path") {
       fileset(dir: libDir) {
          include(name: "**/*.jar")
          }
       pathelement(location: targetClassDir)
       }

    echo "Defining groovyc task"

    taskdef(name: "groovyc", classname: "org.codehaus.groovy.ant.Groovyc", classpathref: "path")
    taskdef(name: "groovydoc", classname: "org.codehaus.groovy.ant.Groovydoc", classpathref: "path")

    echo "Erhoehe Build ID Bundle"

    // Automatisches erhhen
    // der Buildnummer des start.groovy Files
    // HINT:
    //      in start.groovy wird der Applikation die Version und Buildnummer mitgeteilt
    //      dies geschieht beim Konstruktoraufruf der Applikationsobjekts
    //      mit dem benannten Parameter applikationsversion.
    //      Das Programm geht davon aus, dass die Versionnummer mit Version einem Trennerbuchstaben
    //      (E/B) und einer dreisteligen Buildnummer versehen ist. Beispiel 1.1B003. Die dreistellige
    //      Buildnummer wird um eins erhoeht.
    //      1.1B003 --> 1.1B004
    //
    File f = new File("start.groovy")

    List filebuffer = []
    String newVersion = ''
    f.eachLine{line ->
       if (line =~ /^\s*applicationversion\s*:/) {
          // Herauslsen des Versionsstrings
          String ver = (line =~ /['"](.*?)['"]/)[0][1]    //"
          String sBuild = ver[ver.size()-3..ver.size()-1]
          Integer build = sBuild.toInteger()
          build++
          newVersion = ver[0..ver.size()-4]+sprintf('%03d',build)
          line = (line =~ /:(.*?)$/).replaceFirst(':\''+newVersion+'\',')
          }
       filebuffer.add(line)
       }
   
    echo "NEUE Versionsnummer Applikation:  $newVersion"

    echo "Erhoehe Build ID"

    // Automatisches erhhen
    // der Buildnummer des mframe VERSIONSFILES Files
    //
    f = new File("./WEB-INF/classes/framework/helper/Version.groovy")

    filebuffer = []
    newVersion = ''
    f.eachLine{line ->
       if (line =~ /^\s*static\s+String\s+VERSION\s*=/) {
          // Herauslsen des Versionsstrings
          String ver = (line =~ /['"](.*?)['"]/)[0][1]    //"
          String sBuild = ver[ver.size()-3..ver.size()-1]
          Integer build = sBuild.toInteger()
          build++
          newVersion = ver[0..ver.size()-4]+sprintf('%03d',build)
          line = (line =~ /=(.*?)$/).replaceFirst('=\''+newVersion+'\'')
          }
       filebuffer.add(line)
       }


    File o = new File('./WEB-INF/classes/framework/helper/Version.groovy')
    o.write(filebuffer.join("\n"))

    echo "*\n*NEUE Versionsnummer framework:  $newVersion\n*"

    echo "Compiling classes"
    groovyc(srcdir: srcDir, 
            destdir: targetClassDir, 
            classpathref: "path")

    echo "Copy start.groovy to ${targetDir}/start.groovy"
    ant.copy(file:'start.groovy', tofile:"${targetRoot}/start.groovy",overwrite: true)

    echo "copy LIBs "
    copy(todir:targetLibDir,overwrite: true) {
       fileset(dir:libDir) {
          include(name:"**/*.jar")
          }
       }

/** BUILD mframe JAR */
    echo "*\n* build framework jar\n*"
    ant.jar(
        destfile: "${targetLibDir}/framework-${newVersion}.jar", 
        basedir: "${targetClassDir}"
        )
/**/        


    echo "copy GROOVY-Files "
    copy(todir:targetClassDir,overwrite: true) {
       fileset(dir:srcDir) {
          include(name:"**/*.groovy")
          }
       }

/**   BUILD mfram JAR */
    echo "*\n* delete mframe directory:   ${targetDir}/classes/framework\n*"
    delete(dir: "${targetDir}/classes/framework")
/**/

    // 
    // Augenscheilich entscheidet der Groovy-Compiler ob er die class files vor 
    // compelieren soll, wenn die Class-Files aelter und nicht gleichalt oder aelter sind.
    // Die Groovy-files werden in die Vergangenheit (0:00 Uhr des aktuellen Tages gesetzt
    // damit sie auf jeden Fall 'aelter' sind.
    //
    echo "setze Datum der *.groovy files in die Vergangenheit"
    Date now = new Date()
    String targetDate = (now.getMonth()+1)+'/'+now.getDate()+'/'+(now.getYear()+1900)+' 00:00 am'

    ant.touch(datetime:targetDate) {
       fileset(dir:targetClassDir) {
          include(name:'**/*.groovy')
          }
       }

    echo "copy Templates "
    copy(todir:targetClassDir,overwrite: true) {
       fileset(dir:srcDir) {
          include(name:"**/*.tpl")
          }
       }

    echo "copy some stuff "

    // Bild und Icon Datein
    copy(todir:"${targetRoot}/images",overwrite: true) {
       fileset(dir:'./images') {
          include(name:"**/*.*")
          }
       }

    // DATENBANK
    copy(todir:"${targetDir}/database",overwrite: true) {
       fileset(dir:'./WEB-INF/database') {
          include(name:"**/*.*")
          }
       }

    
    
    // CSS Datein
    copy(todir:"${targetRoot}/css",overwrite: true) {
       fileset(dir:'./css') {
          include(name:"**/*.*")
          }
       }
    // Main und Logon GSP
    //
    copy(todir:targetDir,overwrite: true) {
       fileset(dir:webinfDir) {
          include(name:"*.gsp")
          }
       }
    
    // Hauptkonfigurationsdatei
    //
    copy(todir:targetDir,overwrite: true) {
       fileset(dir:webinfDir) {
          include(name:"*.xml")
          }
       }

    // ////////// Applikationsspezifische Datein Verwalten
    //

    // Erzeugen Logfileverzeichnisse
    echo "erzeuge logdirectory '$targetDir/log'"
    mkdir(dir: new File("${targetDir}/log"))

    def createlog = new File("${webinfDir}/log/create.log")
    createlog.write('Build erzeugt am: '+new Date()+"\nVERSION: ${newVersion}\n")
    echo "ezeuge globales Logfile"

    copy(todir:"${targetDir}/log",overwrite: true) {
       fileset(dir:"$webinfDir/log") {
          include(name:"create.log")
          }
       }

    echo "erzeuge temp-dir '$targetDir'"
    mkdir(dir: new File("${targetDir}/work"))
    try {
       def createtemp = new File("${webinfDir}/temp/temp.tmp")
       createtemp.write('Kann gelscht werden: '+new Date())
       }
    catch (Exception e) {
       println "*********** ${e.message} *******"
       }
    copy(todir:"${targetDir}/temp",overwrite: true) {
       fileset(dir:"$webinfDir/temp") {
          include(name:"temp.tmp")
          }
       }

    if (addDoc) {
       echo "Doc"
       groovydoc(sourcepath: srcDir, 
                 destdir: gapiDir, 
                 packagenames:"**.*",
                 use:"true",
                 windowtitle:"Groovydoc",
                 'private':"false"
                 )

       echo "FramworkDoc"
       groovydoc(sourcepath: "${srcDir}/framework", 
                 destdir: gapeFramework, 
                 packagenames:"**.*",
                 use:"true",
                 windowtitle:"Frameworkdoc",
                 'private':"false"
                 )
       
       }

    // 
    // ////////////////////////////////////////



    // Den ganzen Kramuri zusammenpacken
    // und versandfertig machen
    //
    ant.zip ( destfile : "${targetRoot}/../${WAR_Filename}" ) {
       zipfileset ( dir :  targetRoot, includes : '**/*.*')
       }
    echo "*"
    echo "*"
    echo "Das WAR File wurde in "
    echo " <jettyroot>/distributionen als ${WAR_Filename} hinterlegt"
    echo "*"
    echo "*"
    }
rem = ''' ************* ENDE GROOVY ***************
:exit
rem '''
